/*!
 ***********************************************************************
 \par Copyright
 \verbatim
  ________  _____           _____   _____           ____  ____   ____		
     /     /       /       /       /       /     /   /    /   \ /			
    /     /___    /       /___    /       /____ /   /    /____/ \___			
   /     /       /       /       /       /     /   /    /           \		
  /     /_____  /_____  /_____  /_____  /     / _ /_  _/_      _____/ 		
   																				
  Copyright (c) 2009 Telechips Inc.
  Korad Bldg, 1000-12 Daechi-dong, Kangnam-Ku, Seoul, Korea					
 \endverbatim
 ***********************************************************************
 */
/*!
 ***********************************************************************
 *
 * \file
 *		TCC_WMV78_DEC.h
 * \date
 *		2009/07/20
 * \author
 *		Seock-Hoon Woo(AValgorithm@telechips.com) 
 * \brief
 *		WMV78 decoder project main api
 * \version
 *		1.0 : 2009/07/20
 *
 ***********************************************************************
 */
#ifndef _WMV78_DECODER_H_
#define _WMV78_DECODER_H_

#ifdef INCLUDE_WMV78_DEC

#ifndef _STDINT_H
//#include "inttypes.h"
#endif

// Op Code 
#define WMVD_INIT		0	//!< initialize
#define WMVD_CLOSE		4	//!< close
#define WMVD_DECODE		2	//!< decode


/************************************************************************

	Common defines

************************************************************************/

typedef int WMV78_HANDLE;
typedef int WMV78_result_t;

//! common callback function
typedef struct __tagWMV78_FILE_FUNCS
{
	void* (*m_pMalloc		 ) ( unsigned int );					//!< malloc
	void  (*m_pFree			 ) ( void* );							//!< free
	void* (*m_pMemcpy		 ) ( void*, const void*, unsigned int );//!< memcpy
	void  (*m_pMemset		 ) ( void*, int, unsigned int );		//!< memset
	void* (*m_pRealloc		 ) ( void*, unsigned int );				//!< realloc
	void* (*m_pMemmove		 ) ( void*, const void*, unsigned int );//!< memmove
} WMV78_FILE_FUNCS;

typedef struct tagYUV420Frame_WMV
{
	unsigned char* m_pucYPlane;
	unsigned char* m_pucUPlane;
	unsigned char* m_pucVPlane;
	int m_Reserved[32-3];
} tYUV420Frame_WMV;

//! init parameter of TCC_WMV78_DEC()
typedef struct __tagWMV78_INIT
{
	WMV78_FILE_FUNCS m_sCallbackFunc;//!< [in] callback functions of libasf
	int m_iWidth;
	int m_iHeight;
	int m_iFourCC;
	int m_iExtraDataLen;
	unsigned char* m_pExtraData;
	unsigned char* m_pHeapAddress;
	unsigned char* m_pHuff_tbl_address;
	tYUV420Frame_WMV* m_pCurFrameAddress;
	tYUV420Frame_WMV* m_pRef0FrameAddress;
//	tYUV420Frame_WMV* Ref1FrameAddress;
	int m_Reserved[32-10];
} WMV78_INIT;

//! input parameter
typedef struct __tagWMV78_INPUT
{
	unsigned char* m_pPacketBuff;		//!< [in] allocated packet(video or audio) buffer pointer
	int m_iPacketBuffSize;				//!< [in] allocated packet(video or audio) buffer size
	int	m_Reserved[32-2];
} WMV78_INPUT;

//! output parameter
typedef struct __tagWMV78_OUTPUT
{
	tYUV420Frame_WMV* m_pDecodedData;
	int m_iPictureType;				//!< packet type of output data
	int m_iDecodingStatus;
	int m_Reserved[32-3];			//!< reserved...
} WMV78_OUTPUT;

/*!
***********************************************************************
* \brief
*		TCC_WMV78_DEC		: main api function of WMV78 decoder
* \param
*		[in]Op			: decoder operation 
* \param
*		[in,out]pHandle	: handle of WMV78 decoder
* \param
*		[in]pParam1		: init or input parameter
* \param
*		[in]pParam2		: output or information parameter
* \return
*		If successful, TCC_WMV78_DEC returns 0 or plus. Otherwise, it returns a minus value.
***********************************************************************
*/
 
WMV78_result_t TCC_WMV78_DEC(int opCode, WMV78_HANDLE *handle, void *param1, void *param2);

#endif	// #ifdef INCLUDE_WMV78_DEC
#endif	//_WMV78_DEMUXER_H_
