/*!
 ***********************************************************************
 \par Copyright
 \verbatim
  ________  _____           _____   _____           ____  ____   ____		
     /     /       /       /       /       /     /   /    /   \ /			
    /     /___    /       /___    /       /____ /   /    /____/ \___			
   /     /       /       /       /       /     /   /    /           \		
  /     /_____  /_____  /_____  /_____  /     / _ /_  _/_      _____/ 		
   																				
  Copyright (c) 2009 Telechips Inc.
  Korad Bldg, 1000-12 Daechi-dong, Kangnam-Ku, Seoul, Korea					
 \endverbatim
 ***********************************************************************
 */
/*!
 ***********************************************************************
 *
 * \file
 *		cdk_core.h
 * \date
 *		2009/05/29
 * \author
 *		AV algorithm group(AValgorithm@telechips.com) 
 * \brief
 *		cdk library core header
 * \version
 *		0.0.1 : 2009/05/29
 *
 ***********************************************************************
 */
#ifndef _CDK_CORE_H_
#define _CDK_CORE_H_


#include "main.h"
#include "cdk_pre_define.h"
#include "cdk_user_debug_level.h"
#include "cdk.h"
#include "cdk_sys.h"
//! Container format
#include "cdmx.h"
#include "cmux.h"
//! Video codec
#include "vdec.h"
#include "venc.h"
//! Audio codec
#include "adec.h"
#include "aenc.h"


typedef struct cdk_t
{
	int CDK_SIZE[1024]; //! 1KBytes
	cdk_core_t m_sCore;
} cdk_t;

#define CVDEC_DISP_INFO_INIT	0
#define CVDEC_DISP_INFO_UPDATE	1
#define CVDEC_DISP_INFO_GET		2
#define CVDEC_DISP_INFO_RESET	3
#if 0
typedef struct dec_disp_info_ctrl_t {
	int		m_iTimeStampType;	//! TS(Timestamp) type (0: Presentation TS(default), 1:Decode TS)
	int		m_iStdType;			//! STD type
	int		m_iFmtType;			//! Formater Type

	int		m_iUsedIdxPTS;		//! total number of decoded index for PTS
	int		m_iRegIdxPTS[32];	//! decoded index for PTS
	void	*m_pRegInfoPTS[32];	//! side information of the decoded index for PTS

	int		m_iDecodeIdxDTS;	//! stored DTS index of decoded frame
	int		m_iDispIdxDTS;		//! display DTS index of DTS array
	int		m_iDTS[32];			//! Decode Timestamp (decoding order)
	
	int		m_Reserved;
} dec_disp_info_ctrl_t;
#endif
#if 0
typedef struct dec_disp_info_t {
	int m_iFrameType;			//! Frame Type

	int m_iTimeStamp;			//! Time Stamp
	int m_iRvTimeStamp;			//! TR(RV)

	int m_iPicStructure;		//! PictureStructure
	int m_iM2vFieldSequence;	//! Field sequence(MPEG2) 
	int m_iFrameDuration;		//! MPEG2 Frame Duration

	int m_iFrameSize;			//! Frame size
} dec_disp_info_t;
#endif
#if 0
typedef struct dec_disp_info_input_t {
	int m_iFrameIdx;			//! Display frame buffer index for CVDEC_DISP_INFO_UPDATE command
								//! Decoded frame buffer index for CVDEC_DISP_INFO_GET command
	int m_iStdType;				//! STD type for CVDEC_DISP_INFO_INIT
	int m_iTimeStampType;		//! TS(Timestamp) type (0: Presentation TS(default), 1:Decode TS) for CVDEC_DISP_INFO_INIT
	int m_iFmtType;			//! Formater Type specification
	int m_iFrameRate;
} dec_disp_info_input_t;
#endif



extern void get_ext_name( char* pPathName, char* pExtName );
extern int get_input_container_type( char* pFileName );
extern int get_video_codec_type( int iFourCC );
#if 1
extern int get_audio_codec_type( int iFourId );
#else
extern int get_audio_codec_type( cdk_func_t* pstCDmx, int iFourId, cdmx_info_t *pstCdmxInfo, int iDmxType);
#endif
extern int identify_ac3_stream_sync();


cdk_core_t* TCC_CDK_GetCoreAddr(void);
cdk_func_t* TCC_CDK_GetDMXFunc(unsigned int container_type);
cdk_func_t* TCC_CDK_GetDECFunc(unsigned int video_type);
#ifdef INCLUDE_VOB_DMX
cdk_func_t* TCC_CDK_Get_VOBDMXFunc(void);
#endif

unsigned int TCC_CDK_GetRegBaseAddr(void);
unsigned int TCC_CDK_GetRegBaseSize(void);
unsigned int TCC_CDK_GetBitstreamBufAddr(unsigned int index);
unsigned int TCC_CDK_GetBitstreamBufSize(void);

void TCC_CDK_SetRegBaseSize(void);
void TCC_CDK_SetRegBaseAddr(void);
void TCC_CDK_SetBitstreamBufSize(void);
void TCC_CDK_SetBitstreamBufAddr(void);

void TCC_CDK_FreeRegBaseAddr(void);
void TCC_CDK_FreeBitstreamBufAddr(void);


void TCC_CDK_SetRegBaseVideoBusAddr(void);
void TCC_CDK_FreeRegBaseVideoBusAddr(void);
int TCC_CDK_GetRegBaseVideoBusSize(void);

void TCC_CDK_SetUserDataBufAddr(void);
int TCC_CDK_GetUserDataBufAddr(unsigned int index);
int TCC_CDK_GetUserDataBufSize(void);
void TCC_CDK_FreeUserDataBuf(void);

int TCC_CDK_GetMpeg2LatesPTS(void);
void TCC_CDK_SetMpeg2LatesPTS(int value);
int TCC_CDK_GetMpeg2PTSInterval(void);
int TCC_CDK_GetMpeg2RemainDuration(void);
void TCC_CDK_SetMpeg2RemainDuration(int value);
#endif //_CDK_CORE_H_
