//#include "include/rv_depack.h"
//#include "../RV_DECODER/include/rv_decode.h"
/************************************************************************

	Operation Code / Return value

************************************************************************/
// Op Code 
#define RM_INIT		0	//!< initialize
#define RM_CLOSE		1	//!< close
#define RM_DECODE		2	//!< decode
#define RM_SEEK		3	//!< seek -------------------------------------for demuxer
#define RM_GET_STREAM	4	//!< get stream -------------------------------for demuxer
#define RM_PUT_STREAM	4	//!< put stream -------------------------------for muxer
#define RM_INIT_FROM_FILE_HANDLE	5	//!< initialize from file handle --for demuxer
#define RM_OPEN_FOR_MEDIASCAN	 7 //!< get media info quickly


// Return value
#define RM_ERR_NONE				 0
#define RM_ERR_END_OF_FILE			-1
#define RM_ERR_END_OF_VIDEO_FILE	-2
#define RM_ERR_END_OF_AUDIO_FILE	-3
#define RM_ERR_FILE_NOT_FOUND		-4
#define RM_ERR_FILE_OPEN			-5
#define RM_ERR_FILE_WRITE			-6
#define RM_ERR_FILE_SEEK			-7
#define RM_ERR_VIDEO_SEEK			-8
#define RM_ERR_AUDIO_SEEK			-9
#define RM_ERR_BS_IO_EOF			-10

#define	RM_ERR_PROTECTION_CODE			-100
#define RM_ERR_OUT_OF_MEMORY			-101
#define RM_ERR_INVALID_DATA			-102
#define RM_ERR_NOT_SUPPORTED			-103
#define RM_ERR_STREAM_DOES_NOT_EXIST	-104
#define RM_ERR_NEED_CHCAE_MEM			-105
#define RM_ERR_SEEK_NO_MORE_KEY_FRAME	-106
#define RM_ERR_REALLOC					-107
#define RM_ERR_MALLOC					-108

#define RM_ERR_BS_CACHE_SIZE			-109

/* video codec error */
#define RM_ERR_UNKNOWN_VIDEO_CODEC		-200
#define RM_ERR_H264_NAL_LENGTH			-201
#define RM_ERR_GET_AVC_INFO			-202
//mp4
//#define ERR_MP4_VIDEO_OBJECT_IDC	-1100

/*audio codec error */
#define RM_ERR_UNKNOWN_AUDIO_CODEC		-300

//mp4
//#define ERR_MP4_AUDIO_OBJECT_IDC	-2100

// for debug
#define RM_ERR_DBG_COMPARE				-9999

/************************************************************************

	Common defines

************************************************************************/
// packet(stream) type
#define RM_PACKET_NONE			0			//!< default packet
#define RM_PACKET_VIDEO		1			//!< video packet
#define RM_PACKET_AUDIO		2			//!< audio packet
#define RM_AUDIO_VIDEO			3			//!< dual packet
#define RM_PACKET_SUBTITLE		4			//!< reserved(not used)
#define RM_TICKSPERSEC			1000		//!< ticks/sec

#if 1
//! common callback function
typedef struct rm_common_callback_func_t
{
	void* (*m_pMalloc		 ) ( unsigned int );					//!< malloc
	void* (*m_pNonCacheMalloc) ( unsigned int );					//!< non-cacheable malloc 
	void  (*m_pFree			 ) ( void* );							//!< free
	void  (*m_pNonCacheFree	 ) ( void* );							//!< non-cacheable free
	void* (*m_pMemcpy		 ) ( void*, const void*, unsigned int );//!< memcpy
	void* (*m_pMemset		 ) ( void*, int, unsigned int );		//!< memset
	void* (*m_pRealloc		 ) ( void*, unsigned int );				//!< realloc
	void* (*m_pMemmove		 ) ( void*, const void*, unsigned int );//!< memmove

	void*		 (*m_pFopen	) (const char *, const char *);						//!< fopen
	unsigned int (*m_pFread	) (void*, unsigned int, unsigned int, void* );		//!< fread
	int			 (*m_pFseek	) (void*, long, int );								//!< fseek
	int			 (*m_pFseek64	) (void*, signed long long, int );						//!< fseek64
	long		 (*m_pFtell	) (void* );											//!< ftell
	signed long long		 (*m_pFtell64	) (void* );											//!< ftell
	unsigned int (*m_pFwrite) (const void*, unsigned int, unsigned int, void*);	//!< fwrite
	int			 (*m_pFclose) (void *);											//!< fclose
	int			 (*m_pUnlink) ( const char *);									//!< _unlink
} rm_common_callback_func_t;
#endif


//! common demuxer information
typedef struct rm_common_dmx_info_t
{
	//! file information : 128 Bytes
	struct 
	{
		char* m_pszOpenFileName;	//!< 1.open file name
		char* m_pszCopyright;		//!< 2.copyright
		char* m_pszCreationTime;	//!< 3.creation time
		int m_iFileSize;			//!< 4.total file size
		int m_iRunningtime;			//!< 5.runing time * 1000
		int m_iTimeScale;			//!< 6.timescale of file
		int m_iUserDataPos;			//!< 7.user data position
		int m_iUserDataLen;			//!< 8.user data length
		int m_iIndex;
		int m_ilast_key_frame_time;
		int m_iKey_frame_num;
		int m_Reserved[32-11];
	} m_sFileInfo;


	//! video information : 128 Bytes
	struct 
	{
				/* common */
		int m_iWidth;				//!< width
		int m_iHeight;				//!< height
		int m_iFrameRate;			//!< framerate * 1000;
		int m_iFourCC;				//!< fourcc

		/* extra info (common) */
		char* m_pszCodecName;		//!< codec name
		char* m_pszCodecVendorName;	//!< codec vendor
		unsigned char* m_pExtraData;//!< extra data
		int m_iExtraDataLen;		//!< extra data length

		/* RM info */
		///////rv_video/////////	
		unsigned int   m_ulLength;
		unsigned int   m_ulMOFTag;
		unsigned int   m_ulSubMOFTag;
		unsigned short int m_usBitCount;
		unsigned short int   m_usPadWidth;
		unsigned short int   m_usPadHeight;
		unsigned short int   m_usReserved;
		unsigned int m_ulFramesPerSecond;
		unsigned int   m_ulOpaqueDataSize;
		char*    m_pOpaqueData;
		unsigned int m_ulmax_bit_rate;        //!< maximum bit rate of stream 
		unsigned int m_ulavg_bit_rate;        //!< average bit rate of stream 
		int m_iStreamNum;
		///////rv_video/////////	

		int m_Reserved[32-18];
	} m_sVideoInfo;

	//! audio information : 128 Bytes
	struct 
	{
		/* common */
		int m_iTotalNumber;			//!< total audio number
		int m_iSamplePerSec;		//!< samples/sec
		int m_iBitsPerSample;		//!< bits/sample
		int m_iChannels;			//!< channels
		int m_iFormatId;			//!< format id

		/* extra info (common) */
		char* m_pszCodecName;		//!< codec name
		char* m_pszCodecVendorName;	//!< codec vendor
		unsigned char* m_pExtraData;//!< extra data
		int m_iExtraDataLen;		//!< extra data length

		int m_iActualRate;
		short m_sAudioQuality;
		short m_sFlavorIndex;
		int m_iBitsPerFrame;
		int m_iGranularity;
		int m_iOpaqueDataSize;
		char*  m_pOpaqueData;

		int m_iSamplesPerFrame;
		int m_iframeSizeInBits;
		int m_iSampleRate;
		short m_scplStart;
		short m_scplQBits;
		short m_snRegions;
		short m_sdummy;

		unsigned int m_ulmax_bit_rate;        //!< maximum bit rate of stream 
		unsigned int m_ulavg_bit_rate;        //!< average bit rate of stream 

		int m_iStreamNum;
		int m_Reserved[32-22];
	} m_sAudioInfo;

} rm_common_dmx_info_t;
