/*!
 ***********************************************************************
 \par Copyright
 \verbatim
  ________  _____           _____   _____           ____  ____   ____		
     /     /       /       /       /       /     /   /    /   \ /			
    /     /___    /       /___    /       /____ /   /    /____/ \___			
   /     /       /       /       /       /     /   /    /           \		
  /     /_____  /_____  /_____  /_____  /     / _ /_  _/_      _____/ 		
   																				
  Copyright (c) 2008-2009 Telechips Inc.
  Korad Bldg, 1000-12 Daechi-dong, Kangnam-Ku, Seoul, Korea					
 \endverbatim
 ***********************************************************************
 */
/*!
 ***********************************************************************
 *
 * \file
 *		tcc_wav_mux.h
 * \date
 *		2009/11/25
 * \author
 *		Jonathan-Kim(AValgorithm@telechips.com) 
 * \brief
 *		wav muxer for tcc_wav_enc
 * \version
 *		
 *
 ***********************************************************************
  */
#ifndef _TCC_WAV_MUXER_H_
#define _TCC_WAV_MUXER_H_

#include "common.h"

#define TCC_WAV_MAX_PCM_LENGTH				4096

#define WAV_LPCM_HEADER_SIZE				44
#define WAV_MSADPCM_HEADER_SIZE				90
#define WAV_A_MU_LAW_HEADER_SIZE			46

#pragma pack(1)

typedef struct lpcm_wav_header_t
{
	//"RIFF" chunk descriptor
	char			m_cChunkId[4];		//"RIFF"
	unsigned int	m_uiChunkSize;
	char			m_cFormat[4];		//"WAVE"
	
	//"fmt " sub-chunk
	char            m_cFmtChunkID[4];	//"fmt "
	unsigned int    m_uiFmtChunkSize;
	unsigned short  m_usAudioFormat;
	unsigned short  m_usNumChannels;
	unsigned int	m_uiSampleRate;
	unsigned int	m_uiByteRate;
	unsigned short  m_usBlockAlign;
    unsigned short  m_usBitsPerSample;

	//"data" sub-chunk
	char			m_cDataChunkId[4];	//"data"
	unsigned int	m_uiDataChunkSize;
	//data

} lpcm_wav_header_t;

typedef struct ms_adpcm_coefset_t
{
    short m_sCoef1;
    short m_sCoef2;
} ms_adpcm_coefset_t;

typedef struct ms_adpcm_wav_header_t
{
	//"RIFF" chunk descriptor
	char			m_cChunkId[4];		//"RIFF"
	unsigned int	m_uiChunkSize;
	char			m_cFormat[4];		//"WAVE"
	
	//"fmt " sub-chunk
	char            m_cFmtChunkID[4];	//"fmt "
	unsigned int    m_uiFmtChunkSize;

	unsigned short  m_usAudioFormat;
	unsigned short  m_usNumChannels;
	unsigned int	m_uiSampleRate;
	unsigned int	m_uiAvgBytesPerSec;
	unsigned short  m_usBlockAlign;
	unsigned short  m_usBitsPerSample;

	unsigned short  m_usCbSize;
	unsigned short  m_usSamplesPerBlock;
	unsigned short  m_usNumCoef;  //42
	
	ms_adpcm_coefset_t	m_sCoef[7]; //28

	//"fact" sub-chunk
	char            m_cFactChunkID[4];	//"fact"
	unsigned int    m_uiFactChunkSize;
	unsigned int    m_uiNumberOfSamples;

	//"data" sub-chunk
	char			m_cDataChunkId[4];	//"data"
	unsigned int	m_uiDataChunkSize;
	//data

} ms_adpcm_wav_header_t;

typedef struct mu_a_law_wav_header_t
{
	//"RIFF" chunk descriptor
	char			m_cChunkId[4];		//"RIFF"
	unsigned int	m_uiChunkSize;
	char			m_cFormat[4];		//"WAVE"
	
	//"fmt " sub-chunk
	char            m_cFmtChunkID[4];	//"fmt "
	unsigned int    m_uiFmtChunkSize;

	unsigned short  m_usAudioFormat;
	unsigned short  m_usNumChannels;
	unsigned int	m_uiSampleRate;
	unsigned int	m_uiAvgBytesPerSec;
	unsigned short  m_usBlockAlign;
	unsigned short  m_usBitsPerSample;
	unsigned short  m_usCbSize;

	//"data" sub-chunk
	char			m_cDataChunkId[4];	//"data"
	unsigned int	m_uiDataChunkSize;
	//data

} mu_a_law_wav_header_t;

#pragma pack()

typedef struct wav_enc_init_extra_info_t //tcc_wav_enc extrainfo
{
	unsigned int            m_uiWaveSamplesPerBlock;
	unsigned int            m_uiWaveBytesPerBlock;
	unsigned int			m_uiWaveByteRate;
} wav_enc_init_extra_info_t;

typedef struct wav_mux_info_t
{
	void					*m_pWavefile;

	void					*m_psWaveHdr;

	unsigned int			m_uiWaveFormatId;
	unsigned int			m_uiWaveSamplerate;
	unsigned int			m_uiWaveChannels;
	unsigned int			m_uiWaveBitsPerSample;
	unsigned int			m_uiWaveDataSize;
	unsigned int            m_uiWaveBytesPerBlock;
	unsigned int            m_uiWaveSamplesPerBlock;
	unsigned int			m_uiWaveByteRate;

	unsigned int			m_uiNumberOfSample;
	unsigned int			m_uiInitOk;
	//unsigned int			m_uiFrameCount;

	wav_enc_init_extra_info_t		m_sEncSetInfo;

} wav_mux_info_t;

/*!
 ***********************************************************************
 * \brief
 *		cmux_wav		: main api function of cmux_wav
 * \param
 *		[in]Op			: muxer operation 
 * \param
 *		[in,out]pHandle	: cmux_wav handle
 * \param
 *		[in]pParam1		: init or input parameter
 * \param
 *		[in]pParam2		: output or information parameter
 * \return
 *		If successful, cmux_wav returns 0 or plus. Otherwise, it returns a minus value.
 ***********************************************************************
 */
common_result_t 
cmux_wav( int Op, common_handle_t* pWavHandle, void* pParam1, void* pParam2 );

#endif//_TCC_WAV_MUXER_H_
