/*****************************************************************************
 * 
 * File Name: th263dec.h
 * Ver      : 1.26
 * 
 ****************************************************************************/
#ifndef _Th263_H_
#define _Th263_H_


/*****************************************************************************
 * error codes
 ****************************************************************************/

/*	all functions return values <0 indicate error */

#define Th263_ERR_FAIL			-1		/* general fault */
#define Th263_ERR_MEMORY		-200	/* memory allocation error */
#define Th263_ERR_FORMAT		-201	/* file format error */
#define Th263_ERR_NOT_SUPPORT	-202	/* file format not supported */
#define Th263_ERR_END			-203	/* encoder only; end of stream reached */


/* colorspace values */

#define Th263_CSP_I420		(1<< 1) /* 4:2:0 planar */

typedef struct {
	int csp;				/* [in] colorspace : Th263_CSP_I420 only*/
	void * plane[4];		/* [in] image plane ptrs */
	int stride[4];			/* [in] image stride; "bytes per row"*/
	int reserved;
} th263_image_t;


/* frame type flags */

#define Th263_TYPE_VOL     -1 /* decoder only: vol was decoded */
#define Th263_TYPE_NOTHING  0 /* decoder only (encoder stats): nothing was decoded/encoded */
#define Th263_TYPE_AUTO     0 /* encoder: automatically determine coding type */
#define Th263_TYPE_IVOP     1 /* intra frame */
#define Th263_TYPE_PVOP     2 /* predicted frame */
#define Th263_TYPE_SVOP     4 /* predicted+sprite frame */
#define Th263_TYPE_DVOP     6 /* disposable frame in Sorenson's H.263 */


/* codec types */

#define Th263_ITU_H263			1
#define Th263_SORENSON_H263		2

#define Th263_GBL_INIT			0		/* initialize th263core; must be called before using th263_decore */
#define Th263_GETIMAGESIZE		(1<<2)
#define Th263_SETIMAGEALIGN16	(1<<3)

#define Th263_DEC_CREATE  0 /* create decore instance; return 0 on success */
#define Th263_DEC_DESTROY 1 /* destroy decore instance: return 0 on success */
#define Th263_DEC_DECODE  2 /* decode a frame: returns number of bytes consumed >= 0 */


/*****************************************************************************
 * th263_global()
 ****************************************************************************/

extern int  th263_global(void *handle, int opt, void *param1, void *param2);
extern void th263_heap_init(unsigned char *heap, unsigned int heap_size);


/*****************************************************************************
 * th263_decore()
 ****************************************************************************/

extern int th263_decore( void *handle, int opt, void *param1, void *param2 );


/* Th263_DEC_CREATE param 1
	image width & height may be specified here when the dimensions are
	known in advance. */

typedef struct {
	int		width;     /* [in:opt] image width */
	int		height;    /* [in:opt] image width */

	void*	handle;		/* [out]	   decore context handle */

	// Callback Func
	void* (*m_Memcpy ) ( void*, const void*, unsigned int );	//!< memcpy
	void  (*m_Memset ) ( void*, int, unsigned int );			//!< memset

	int		reserved;
} th263_dec_create_t;


/* Th263_DEC_DECODE param1 */

typedef struct {
	int general;					/* [in] general flags(option)
						 				    Th263_GETIMAGESIZE : get width and height of stream, only for Header parsing 
									*/
	void *bitstream;				/* [in]     bitstream (read from)*/
	int length;						/* [in]     bitstream length */
	
	th263_image_t output;			/* [in]     output image (written to) */

	int reserved;
} th263_dec_frame_t;


/* Th263_DEC_DECODE param2 :: optional */

typedef struct
{
	int type;                   	/* [out] output data type */
	int first_frame_P_picture;		/* [out] indicate the picture coding type
											 (It is only useful when the picuter is the first frame of input video stream)
										     0: I-VOP 
										     1: Otherwise
									*/
	int inter_mb_in_P_check_flag;   /* [in] check whether or not the P-VOP has only intra coded MBs
											0: disable
											1: enable
									*/
	int inter_mb_in_P_flag;			/* [out] indicate whether or not the P-VOP has only intra coded MBs
											 (It is only useful when the inter_mb_in_P_check_flag is set.)
											 0: the P-VOP has only intra coded MBs
											 1: otherwise
								 	*/	
	union {
		struct { /* type>0 {Th263_TYPE_IVOP,Th263_TYPE_PVOP,Th263_TYPE_BVOP,Th263_TYPE_SVOP} */
			int general;        	/* [out] flags */
			int time_base;      	/* [out] time base */
			int time_increment; 	/* [out] time increment */
			int reserved;
		} vop;
		struct {	/* Th263_TYPE_VOL */
			int general;        	/* [out] flags */
			int width;          	/* [out] width : multiple of 2 or 16 */
			int height;         	/* [out] height : multiple of 2 or 16 */
			int par;            	/* [out] pixel aspect ratio (refer to Th263_PAR_xxx above) */
			int par_width;      	/* [out] aspect ratio width  [1..255] */
			int par_height;     	/* [out] aspect ratio height [1..255] */
			int crop_left_offset;	/* [out] Left offset for luma samples (V.114) */
			int crop_right_offset;	/* [out] Right offset for luma samples (V.114) */
			int crop_top_offset;	/* [out] Top offset for luma samples (V.114) */
			int crop_bottom_offset;	/* [out] Bottom offset for luma samples (V.114) */
		} vol;
	} data;

	int	used_bytes;				/* [out] Used bytes */

	int reserved[24-15];

} th263_dec_stats_t;


//////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * TCC_H263_DEC()
 ****************************************************************************/

#define H263_RETCODE_SUCCESS	0
//#define H263_RETCODE_INVALID_HANDLE 2
//#define H263_RETCODE_INVALID_COMMAND 4
//#define H263_RETCODE_NOT_INITIALIZED 13

// Decoder Op Code 
#define H263_DEC_INIT				0x00	//!< init
#define H263_DEC_SEQ_HEADER			0x01	//!< decode sequence header 
//#define H263_DEC_GET_INFO			0x02
#define H263_DEC_REG_FRAME_BUFFER	0x03	//!< register frame buffer
#define H263_DEC_DECODE				0x10	//!< decode
#define H263_DEC_CLOSE				0x20	//!< close


extern int TCC_H263_DEC( int OpCode, void *handle, void *param1, void *param2 );


typedef struct h263dec_image_t
{
	int m_iCsp;					//!< [in] colorspace : Th263_CSP_I420 only
	void * m_pPlane[4];			//!< [in] image plane ptrs
	int m_iStride[4];			//!< [in] image stride; "bytes per row"
	int m_Reserved;
} h263dec_image_t;


/* H263_DEC_INIT param 1
	heap mem. is used in the instance mem. of handle
*/

#define H263_DEC_HEAP_SIZE	(64*4+128)

typedef struct 
{
	void*	m_pHandle;			//!< [out] decoder context handle

	int		m_iHeapSize;		//!< [in] size of heap memory for handle
	char*	m_pHeapAddr;		//!< [in] address of Heap memory for handle
	int		m_iCodecType;		//!< [in] Codec type( 1: ITU H.263, 2: Sorenson H.263 )
	
	//! Callback Func
	void* (*m_Memcpy ) ( void*, const void*, unsigned int );	//!< [in] memcpy
	void  (*m_Memset ) ( void*, int, unsigned int );			//!< [in] memset

} h263dec_init_t;


/* Th263_DEC_DECODE param1 */

typedef struct h263dec_input_t 
{
	int m_iDecOption;				//!< [in] Decoding option

	void *m_pBitstreamStartAddr;	//!< [in] bitstream (read from)
	int m_iBitstreamSize;			//!< [in] bitstream length
	
	h263dec_image_t m_DispOout;		//!< [in] output image (written to)

	int m_Reserved;
} h263dec_input_t;


/* Th263_DEC_DECODE param2 :: optional */

typedef struct h263dec_stats_t
{
	int m_iDataType;					//!< [out] output data type
	int m_iFirstFramePframe;			//!< [out] indicate the picture coding type
										//!<		 (It is only useful when the picuter is the first frame of input video stream)
										//!<	     0: I-VOP 
										//!<	     1: Otherwise
	int m_iInterMBinP_CheckFlag;		//!< [in] check whether or not the P-VOP has only intra coded MBs
										//!<		0: disable
										//!<		1: enable
	int m_InterMBinP_Flag;				//!< [out] indicate whether or not the P-VOP has only intra coded MBs
										//!<		 (It is only useful when the inter_mb_in_P_check_flag is set.)
										//!<		 0: the P-VOP has only intra coded MBs
										//!<		 1: otherwise
	union 
	{
		struct							//!< type>0 {Th263_TYPE_IVOP,Th263_TYPE_PVOP,Th263_TYPE_BVOP,Th263_TYPE_SVOP} 
		{ 
			int m_iGeneral;        		//!< [out] flags
			int m_iTimeBase;      		//!< [out] time base
			int m_iTimeIncrement; 		//!< [out] time increment
			int m_Reserved;
		} m_sVop;

		struct							//!< Th263_TYPE_VOL
		{	
			int m_iGeneral;        		//!< [out] flags

			int m_iWidth;          		//!< [out] width : multiple of 2 or 16
			int m_iHeight;         		//!< [out] height : multiple of 2 or 16

			int m_iPixelAspectRatio;    //!< [out] pixel aspect ratio
			int m_iPar_Width;      		//!< [out] aspect ratio width  [1..255]
			int m_iPar_Height;     		//!< [out] aspect ratio height [1..255]

			int m_iCropOffset_Left;		//!< [out] Left offset for luma samples
			int m_iCropOffset_Right;	//!< [out] Right offset for luma samples
			int m_iCropOffset_Top;		//!< [out] Top offset for luma samples
			int m_iCropOffset_Bottom;	//!< [out] Bottom offset for luma samples
		} m_sVol;

	} m_uInfo;

	int	m_iUsedBytes;					//!< [out] Used bytes
	int m_iNeedFrameBufferSize;			//!< [out] the size of frame buffers that are required for decoding 
										//!<  	   (only when sequence header decoding) 

	int m_Reserved[24-16];

} h263dec_stats_t;


#endif //_Th263_H_
