#ifndef _AUDIO_SYSTEM_H_
#define _AUDIO_SYSTEM_H_
/*===========================================================================

           A U D I O   S Y S T E M   H E A D E R   F I L E

DESCRIPTION
  This header file contains all the definitions necessary to interface of Audio
  Codec API with the System layer

Copyright (c) 2008 by TELECHIPS, Incorporated.  All Rights Reserved.
===========================================================================*/


/*===========================================================================

                      EDIT HISTORY FOR FILE

This section contains comments describing changes made to this file.
Notice that changes are listed in reverse chronological order.
   
  $Header: location of this file 
   
when       who    	 what, where, why
--------   ---    	 ----------------------------------------------------------
08/03/13   Brian     Test version #2
08/02/19   Brian     Test version #1
===========================================================================*/


/*===========================================================================

                      INCLUDE FILES FOR MODULE

===========================================================================*/
//                           nothing                                       */

#ifndef bool
typedef enum
{
	false = 0,
	true  = 1
}bool;
#endif

typedef enum
{
	TCAS_FS_6000	= 6000,
	TCAS_FS_8000	= 8000,
	TCAS_FS_11025	= 11025,
	TCAS_FS_12000	= 12000,
	TCAS_FS_16000	= 16000,
	TCAS_FS_22050	= 22050,
	TCAS_FS_24000	= 24000,
	TCAS_FS_32000	= 32000,
	TCAS_FS_44100	= 44100,
	TCAS_FS_48000	= 48000,
	TCAS_FS_64000	= 64000,
	TCAS_FS_88200	= 88200,
	TCAS_FS_96000	= 96000,	
}TCAS_sample_freq;

typedef enum
{
	TCAS_PCMRES_16 = 1,
	TCAS_PCMRES_24 = 2
}TCAS_sample_res;

typedef enum
{
	TCAS_CH_UNKNOWN		= 0,
	TCAS_CH_MONO 			= 1,
	TCAS_CH_STEREO 		= 2,
	TCAS_CH_21		 		= 3,
	TCAS_CH_31		 		= 4,
	TCAS_CH_41		 		= 5,
	TCAS_CH_51		 		= 6,
	TCAS_CH_71		 		= 8	
}TCAS_chnum;


#if 1
enum	
	{	
		UNDETECTED_VAD =0,
		DETECTED_VAD 
	};

enum	
	{	
		VAD_DISABLE =0,
		VAD_ENABLE 
	};
#endif




typedef struct 
{	
	char 						*ptr;
	int							 used;
}TCAS_buffer_state;

typedef struct 
{	
	TCAS_sample_res	 	sample_res;
	TCAS_sample_freq 	sample_freq;
	TCAS_chnum 				ch_num;
	
	unsigned int			len	;
}TCAS_pcm_info;

typedef struct 
{		
	TCAS_pcm_info			in_pcm_info;
	TCAS_pcm_info			out_pcm_info;
	unsigned int			bit_rate;
	unsigned int			bIsVBR;
	unsigned int			firstFrameOfst;
	unsigned int			filelen;
	unsigned int			total_time;	
}TCAS_codec_info;

typedef struct 
{	
	bool		use_bandex;
	bool		use_chex;
	bool		use_internal_parser;			
}TCAS_decoder_open_info;

typedef struct 
{	
	unsigned int			bitrate;
	TCAS_pcm_info			pcminfo;
	void *	pEncode_format;
}TCAS_encoder_open_info;

typedef struct 
{	
	void    *fileid;
	TCAS_decoder_open_info dec_info;
	TCAS_encoder_open_info enc_info;	
}TCAS_codec_open_info;


// structure for CDSync
typedef struct _CDSync
{
	unsigned long	CDsynclevel;		// Check level	
	unsigned long	CDsynctime;			// Check time
	unsigned long 	CDsyncsilence_cnt;	// counter	
	unsigned long 	Gsum_sound;

	char	CDsync_enable;					// enable CDsync Block
	char	CDsync_detect;					// indicates the CDSync status  
									// '0' - non-sync status, '1' - sync status

	char	reset_flag;		
	char	fUseablewrite;			

} TCAS_CDSync_info;

//VAD
typedef struct VAD_Struct
{
	long	vad_data_length;	// frame sample length
	long	vad_channel_mode;	// stereo(1) or mono(0) 
	long	vad_level;			// reference level	
	long	vad_time;
	long	vad_silence_cnt;	// counter of the silence frame
	long	vad_detect;
	long	vad_enable;			// enable VAD Block
}TCAS_VAD_Struct_info ;


#define In
#define Out
#define InOut


typedef enum
{
	TCAS_SUCCESS								=0x0,	
	TCAS_ERROR_NULL_INSTANCE					=0x00000010,
	TCAS_ERROR_INVALID_OPINFO					=0x00000011,
	TCAS_ERROR_FILE_OPEN						=0x00000012,
	TCAS_ERROR_NOT_SUPPORT_FORMAT				=0x00000013,
	TCAS_ERROR_END_FILE							=0x00000014,
	TCAS_ERROR_INVALID_BUFSTATE					=0x00000015,
	TCAS_ERROR_INVALID_DATA						=0x00000016,
	TCAS_ERROR_NULL_PCMBUFF						=0x00000017,
	TCAS_ERROR_DECODE 							=0x00000018,
	TCAS_ERROR_ENCODE							=0x00000019,
	TCAS_ERROR_SEEK_FAIL						=0x00000020,
	TCAS_ERROR_INVALID_POS						=0x00000021,
	TCAS_ERROR_CLOSE_FAIL						=0x00000022,
	// Codec specified error code is defined here.
	TCAS_ERROR_CODEC_SPECIFY	= 0x00000100
}TCAS_ERROR_TYPE;


TCAS_CDSync_info	*gCDSync;	
TCAS_VAD_Struct_info	*gVAD;


#endif
