/******************************************************************************
*
*  (C)Copyright All Rights Reserved by Telechips Inc.
*                                   
*  This material is confidential and shall remain as such. 
*  Any unauthorized use, distribution, reproduction is strictly prohibited. 
*
*   FileName    : main.h
*   Description : 
*   TCC Version 1.0
*   Copyright (c) Telechips, Inc.
*   ALL RIGHTS RESERVED
*******************************************************************************/


#ifndef _MAIN_H_
#define _MAIN_H_

#include <stdio.h>
#include <linux/unistd.h>


/*************************************************************
 *  Printf Debugging
 *************************************************************/

// ̺κ  .
#ifdef PRINTF_DEBUG
#define	PRINTF printf
#else
#define	PRINTF
#endif

#define TCC_DEBUG_GET_TID			0

#define	PRINTF printf

#define TCC_DEBUG_NO_OUTPUT			0

// 1. TCC_DEBUG_PART_SELECT
#define TCC_DEBUG_GUI				1
#define TCC_DEBUG_CUI				(TCC_DEBUG_GUI << 1)
#define TCC_DEBUG_FRAMEWORK			(TCC_DEBUG_GUI << 2)
#define TCC_DEBUG_OMX_IL			(TCC_DEBUG_GUI << 3)
#define TCC_DEBUG_DRIVER			(TCC_DEBUG_GUI << 4)
#define TCC_DEBUG_COMPONENT			(TCC_DEBUG_GUI << 5)
#define TCC_DEBUG_CDK				(TCC_DEBUG_GUI << 6)
#define TCC_DEBUG_IPC				(TCC_DEBUG_GUI << 7)
#define TCC_DEBUG_COMMON			(TCC_DEBUG_GUI << 8)

// ⿡ part ߰մϴ.
#define TCC_DEBUG_ALL				0xFFFFFFFF

// 2. TCC_DEBUG_LEVEL
#define TCC_DEBUG_LEV_ERR        	1
#define TCC_DEBUG_LEV_WARNING		(TCC_DEBUG_LEV_ERR << 1)
#define TCC_DEBUG_LEV_SIMPLE_SEQ	(TCC_DEBUG_LEV_ERR << 2)
#define TCC_DEBUG_LEV_FULL_SEQ		(TCC_DEBUG_LEV_ERR << 3)
#define TCC_DEBUG_LEV_FUNC_NAME 	(TCC_DEBUG_LEV_ERR << 4)
#define TCC_DEBUG_LEV_INFO		(TCC_DEBUG_LEV_ERR << 5)

//  ʹ ߰ϰ ϴ LEVEL  ˴ϴ.
//    (TCC_DEBUG_LEV_ERR << 6) ~ (TCC_DEBUG_LEV_ERR << 30) Դϴ.
#define TCC_DEBUG_LEV_ALL			(TCC_DEBUG_LEV_ERR << 31)


#define TCC_DEBUG_PART_SELECT (TCC_DEBUG_GUI | TCC_DEBUG_IPC | TCC_DEBUG_CUI | TCC_DEBUG_COMPONENT | TCC_DEBUG_FRAMEWORK | TCC_DEBUG_OMX_IL | TCC_DEBUG_COMMON | TCC_DEBUG_CDK)
#define TCC_DEBUG_LEVEL (TCC_DEBUG_LEV_ERR | TCC_DEBUG_LEV_INFO)

#if TCC_DEBUG_PART_SELECT > 0
#define TCC_DEBUG_M(s,n, fmt, args...) \
	do { \
		if(TCC_DEBUG_PART_SELECT & (s)) \
		{ \
			if (TCC_DEBUG_LEVEL & (n)) \
			{ \
				fprintf(stderr, "[%s] [%d] " fmt, __FUNCTION__, __LINE__, ## args); \
			} \
		} \
	} while (0)
#else
#define TCC_DEBUG_M(s,n, fmt, args...)
#endif

// for temporary debugging
#define DBG(fmt, args...) 			do{ printf("[%s][%d][tid:%d]" fmt, __FUNCTION__, __LINE__, (int)syscall(224), ## args); }while(0)

//===============================================================================
// print the User message into the kernel message with the elapsed time after boot
//===============================================================================
#define ELASPED_TIME_CHECK_ENABLE

#ifdef ELASPED_TIME_CHECK_ENABLE
#define MSG_CAM_APP_START						"@@@@ [CAMERA APP START]\n"

#define MSG_CAM0_OPEN_START						"@@@@ [CAM_0 OPEN START]\n"
#define MSG_CAM0_OPEN_FINISH					"@@@@ [CAM_0 OPEN FINISH]\n"
#define MSG_CAM0_CONFIG_START					"@@@@ [CAM_0 CONFIG START]\n"
#define MSG_CAM0_CONFIG_FINISH					"@@@@ [CAM_0 CONFIG FINISH]\n"
#define MSG_CAM0_CLOSE_START					"@@@@ [CAM_0 CLOSE START]\n"
#define MSG_CAM0_CLOSE_FINISH					"@@@@ [CAM_0 CLOSE FINISH]\n"

#define MSG_CAM1_OPEN_START						"@@@@ [CAM_1 OPEN START]\n"
#define MSG_CAM1_OPEN_FINISH					"@@@@ [CAM_1 OPEN FINISH]\n"
#define MSG_CAM1_CONFIG_START					"@@@@ [CAM_1 CONFIG START]\n"
#define MSG_CAM1_CONFIG_FINISH					"@@@@ [CAM_1 CONFIG FINISH]\n"
#define MSG_CAM1_CLOSE_START					"@@@@ [CAM_1 CLOSE START]\n"
#define MSG_CAM1_CLOSE_FINISH					"@@@@ [CAM_1 CLOSE FINISH]\n"

#define MSG_VPU0_INIT_START						"@@@@ [VPU_0 INIT START]\n"
#define MSG_VPU0_INIT_FINISH					"@@@@ [VPU_0 INIT FINISH]\n"
#define MSG_VPU0_DEINIT_START					"@@@@ [VPU_0 DEINIT START]\n"
#define MSG_VPU0_DEINIT_FINISH					"@@@@ [VPU_0 DEINIT FINISH]\n"

#define MSG_VPU1_INIT_START						"@@@@ [VPU_1 INIT START]\n"
#define MSG_VPU1_INIT_FINISH					"@@@@ [VPU_1 INIT FINISH]\n"
#define MSG_VPU1_DEINIT_START					"@@@@ [VPU_1 DEINIT START]\n"
#define MSG_VPU1_DEINIT_FINISH					"@@@@ [VPU_1 DEINIT FINISH]\n"

#define MSG_CAM_APP_AUTOSTART_START				"@@@@ [CAMERA APP AUTOSTART START]\n"
#define MSG_CAM_APP_AUTOSTART_FINISH			"@@@@ [CAMERA APP AUTOSTART FINISH]\n"

#define MSG_CAM_APP_PLAYER_INIT_FINISH			"@@@@ [CAMERA APP PLAYER_INIT FINISH]\n"
#define MSG_CAM_APP_GUI_IPC_INIT_FINISH			"@@@@ [CAMERA APP GUI_IPC_INIT FINISH]\n"
#define MSG_CAM_APP_THREAD_INIT_FINISH			"@@@@ [CAMERA APP THREAD_INIT FINISH]\n"
#define MSG_CAM_APP_HOTPLUG_CHECK_FINISH		"@@@@ [CAMERA APP HOTPLUG_CHECK FINISH]\n"


#define MSG_CAM_OMXIL_OPEN_PROC_START			"@@@@ [CAMERA OMXIL OPEN PROCESS START]\n"
#define MSG_CAM_OMXIL_OPEN_PROC_FINISH			"@@@@ [CAMERA OMXIL OPEN PROCESS FINISH]\n"
#define MSG_CAM_OMXIL_LOAD_TO_IDLE_START		"@@@@ [CAMERA OMXIL LOAD TO IDLE START]\n"
#define MSG_CAM_OMXIL_LOAD_TO_IDLE_FINISH		"@@@@ [CAMERA OMXIL LOAD TO IDLE FINISH]\n"
#define MSG_CAM_OMXIL_IDLE_TO_EXEC_START		"@@@@ [CAMERA OMXIL IDLE TO EXECUTING START]\n"
#define MSG_CAM_OMXIL_IDLE_TO_EXEC_FINISH		"@@@@ [CAMERA OMXIL IDLE TO EXECUTING FINISH]\n"
#define MSG_CAM_OMXIL_EXEC_TO_IDLE_START		"@@@@ [CAMERA OMXIL EXECUTING TO IDLE START]\n"
#define MSG_CAM_OMXIL_EXEC_TO_IDLE_FINISH		"@@@@ [CAMERA OMXIL EXECUTING TO IDLE FINISH]\n"
#define MSG_CAM_OMXIL_IDLE_TO_LOAD_START		"@@@@ [CAMERA OMXIL IDLE TO LOAD START]\n"
#define MSG_CAM_OMXIL_IDLE_TO_LOAD_FINISH		"@@@@ [CAMERA OMXIL IDLE TO LOAD FINISH]\n"

#define MSG_BB_GUI_START						"@@@@ [BLACKBX GUI START]\n"
#define	MSG_BB_GUI_EVENT_THREAD_START			"@@@@ [BLACKBX GUI EVENT_THREAD START]\n"
#define MSG_BB_GUI_DRAW_MAIN_START				"@@@@ [BLACKBX GUI DRAW_MAIN START]\n"
#define MSG_BB_GUI_DRAW_MAIN_FINISH				"@@@@ [BLACKBX GUI DRAW_MAIN FINISH]\n"
#define MSG_BB_GUI_SEND_AUTOSTART				"@@@@ [BLACKBX GUI SEND_AUTOSTART]\n"
#define MSG_BB_GUI_SEND_CAMERA_OFF				"@@@@ [BLACKBX GUI SEND_CAMERA_OFF]\n"
#define MSG_BB_GUI_AUTOSTART_DONE				"@@@@ [BLACKBX GUI AUTOSTART_DONE]\n"
#define MSG_BB_GUI_CAMERA_OFF_DONE				"@@@@ [BLACKBX GUI CAMERA_OFF_DONE]\n"

#define MSG_SD_MOUNT_CHECK_START				"@@@@ [SD MOUNT CHECK START]\n"
#define MSG_SD_MOUNT_CHECK_FINISH				"@@@@ [SD MOUNT CHECK FINISH]\n"
#define MSG_MKFS_CHECK_START					"@@@@ [MKFS CHECK START]\n"
#define MSG_MKFS_CHECK_FINISH					"@@@@ [MKFS CHECK FINISH]\n"
#define MSG_FILEMANAGER_INIT_START				"@@@@ [FILEMANAGER_INIT_START]\n"
#define MSG_FILEMANAGER_INIT_FINISH				"@@@@ [FILEMANAGER_INIT_FINISH]\n"
#define MSG_FILEMANAGER_SEARCHING_START		"@@@@ [FILEMANAGER_SEARCHING_START]\n"
#define MSG_FILEMANAGER_SEARCHING_FINISH		"@@@@ [FILEMANAGER_SEARCHING_FINISH]\n"

#define MSG_CAM_PREVIEW_START					"@@@@ [CAM PREVIEW START]\n"
#define MSG_MUXER_WRITE_FIRST_VIDEO				"@@@@ [MUXER WRITE FIRST VIDEO]\n"

#define KLOG(msg)	do { 	\
						int fd = open("/dev/ttyprintk", O_RDWR | O_NONBLOCK); 	\
						write(fd, msg, strlen(msg)); 	\
					    close(fd);	\
					}while(0)
#else
#define KLOG(msg)
#endif
//===============================================================================

#endif //MAIN_H_
/* end of file */
