/******************************************************************************
*
*  (C)Copyright All Rights Reserved by Telechips Inc.
*
*  This material is confidential and shall remain as such.
*  Any unauthorized use, distribution, reproduction is strictly prohibited.
*
*   FileName    : tcc_camera_common.h
*   Description :
*   TCC Version 1.0
*   Copyright (c) Telechips, Inc.
*   ALL RIGHTS RESERVED
*******************************************************************************/
#ifndef __TCC_CAMERA_COMMON_H__
#define __TCC_CAMERA_COMMON_H__


/******************************************************************************
* include
******************************************************************************/
#include <OMX_Types.h>
#include <OMX_IVCommon.h>
#include "tcc_time.h"
#include "tcc_performance.h"
#include "tcc_gps_userdata.h"
#include "tcc_gsensor_userdata.h"
#include "main.h"
/******************************************************************************
* defines & enum
******************************************************************************/

//FILEINFO Struct
#define FILEINFO_VENDOR 			"tcc"  	//max char[24]
#define FILEINFO_MODEL_ID 		0		//uint
#define FILEINFO_SERIAL_NUM 		0		//uint
#define FILEINFO_HW_VER 			0		//uint
#define FILEINFO_SW_VER_MAJOR	1		//uint
#define FILEINFO_SW_VER_MINOR	0		//uint

#define TCC_CAMERA_DEVICE_NAME "/mnt/SD0"

#define TCC_CAMERA_NAND_DEVICE_NAME "/nand2"
#define TCC_CAMERA_SD_DEVICE_NAME "/mnt/SD0"

#define TCC_CAMERA_DIR_NAME "/camera"
#define TCC_CAMERA_NORMAL_DIR_NAME "/normal"
#define TCC_CAMERA_EVENT_DIR_NAME "/event"
#define TCC_CAMERA_USER_DIR_NAME "/user"
#define TCC_CAMERA_MOTION_DIR_NAME "/parking"
#define TCC_CAMERA_PICTURE_DIR_NAME "/pictures"
#ifdef TCC_CAMERA_DETAIL_DIR_INCLUDE
#define TCC_CAMERA_PICTURE_SUB1_DIR_NAME "/N_Camera" // Camera 1 Frame Capture
#define TCC_CAMERA_PICTURE_SUB2_DIR_NAME "/C_Camera" // Camera 15 Frame Capture
#define TCC_CAMERA_PICTURE_SUB3_DIR_NAME "/A_Camera" // Video 30 Frame Capture
#else
#define TCC_CAMERA_PICTURE_SUB1_DIR_NAME "" // Camera 1 Frame Capture
#define TCC_CAMERA_PICTURE_SUB2_DIR_NAME "" // Camera 15 Frame Capture
#define TCC_CAMERA_PICTURE_SUB3_DIR_NAME "" // Video 30 Frame Capture
#endif

#define TCC_CAMERA_PRE_FIX_NAME "tcc_"
#define TCC_CAMERA_EXTENSION_MP4 ".mp4"
#define TCC_CAMERA_EXTENSION_TS ".ts"
#define TCC_CAMERA_EXTENSION_JPEG ".jpg"

#define TCC_CAMERA_SENSOR_PATH "/sys/devices/platform/tcc-vin-video1.1/sensor_attach"

// Camera
#define CAMERA_SRC_0_NAME "OMX.tcc.camera.src_0"
#define CAMERA_SRC_1_NAME "OMX.tcc.camera.src_1"
#define CAMERA_SRC_2_NAME "OMX.tcc.camera.src_2"
#define CAMERA_SRC_3_NAME "OMX.tcc.camera.src_3"
#define AUDIO_SRC_NAME "OMX.tcc.alsa.alsasrc"
#define AUDIO_CDK_ENC_NAME "OMX.tcc.audio_encoder.cdk"

#define CAMERA_RENDER_0_NAME			"OMX.tcc.camera.render_0"
#define CAMERA_RENDER_1_NAME			"OMX.tcc.camera.render_1"


#define VIDEO_ENC_0_NAME "OMX.tcc.video_encoder_0"
#define VIDEO_ENC_1_NAME "OMX.tcc.video_encoder_1"
#define VIDEO_ENC_2_NAME "OMX.tcc.video_encoder_2"
#define VIDEO_ENC_3_NAME "OMX.tcc.video_encoder_3"

#define MUX_SINK_0_NAME "OMX.tcc.mux_sink_0"
#define MUX_SINK_1_NAME "OMX.tcc.mux_sink_1"
#define MUX_SINK_2_NAME "OMX.tcc.mux_sink_2"
#define MUX_SINK_3_NAME "OMX.tcc.mux_sink_3"

/* Motion Detection Component Name Definition */
#define MOTION_DETECTION_COMP_NAME "OMX.tcc.video.motion_detection"
#define MOTION_DETECTION_COMP_ROLE "video.motion_detection"

#define VIDEO_ENC_MPEG4_NAME "OMX.video_encoder.mpeg4"
#define VIDEO_ENC_MPEG4_ROLE "video_encoder.mpeg4"
#define VIDEO_ENC_AVC_NAME "OMX.tcc.video_encoder.avc"
#define VIDEO_ENC_AVC_ROLE "video_encoder.avc"


/* VIQE Component Name Definition */
#define VIQE_COMP_NAME "OMX.tcc.video.viqe"
#define VIQE_COMP_ROLE "video.viqe"


/* Jpeg encoder Component Name Definition */
#define CAMERA_JPEGENC_COMP_NAME "OMX.tcc.camera.jpeg_encoder"
#define CAMERA_JPEGENC_COMP_ROLE "camera.jpeg_encoder"

#define CAMERA_ENC_SRC_BUFFER_SIZE 32 * 1024

#define DEFAULT_EVENT_RECORDING_START_TIME_SEC 15 // Sec
#define DEFAULT_EVENT_RECORDING_END_TIME_SEC 15 // Sec
#define DEFAULT_EVENT_RECORDING_ADDED_TIME_SEC 15 // Sec
#define ADDED_EVENT_RECORDING_MAX 2

#define DEFAULT_FRAME_RATE 30 // default fps & default i-frame interval
#define DEFAULT_VIDEO_ENCODING_BITRATE (8 * 1024 * 1024) // 8M bitrate
#define DEFAULT_AUDIO_ENCODING_BITRATE (128 * 1000)
#define DEFAULT_ENCODING_QUALITY_LEVEL 11

#if 0
#define DEFAULT_FRAME_WIDTH 1920
#define DEFAULT_FRAME_HEIGHT 1080
#else
#define DEFAULT_FRAME_WIDTH 1280
#define DEFAULT_FRAME_HEIGHT 720
#endif


#define LCD_FRAME_WIDTH 240
#define LCD_FRAME_HEIGHT 128

#define ROTATE_LCD_FRAME_WIDTH 240
#define ROTATE_LCD_FRAME_HEIGHT 320

#define TVOUT_FRAME_WIDTH 720
#define TVOUT_FRAME_HEIGHT 480

#define HDMI_FRAME_WIDTH	1920
#define HDMI_FRAME_HEIGHT	1080

#ifdef TCC_CAMERA_EXTEND_DISPLAY_HDMI_INCLUDE
#define EXTEND_DISPLAY_WIDTH	HDMI_FRAME_WIDTH
#define EXTEND_DISPLAY_HEIGHT	HDMI_FRAME_HEIGHT
#else
#define EXTEND_DISPLAY_WIDTH	TVOUT_FRAME_WIDTH
#define EXTEND_DISPLAY_HEIGHT	TVOUT_FRAME_HEIGHT
#endif

#define DISPLAY_WIDTH	ROTATE_LCD_FRAME_WIDTH
#define DISPLAY_HEIGHT	ROTATE_LCD_FRAME_HEIGHT


#ifdef TCC_CAMERA_EXTEND_DISPLAY_INCLUDE
	#if ((EXTEND_DISPLAY_WIDTH*EXTEND_DISPLAY_HEIGHT) > (ROTATE_LCD_FRAME_WIDTH*ROTATE_LCD_FRAME_HEIGHT))
		#define DISPLAY_MAX_WIDTH	EXTEND_DISPLAY_WIDTH
		#define DISPLAY_MAX_HEIGHT	EXTEND_DISPLAY_HEIGHT
	#else
		#define DISPLAY_MAX_WIDTH	ROTATE_LCD_FRAME_WIDTH
		#define DISPLAY_MAX_HEIGHT	ROTATE_LCD_FRAME_HEIGHT
	#endif
#else
	#define DISPLAY_MAX_WIDTH	ROTATE_LCD_FRAME_WIDTH
	#define DISPLAY_MAX_HEIGHT	ROTATE_LCD_FRAME_HEIGHT
#endif

#define PHYSIC_BUFFER_USE
#ifdef PHYSIC_BUFFER_USE // when use camera physical memory to vide enc,jpeg enc, motion and etc
#define DEFAULT_MAX_BUFFER_SIZE 1024*10
#else
#define DEFAULT_MAX_BUFFER_SIZE ((DEFAULT_FRAME_WIDTH * DEFAULT_FRAME_HEIGHT * 3) / 2)
#endif

#define CAMERA_DO_QUERY_ERROR -1

#ifndef ALIGNED_BUFF
#define ALIGNED_BUFF(buf, mul) ( ( (unsigned int)buf + (mul-1) ) & ~(mul-1) )
#endif


#define DEFAULT_MAX_VIDEO_ENC_BUFFER_SIZE 	(500 * 1024) //500 kbyte

#define TCC_CAMERA_MUXER_QUEUE_ALPHA_SIZE 6

#define TCC_CAMERA_GPS_DATA_SIZE 120
#define TCC_CAMERA_GSENSOR_DATA_SIZE 28

#define TCC_CAMERA_GPS_QUEUE_SIZE (TCC_CAMERA_GPS_DATA_SIZE * (DEFAULT_EVENT_RECORDING_START_TIME_SEC + TCC_CAMERA_MUXER_QUEUE_ALPHA_SIZE))
#define TCC_CAMERA_GSENSOR_QUEUE_SIZE (TCC_CAMERA_GSENSOR_DATA_SIZE * (DEFAULT_EVENT_RECORDING_START_TIME_SEC + TCC_CAMERA_MUXER_QUEUE_ALPHA_SIZE)) * 50

#define TCC_CAMERA_GPS_QUEUE_MAX_ITEM (DEFAULT_EVENT_RECORDING_START_TIME_SEC + (TCC_CAMERA_MUXER_QUEUE_ALPHA_SIZE / 2))
#define TCC_CAMERA_GSENSOR_QUEUE_MAX_ITEM (DEFAULT_EVENT_RECORDING_START_TIME_SEC + (TCC_CAMERA_MUXER_QUEUE_ALPHA_SIZE / 2)) * 50

#define VIDEO_PORT_INDEX 0 /* video port index */
#define AUDIO_PORT_INDEX 1 /* audio port index */

#define TCC_CAMERA_SD_DEVICE SD_DEVICE0
#define TCC_CAMERA_NAND_DEVICE NAND_DEVICE1
#define TCC_CAMERA_OTG0_DEV0_DEVICE OTG0_DEVICE0

#define TCC_CAMERA_REAR_DEVICE CAMERA_DEVICE_REAR

#define DEFAULT_CAMERA_COLOR_FORMAT OMX_COLOR_FormatYUV420PackedPlanar //OMX_COLOR_FormatYUV420PackedPlanar //OMX_COLOR_FormatYUV420SemiPlanar
#define DEFAULT_FBSINK_COLOR_FORMAT OMX_COLOR_Format24bitRGB888
#define DEFAULT_CAPTURE_COLOR_FORMAT DEFAULT_CAMERA_COLOR_FORMAT
#define DEFAULT_COLOR_FORMAT OMX_COLOR_FormatYUV420PackedPlanar

#define TCC_CAMERA_GPS_DEFAULT_LATITUDE 3730.9033
#define TCC_CAMERA_GPS_DEFAULT_LONGITUDE 12706.3379
#define TCC_CAMERA_GPS_GMT_OFFSET 9 /* Seoul, Korea */
#define TCC_CAMERA_GPS_SPEED 1.852
#define TCC_CAMERA_GPS_YEAR_OFFSET 2000
#define TCC_CAMERA_GPS_TIME_SET_GAP 60 * 10 // 10minute
#define TCC_CAMERA_GPS_HDOP 6.0

#define TCC_CAMERA_MOTION_DEFAULT_PXL_THRES 5
#define TCC_CAMERA_MOTION_DEFAULT_BLOCK_THRES 12
#define TCC_CAMERA_MOTION_DEFAULT_MOTION_THRES 10
#define TCC_CAMERA_MOTION_DEFAULT_NVAL 4
#define TCC_CAMERA_MOTION_DEFAULT_ALG_INTERVAL 2
#define TCC_CAMERA_MOTION_DEFAULT_SAMPLING_RATE 24
#define TCC_CAMERA_MOTION_DEFAULT_BLOCK_SIZE_WIDTH 8
#define TCC_CAMERA_MOTION_DEFAULT_BLOCK_SIZE_HEIGHT 5

#define TCC_CAMERA_MULTI_CAMERA_MAX 4

#ifdef TCC_CAMERA_MULTI_CAMERA_INCLUDE
#define TCC_CAMERA_MULTI_CAMERA_N 2
#else
#define TCC_CAMERA_MULTI_CAMERA_N 1
#endif

#define TCC_CAMERA_RENDER_HANDLE_0		0
#define TCC_CAMERA_RENDER_HANDLE_1		1
#define TCC_CAMERA_RENDER_HANDLE_MAX	2

#define TCC_CAMERA_RENDER_PORT_0	0
#define TCC_CAMERA_RENDER_PORT_1	1

enum CAMERA_RENDER_OUTPUT_PORT_TYPE_e
{
	TCC_CAMERA_RENDER_OUTPUT_PORT_ENC_0 = 2,
	TCC_CAMERA_RENDER_OUTPUT_PORT_ENC_1,
	TCC_CAMERA_RENDER_OUTPUT_PORT_JPEG_0,
	TCC_CAMERA_RENDER_OUTPUT_PORT_JPEG_1,
};

#define CLEAR_INDEX_CONFIRM_VALUE	0xD0
#define CLEAR_INDEX_IGNORE_VALUE	0xE0

#define TCC_CAMERA_CAM_INDEX_0	0
#define TCC_CAMERA_CAM_INDEX_1	1
#define TCC_CAMERA_CAM_INDEX_2	2
#define TCC_CAMERA_CAM_INDEX_3	3
#define TCC_CAMERA_CAM_INDEX_ALL 4

#define TCC_CAMERA_0_SOURCE CAMERA_SOURCE_CAMERA_DRIVER
#define TCC_CAMERA_1_SOURCE CAMERA_SOURCE_CAMERA_DRIVER
#define TCC_CAMERA_2_SOURCE CAMERA_SOURCE_CAMERA_DRIVER
#define TCC_CAMERA_3_SOURCE CAMERA_SOURCE_CAMERA_DRIVER

#define TCC_CAMERA_VIQE_COMPONENT_INDEX 0

#define TCC_CAMERA_CABLE_CONNECTION_OK			0
#define TCC_CAMERA_CABLE_CONNECTION_FAIL		1

#define TCC_CAMERA_0_SOURCE_FILE_NAME "/nand1/v4l2_camera_0.yuv"
#define TCC_CAMERA_1_SOURCE_FILE_NAME "/nand1/v4l2_camera_1.yuv"
#define TCC_CAMERA_2_SOURCE_FILE_NAME "/nand1/v4l2_camera_2.yuv"
#define TCC_CAMERA_3_SOURCE_FILE_NAME "/nand1/v4l2_camera_3.yuv"

#define TCC_CAMERA_FORMAT_CONFIRM_TIME 10 * 1000 // msec
#define TCC_CAMERA_JPEG_AGING_TIME 1000 // msec

#define TCC_CAMERA_USER_REC_AGING_TIME 1000 * 60 // msec
#define TCC_CAMERA_REBOOT_AGING_TIME 1000 * 60 * 5

#define TCC_CAMERA_SD_ERROR_TIME 15 * 1000 // 15sec
#define TCC_CAMERA_SD_ERROR_TIME_ONE 5 * 1000 // 5sec

#define TCC_CAMERA_OPEN_ERROR_COUNT 10

//#define TCC_CAMERA_START_PROFILE

typedef enum TCC_CAMERA_APP_STATE_TYPE
{
	TCC_CAMERA_APP_STATE_READY = 0,
	TCC_CAMERA_APP_STATE_OPEN,
	TCC_CAMERA_APP_STATE_RUN,
	TCC_CAMERA_APP_STATE_EXIT,
}TCC_CAMERA_APP_STATE_TYPE;

typedef enum
{
	CAMERA_DEVICE_PLUG_INOUT_FRONT = 0,
	CAMERA_DEVICE_PLUG_INOUT_REAR,
	CAMERA_DEVICE_PLUG_INOUT_LEFT,
	CAMERA_DEVICE_PLUG_INOUT_RIGHT,
}CAMERA_DEVICE_PLUG_IN_OUT_TYPE;

typedef enum
{
	CAMERA_SUB_BATTERY_LEVEL_0 = 0,
	CAMERA_SUB_BATTERY_LEVEL_1,
	CAMERA_SUB_BATTERY_LEVEL_2,
	CAMERA_SUB_BATTERY_LEVEL_3,
}CAMERA_SUB_BATTERY_TYPE;

typedef enum
{
	CAMERA_DEVICE_PLUGINIT = 0,
	CAMERA_DEVICE_PLUGIN,
	CAMERA_DEVICE_PLUGOUT,
	CAMERA_DEVICE_PLUGERROR,
}CAMERA_DEVICE_PLUG_IN_STATUS;

typedef enum
{
	CAMERA_GPS_PLUGINIT = 0,
	CAMERA_GPS_PLUGIN,
	CAMERA_GPS_PLUGOUT,
	CAMERA_GPS_PLUGERROR,
}CAMERA_GPS_PLUG_IN_STATUS;

typedef struct
{
	unsigned int mode;
	unsigned int width;
	unsigned int height; //!< samples/sec
}EXTEND_DISP_INFO_T;

typedef struct
{
	unsigned int mode;
	unsigned int channel;	//	Main ch
}DISP_INFO_T;

typedef enum
{
	CAMERA_MUX_TS = 0,
	CAMERA_MUX_MP4,
	CAMERA_MUX_NUM
}TCC_CAMERA_MUX_TYPE;


typedef enum
{
	CAMERA_AUDIO_NO = 0,
	CAMERA_AUDIO_MP3,
	CAMERA_AUDIO_AAC,
	CAMERA_AUDIO_WMA,
	CAMERA_AUDIO_PCM,
	CAMERA_AUDIO_NUM
}TCC_CAMERA_AUDIO_TYPE;

typedef enum
{
	CAMERA_VIDEO_NO = 0,
	CAMERA_VIDEO_AVC,
	CAMERA_VIDEO_MPEG,
	CAMERA_VIDEO_NUM
}TCC_CAMERA_VIDEO_TYPE;

typedef enum
{
	CAMERA_RESOLUTION_DEFAULT = 0,
	CAMERA_RESOLUTION_QVGA,
	CAMERA_RESOLUTION_VGA,
	CAMERA_RESOLUTION_SVGA,
	CAMERA_RESOLUTION_720P,
	CAMERA_RESOLUTION_MAX

}TCC_CAMERA_RESOLUTION_TYPE;

//	Audio

typedef enum
{
	CAMERA_AUDIO_INPUT_MIC = 0,
	CAMERA_AUDIO_INPUT_LINE_IN,
	CAMERA_AUDIO_INPUT_MAX,
}TCC_CAMERA_AUDIO_INPUT_TYPE;

typedef enum
{
	CAMERA_AUDIO_FREQ8K = 0,
	CAMERA_AUDIO_FREQ16K,
	CAMERA_AUDIO_FREQ32K,
	CAMERA_AUDIO_FREQ44K,
	CAMERA_AUDIO_FREQ48K,
	CAMERA_AUDIO_FREQ_MAX
}TCC_CAMERA_AUDIO_FREQUENCY_TYPE;

typedef enum
{
	CAMERA_AUDIO_BPS16 = 0,
	CAMERA_AUDIO_BPS32,
	CAMERA_AUDIO_BPS64,
	CAMERA_AUDIO_BPS128,
	CAMERA_AUDIO_BPS192,
	CAMERA_AUDIO_BPS256,
	CAMERA_AUDIO_BPS_MAX
}TCC_CAMERA_AUDIO_BITPERSAMPLE_TYPE;

typedef enum TCC_OMX_CAMERA_APP_EVENT_MODE
{
	TCC_CAMERA_EVENT_ERROR = -1,
	TCC_CAMERA_EVENT_NULL = 0,
	TCC_CAMERA_EVENT_ACCIDENT,
	TCC_CAMERA_EVENT_MOTION,
	TCC_CAMERA_EVENT_USER,
	TCC_CAMERA_EVENT_AUTO_JPEG,
}TCC_OMX_CAMERA_APP_EVENT_MODE;

typedef enum TCC_OMX_CAMERA_APP_RECORDING_MODE
{
	TCC_CAMERA_RECORDING_NULL = 0,
	TCC_CAMERA_RECORDING_NORMAL,
	TCC_CAMERA_RECORDING_AUTO,
	TCC_CAMERA_RECORDING_EVENT,
	TCC_CAMERA_RECORDING_JPEG,
}TCC_OMX_CAMERA_APP_RECORDING_MODE;

typedef enum TCC_OMX_CAMERA_APP_PREVIEW_MODE
{
	TCC_CAMERA_PREVIEW_OFF = 0,
	TCC_CAMERA_PREVIEW_ON,
}TCC_OMX_CAMERA_APP_PREVIEW_MODE;

typedef enum TCC_OMX_CAMERA_APP_CAMERA_EXTEND_DISPLAY_DEVICE
{
	TCC_CAMERA_EXTEND_DISPLAY_DEVICE_NONE = 0, //
	TCC_CAMERA_EXTEND_DISPLAY_DEVICE_LCD,
	TCC_CAMERA_EXTEND_DISPLAY_DEVICE_HDMI,
	TCC_CAMERA_EXTEND_DISPLAY_DEVICE_NTSC,
	TCC_CAMERA_EXTEND_DISPLAY_DEVICE_PAL,
	TCC_CAMERA_EXTEND_DISPLAY_DEVICE_MAX
}TCC_OMX_CAMERA_APP_CAMERA_EXTEND_DISPLAY_DEVICE;

typedef enum TCC_OMX_CAMERA_APP_CAMERA_DISPLAY_MODE
{
	TCC_CAMERA_DISPLAY_MODE_NONE = -1,
	TCC_CAMERA_DISPLAY_MODE_1CH = 0,
	TCC_CAMERA_DISPLAY_MODE_2CH,
	TCC_CAMERA_DISPLAY_MODE_3CH,
	TCC_CAMERA_DISPLAY_MODE_4CH,
	TCC_CAMERA_DISPLAY_MODE_VERTICAL,
	TCC_CAMERA_DISPLAY_MODE_HORIZONTAL,
	TCC_CAMERA_DISPLAY_MODE_QUAD,
	TCC_CAMERA_DISPLAY_MODE_PIP,
	TCC_CAMERA_DISPLAY_MODE_MAX
}TCC_OMX_CAMERA_APP_CAMERA_DISPLAY_MODE;

typedef enum TCC_OMX_CAMERA_APP_PIP_MODE
{
	TCC_CAMERA_DISPLAY_PIP_NONE = -1,
	TCC_CAMERA_DISPLAY_PIP_FRONT_BASE = 0,
	TCC_CAMERA_DISPLAY_PIP_REAR_BASE,
	TCC_CAMERA_DISPLAY_PIP_MAX,
}TCC_OMX_CAMERA_APP_PIP_MODE;

typedef enum TCC_OMX_CAMERA_ASPECT_RATIO_MODE
{
	TCC_CAMERA_DISPLAY_ASPECT_RATIO_ORIGINAL = 0,
	TCC_CAMERA_DISPLAY_ASPECT_RATIO_FULL,

	TCC_CAMERA_DISPLAY_ASPECT_RATIO_MAX
}TCC_OMX_CAMERA_APP_ASPECT_RATIO_MODE;

typedef enum TCC_OMX_CAMERA_APP_CAMERA_EXTEND_DISPLAY_COMPOSITE_MODE
{
	TCC_CAMERA_COMPOSITE_NTSC = 0,
	TCC_CAMERA_COMPOSITE_PAL,
	TCC_CAMERA_COMPOSITE_MAX
}TCC_OMX_CAMERA_APP_CAMERA_EXTEND_DISPLAY_COMPOSITE_MODE;

typedef enum
{
	CAMERA_PREVIEW_OFF = 0,
	CAMERA_PREVIEW_ON,
}TCC_CAMERA_PREVIEW_TYPE;

typedef enum
{
	CAMERA_OFF = 0,
	CAMERA_ON,
}TCC_CAMERA_ONOFF_TYPE;

typedef enum
{
	CAMERA_RESOLUTION_VGA_WIDTH = 640,
	CAMERA_RESOLUTION_HD_WIDTH = 1280,
	CAMERA_RESOLUTION_FHD_WIDTH = 1920,
}TCC_CAMERA_WIDTH_RESOLUTION_TYPE;

typedef enum
{
	CAMERA_MOTION_OFF = 0,
	CAMERA_MOTION_ON,
}TCC_CAMERA_MOTION_TYPE;

typedef enum
{
	CAMERA_MIC_ON = 0,
	CAMERA_MIC_OFF,
}TCC_CAMERA_MIC_TYPE;

typedef enum
{
	CAMERA_LANGUAGE_SET_BY_SETTING_VALUE = -1,
	CAMERA_LANGUAGE_ENGLISH = 0,
	CAMERA_LANGUAGE_KOREAN,
	CAMERA_LANGUAGE_CHINESE,

	CAMERA_LANGUAGE_MAX,
}TCC_CAMERA_LANGUAGE_TYPE;

typedef enum
{
	// TCC_VOLUME_DEFAULT
	CAMERA_MIC_VOLUME_LOW = (90 - 20),
	CAMERA_MIC_VOLUME_MIDDLE = 90,
	CAMERA_MIC_VOLUME_HIGH = (90 + 10),
}TCC_CAMERA_MIC_VOLUME;

typedef enum
{
	CAMERA_SENSOR_SENSITIVITY_LOW = 0,
	CAMERA_SENSOR_SENSITIVITY_MID,
	CAMERA_SENSOR_SENSITIVITY_HIGH
}TCC_CAMERA_GSENSOR_SENSITIVITY_TYPE;

#if 0
typedef enum
{
	CAMERA_SENSOR_SENSITIVITY_LOW_VALUE = 0.8,
	CAMERA_SENSOR_SENSITIVITY_MID_VALUE = 1.2,
	CAMERA_SENSOR_SENSITIVITY_HIGH_VALUE = 1.8
}TCC_CAMERA_GSENSOR_SENSITIVITY_VALUE_TYPE;
#endif


typedef enum
{
	CAMERA_MOTION_DETECTION_SENSITIVITY_LOW_VALUE = 60,
	CAMERA_MOTION_DETECTION_SENSITIVITY_MID_VALUE = 40,
	CAMERA_MOTION_DETECTION_SENSITIVITY_HIGH_VALUE = 20,
}TCC_CAMERA_MOTION_DETECTION_SENSITIVITY_VALUE_TYPE;

typedef enum
{
	CAMERA_LCD_OFF_TIME_OFF = -1,
	CAMERA_LCD_OFF_TIME_10S = 10,
	CAMERA_LCD_OFF_TIME_30S = 30,
	CAMERA_LCD_OFF_TIME_60S = 60,
}TCC_CAMERA_LCD_OFF_TIME_TYPE;

typedef enum
{
	CAMERA_AUTO_REC_1MIN = 60,
	CAMERA_AUTO_REC_3MIN = 180,
	CAMERA_AUTO_REC_5MIN = 300,
}TCC_CAMERA_AUTO_REC_TIME_TYPE;

#define DEFAULT_AUTO_RECODING_TIME_SEC (CAMERA_AUTO_REC_1MIN)

typedef enum
{
	CAMERA_SPEAKER_OFF = 0,
	CAMERA_SPEAKER_ON,
}TCC_CAMERA_SPEAKER_TYPE;

typedef enum
{
	CAMERA_SPEAKER_VOLUME_LOW = (90 - 20),
	CAMERA_SPEAKER_VOLUME_MIDDLE = 90,
	CAMERA_SPEAKER_VOLUME_HIGH = (90 + 10),
}TCC_CAMERA_SPEAKER_VOLUME;

typedef enum
{
	CAMERA_GUI_RESOLUTION_VGA_WIDTH = 640,
	CAMERA_GUI_RESOLUTION_D1_WIDTH = 720,
	CAMERA_GUI_RESOLUTION_HD_WIDTH = 1280,
	CAMERA_GUI_RESOLUTION_FULL_HD_WIDTH = 1920,
}TCC_CAMERA_GUI_RESOLUTION_WIDTH;

typedef enum
{
	CAMERA_GUI_RESOLUTION_VGA_HEIGHT = 480,
	CAMERA_GUI_RESOLUTION_D1_HEIGHT = 480,
	CAMERA_GUI_RESOLUTION_HD_HEIGHT = 720,
	CAMERA_GUI_RESOLUTION_FULL_HD_HEIGHT = 1080,
}TCC_CAMERA_GUI_RESOLUTION_HEIGHT;

typedef enum
{
	CAMERA_GPS_ERROR = -1, // for gui icon update
	CAMERA_GPS_OFF = 0,
	CAMERA_GPS_ON,
	CAMERA_GPS_RANGE_OUT,
}TCC_CAMERA_GPS_TYPE;

typedef enum
{
	CAMERA_GUI_RECORDING_NULL = -1,
	CAMERA_GUI_RECORDING_AUTO = 0,
	CAMERA_GUI_RECORDING_ACCIDENT,
	CAMERA_GUI_RECORDING_MOTION,
	CAMERA_GUI_RECORDING_USER,
}TCC_CAMERA_GUI_RECORDING_TYPE;

typedef enum
{
	CAMERA_SOURCE_CAMERA_DRIVER = 0,
	CAMERA_SOURCE_FILE
}TCC_CAMERA_SOURCE_TYPE;

typedef enum
{
	CAMERA_VIDEO_SCAN_X2 = 0,
	CAMERA_VIDEO_SCAN_X4,
	CAMERA_VIDEO_SCAN_X8,
}TCC_CAMERA_VIDEO_SCAN_SPEED;

typedef enum
{
	CAMERA_IMAGE_ZOOM_X2 = 0,
	CAMERA_IMAGE_ZOOM_X4,
	CAMERA_IMAGE_ZOOM_X8,
}TCC_CAMERA_IMAGE_ZOOM_SPEED;

typedef enum
{
	CAMERA_SD_PLUG_CHECKING = -7,
	CAMERA_SD_PLUG_BROKEN_PHYSICAL = -6, // SD Plug In and physical broken
	CAMERA_SD_PLUG_BROKEN_FILESYTEM = -5, // SD Plug In and mounted, but is broken of file system
	CAMERA_SD_PLUG_READ_ONLY = -4, // SD Plug In and mounted, but is Read Only Type
	CAMERA_SD_PLUG_NOT_SUPORT_FILESYTEM	= -3, // SD Plug In and mounted, but isn't support File system.
	CAMERA_SD_PLUG_NOT_MOUNT = -2, // SD Plug In, but isn't mounted
	CAMERA_SD_PLUG_OUT = 0, // SD Plug Out
	CAMERA_SD_PLUG_IN = 1, // SD Plug Out
	CAEMRA_SD_ERROR_NONE,
	CAMERA_SD_STAT_MAX,
}TCC_CAMERA_SD_STATE;

typedef enum
{
	TCC_CAMERA_UPDATE_NONE = 0,
	TCC_CAMERA_UPDATE_ALL,
	TCC_CAMERA_UPDATE_NORMAL,
	TCC_CAMERA_UPDATE_EVENT,
	TCC_CAMERA_UPDATE_USER,
	TCC_CAMERA_UPDATE_MOTION,
	TCC_CAMERA_UPDATE_JPEG,
	TCC_CAMERA_UPDATE_AUTO_JPEG,
	TCC_CAMERA_UPDATE_MOTION_TO_EVENT,	//	motion recording  accident recording  & file update (move accident -> parking)
	TCC_CAMERA_UPDATE_EVENT_REWIND,		//	rewide clear & file update (move accident -> parking)
	TCC_CAMERA_UPDATE_MOTION_TO_MOTION,	//	motion on  user/motion recording
	TCC_CAMERA_UPDATE_USER_TO_EVENT,
	TCC_CAMERA_UPDATE_MAX
}TCC_CAMERA_UPDATE_TYPE;

typedef enum
{
	TCC_CAMERA_REC_NONE = 0,
	TCC_CAMERA_REC_AUTO,
	TCC_CAMERA_REC_NORMAL,
	TCC_CAMERA_REC_USER,
	TCC_CAMERA_REC_MOTION,
	TCC_CAMERA_REC_EVENT,	// accident
	TCC_CAMERA_REC_MAX
}TCC_CAMERA_REC_TYPE;

typedef enum
{
	TCC_CAMERA_DETETCT_ERROR_NONE = 0,
	TCC_CAMERA_DETETCT_ERROR_DATA_BROKEN,
	TCC_CAMERA_DETETCT_ERROR_MAX
}TCC_CAMERA_DETECT_ERROR_TYPE;



typedef enum
{
	TCC_CAMERA_GPS_INVALID =0,
	TCC_CAMERA_GPS_VALID=1,
	TCC_CAMERA_GPS_RANGE_OUT=2,
}TCC_CAMERA_GPS_VALID_TYPE;



#define FREQ_8KHZ 8000
#define FREQ_11KHZ 11025
#define FREQ_12KHZ 12000
#define FREQ_16KHZ 16000
#define FREQ_22KHZ 22050
#define FREQ_24KHZ 24000
#define FREQ_32KHZ 32000
#define FREQ_44KHZ 44100
#define FREQ_48KHZ 48000

#define BITRATE_16Kbps 16
#define BITRATE_24Kbps 24
#define BITRATE_32Kbps 32
#define BITRATE_40Kbps 40
#define BITRATE_48Kbps 48
#define BITRATE_56Kbps 56
#define BITRATE_64Kbps 64
#define BITRATE_72Kbps 72
#define BITRATE_80Kbps 80
#define BITRATE_96Kbps 96
#define BITRATE_112Kbps 112
#define BITRATE_128Kbps 128
#define BITRATE_160Kbps 160
#define BITRATE_192Kbps 192
#define BITRATE_224Kbps 224
#define BITRATE_256Kbps 256
#define BITRATE_288Kbps 288


/******************************************************************************
* typedefs & structure
******************************************************************************/
typedef struct video_info_t
{
	int m_iFrameRate;
	int m_iBitrate;	//
	unsigned int m_iWidth;
	unsigned int m_iHeight;
	unsigned int m_iResolutionType;
	unsigned int m_iColorFormat;
	int m_iKeyFrameInterval;
}video_info_t;

typedef struct audio_info_t
{
	unsigned int m_iChannels;
	unsigned int m_iBitPerSample;
	unsigned int m_iBitRate;
	unsigned int m_iBitRateType;
	unsigned int m_iSamplePerSec;
	unsigned int m_iInputType;
}audio_info_t;

typedef struct muxer_info_t
{
	TCC_CAMERA_AUDIO_TYPE m_eAudioCodec;
	TCC_CAMERA_VIDEO_TYPE m_eVideoCodec;
	TCC_CAMERA_MUX_TYPE m_eMuxType;
}muxer_info_t;

typedef struct motion_detection_info_t
{
	unsigned int Index;	// motion detector index
	unsigned int OnOff;
	unsigned int PxlThres;
	unsigned int BlockThres;
	unsigned int MotionThres;
	unsigned int AlgInterval;
	unsigned int Nval;
	unsigned int SamplingRate;
	unsigned int BlockSizeWidth;
	unsigned int BlockSizeHeight;
}motion_detection_info_t;

typedef struct camsrc_info_t
{
	// video source info
	struct
	{
		unsigned int m_iFrameRate;
		unsigned int m_iWidth;
		unsigned int m_iHeight;
		unsigned int m_iColorFormat;
	}m_sVideoInfo;

	// audio source info
	struct
	{
		unsigned int m_iChannels;
		unsigned int m_iBitPerSample;
		unsigned int m_iSamplingRate;
	}m_sAudioInfo;

	// muxinfo
	struct
	{
		int m_iACodec;
		int m_iVCodec;
		int m_iMuxType;
		unsigned int m_iRecMode; // [in]
		unsigned int m_iRecTime; // [in]
	}m_sMuxInfo;
}camsrc_info_t;

typedef struct TCC_CAMERA_Info_Structure
{
	unsigned int m_iColorFormat;
	unsigned int m_iWidth;
	unsigned int m_iHeight;
	int m_iFrameRate;
	int m_iIframeInterval;
	int m_iTargetbps;
}TCC_CAMERA_Info_t;

typedef struct TCC_CAMERA_Config_Structure
{
	TCC_CAMERA_Info_t Camera_Info[TCC_CAMERA_MULTI_CAMERA_MAX];
	motion_detection_info_t Motion_Info[TCC_CAMERA_MULTI_CAMERA_MAX];
	int Gps_State;
	unsigned int Camera_Num;
	unsigned int Camera_Connected_Num;
	double GSensor_Threshold;
	unsigned int Speaker_Volume;
	unsigned int Mic_Volume;
	int Is_Audio_Guidance_On;
	int Is_Mic_On;
	int Is_Network_On;
	int Display_State;
	int Is_RearView_Reverse_On;
	int Is_Info_On;

	int LCD_Off_Time;
	int Gsensor_Sensitivity_Level;
	int Motion_Detector_Sensitivity_Level;

	int Year;
	int Month;
	int Day;
	int Hour;
	int Min;
	int Sec;

	int Media_Category;
	int Last_Played_File;

	int Video_Dec_Volume;
	int Video_Dec_Scan_Speed;
	int Image_Zoom;
	int Display_Mode;
	int PIP_OnOff_State;
	int PIP_Mode;
	int Aspect_Ratio;
	int Language_Index;
	int Auto_Rec_Time;
}TCC_CAMERA_Config_t;

typedef struct CAMERA_FRAMESIZETYPE
{
    unsigned int nWidth;
    unsigned int nHeight;
} CAMERA_FRAMESIZETYPE;

typedef struct tagFILEINFO
{
	char vendor[24];
	unsigned int model_id;
	unsigned int serial_num;
	unsigned int hw_version;
	unsigned int sw_version_major;
	unsigned int sw_version_minor;
	unsigned int reserved[3];
}FILEINFO;

typedef struct TCC_FILE_Tag_Structure
{
	unsigned int tag_time;
	unsigned int tag_etime;
	unsigned int tag_frrate;
	unsigned int tag_vbrate;
	unsigned int tag_abrate;
	unsigned int tag_ql;

}TCC_FILE_Tag_t;

typedef struct TCC_CAMERA_Rec_Update_Type_Structure
{
	TCC_CAMERA_UPDATE_TYPE update_rectype;
	unsigned int bStart;
	int flush_buff;
	unsigned int base_time;
}TCC_CAMERA_Rec_Update_Type_t;


typedef struct OMX_IMAGE_AUTO_CAPTURE_CONFIG_INFOTYPE
{
	unsigned int driver_index;
	unsigned int capture_count;
	unsigned int jpeg_auto_capture_mode;
} OMX_IMAGE_AUTO_CAPTURE_CONFIG_INFOTYPE;

typedef struct OMX_CAMERA_BUFFER_INDEX_INFOTYPE
{
	unsigned int clear_buffer_index;
} OMX_CAMERA_BUFFER_INDEX_INFOTYPE;

#if 1
typedef struct TCC_TS_MUX_UserData_Structure
{
	//Gsensor
	tcc_sensor_accel_t stGSensor;
	//GPS
	INFGPSXYZ_t stGPS;

}TCC_TS_MUX_UserData_t;
#endif

/******************************************************************************
* globals
******************************************************************************/
static const CAMERA_FRAMESIZETYPE g_SupportedFramesizeTable[] =
{
	{1280, 720}, // TELECHIPS Default
	{320, 240}, // QVGA
	{640, 480}, // VGA
	{720, 480}, // D1
	{800, 600}, // SVGA
	{1024, 768}, // XGA
	{1280, 720}, // 720P
	{1920, 1080}, //  FULL HD
};

TCC_CAMERA_Config_t TCC_Camera_Config;


/******************************************************************************
* locals
******************************************************************************/


/******************************************************************************
* declarations
******************************************************************************/


#endif // __TCC_CAMERA_COMMON_H__
