/******************************************************************************
*
*  (C)Copyright All Rights Reserved by Telechips Inc.
*
*  This material is confidential and shall remain as such.
*  Any unauthorized use, distribution, reproduction is strictly prohibited.
*
*   FileName    : TCCMemory.h
*   Description :
*   TCC Version 1.0
*   Copyright (c) Telechips, Inc.
*   ALL RIGHTS RESERVED
*******************************************************************************/
#ifndef _TCC_MEMORY_H__
#define _TCC_MEMORY_H__

/******************************************************************************
* include
******************************************************************************/
#include <stdlib.h>


/******************************************************************************
* typedefs & structure
******************************************************************************/
enum TCC_MALLOC_e
{
	TCC_MALLOC_IN_CDK = 0,
	TCC_MALLOC_IN_VIDEO_ENC,	
	TCC_MALLOC_IN_AUDIO_ENC,	
	TCC_MALLOC_IN_IMAGE_ENC,
	TCC_MALLOC_IN_MUX,

	TCC_MALLOC_IN_VIDEO_DEC,
	TCC_MALLOC_IN_AUDIO_DEC,
	TCC_MALLOC_IN_IMAGE_DEC,
	TCC_MALLOC_IN_DEMUX,

	TCC_MALLOC_IN_SUBTITLE,
	TCC_MALLOC_IN_FONT,
	TCC_MALLOC_IN_QUEUE,
	TCC_MALLOC_IN_ETC,
	
	TCC_MALLOC_MAX	
};


/******************************************************************************
* defines
******************************************************************************/


/******************************************************************************
* globals
******************************************************************************/


/******************************************************************************
* locals
******************************************************************************/


/******************************************************************************
* declarations
******************************************************************************/
#ifdef TCC_MEMORY_DEBUG
#define	TCC_malloc(iSize) TCC_debug_malloc(iSize, __FILE__, __FUNCTION__, __LINE__)
#define	TCC_calloc(isize_t, iSize) TCC_debug_calloc(isize_t, iSize, __FILE__, __FUNCTION__, __LINE__)
#define	TCC_realloc(p, iSize) TCC_debug_realloc(p,iSize, __FILE__, __FUNCTION__, __LINE__)
#define	TCC_free(pvPtr) TCC_debug_free(pvPtr, __FILE__, __FUNCTION__, __LINE__)
#else
void* TCC_malloc(unsigned int iSize);
void* TCC_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_realloc(void *p,unsigned int iSize);
int TCC_free(void *pvPtr);
#endif
void* TCC_call_malloc(unsigned int iSize);
void* TCC_call_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_call_realloc(void *p,unsigned int iSize);
int TCC_call_free(void *pvPtr);


void* TCC_CDK_malloc(unsigned int iSize);
void* TCC_CDK_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_CDK_realloc(void *p,unsigned int iSize);
int TCC_CDK_free(void *pvPtr);


void* TCC_MUX_malloc(unsigned int iSize);
void* TCC_MUX_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_MUX_realloc(void *p,unsigned int iSize);
int TCC_MUX_free(void *pvPtr);

void* TCC_Video_Dec_malloc(unsigned int iSize);
void* TCC_Video_Dec_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_Video_Dec_realloc(void *p,unsigned int iSize);
int TCC_Video_Dec_free(void *pvPtr);

void* TCC_AUDIO_Dec_malloc(unsigned int iSize);
void* TCC_AUDIO_Dec_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_AUDIO_Dec_realloc(void *p,unsigned int iSize);
int TCC_AUDIO_Dec_free(void *pvPtr);

void* TCC_Image_Dec_malloc(unsigned int iSize);
void* TCC_Image_Dec_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_Image_Dec_realloc(void *p,unsigned int iSize);
int TCC_Image_Dec_free(void *pvPtr);

void* TCC_DeMUX_malloc(unsigned int iSize);
void* TCC_DeMUX_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_DeMUX_realloc(void *p,unsigned int iSize);
int TCC_DeMUX_free(void *pvPtr);

void* TCC_VIDEO_Enc_malloc(unsigned int iSize);
void* TCC_VIDEO_Enc_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_VIDEO_Enc_realloc(void *p,unsigned int iSize);
int TCC_VIDEO_Enc_free(void *pvPtr);

void* TCC_AUDIO_Enc_malloc(unsigned int iSize);
void* TCC_AUDIO_Enc_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_AUDIO_Enc_realloc(void *p,unsigned int iSize);
int TCC_AUDIO_Enc_free(void *pvPtr);

void* TCC_Image_Enc_malloc(unsigned int iSize);
void* TCC_Image_Enc_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_Image_Enc_realloc(void *p,unsigned int iSize);
int TCC_Image_Enc_free(void *pvPtr);

void* TCC_Image_Eff_malloc(unsigned int iSize);
void* TCC_Image_Eff_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_Image_Eff_realloc(void *p,unsigned int iSize);
int TCC_Image_Eff_free(void *pvPtr);

void* TCC_Subtitle_malloc(unsigned int iSize);
void* TCC_Subtitle_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_Subtitle_realloc(void *p,unsigned int iSize);
int TCC_Subtitle_free(void *pvPtr);

void* TCC_Queue_malloc(unsigned int iSize);
void* TCC_Queue_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_Queue_realloc(void *p,unsigned int iSize);
int TCC_Queue_free(void *pvPtr);

void* TCC_Font_malloc(unsigned int iSize);
void* TCC_Font_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_Font_realloc(void *p,unsigned int iSize);
int TCC_Font_free(void *pvPtr);

void* TCC_ETC_malloc(unsigned int iSize);
void* TCC_ETC_calloc(unsigned int isize_t, unsigned int iSize);
void* TCC_ETC_realloc(void *p,unsigned int iSize);
int TCC_ETC_free(void *pvPtr);

//#define TCC_MEMORY_PROBE
#ifdef TCC_MEMORY_PROBE
long TCC_Memory_Probe_Start(enum TCC_MALLOC_e type);
long TCC_Memory_Probe_Finish(enum TCC_MALLOC_e type);
long TCC_Memory_Probe_Check(enum TCC_MALLOC_e type);
#else
#define TCC_Memory_Probe_Start(a)
#define TCC_Memory_Probe_Finish(a)
#define TCC_Memory_Probe_Check(a)
#endif

#endif //_TCC_UTIL_H___
