/******************************************************************************
*
*  (C)Copyright All Rights Reserved by Telechips Inc.
*
*  This material is confidential and shall remain as such.
*  Any unauthorized use, distribution, reproduction is strictly prohibited.
*
*   FileName    : tcc_performance.h
*   Description :
*   TCC Version 1.0
*   Copyright (c) Telechips, Inc.
*   ALL RIGHTS RESERVED
*******************************************************************************/
#ifndef _TCC_PERFORMANCE_H__
#define _TCC_PERFORMANCE_H__


/******************************************************************************
* include
******************************************************************************/
#include <stdio.h>
#include "tcc_time.h"
#include "tcc_file.h"
#include "tcc_memory.h"


/******************************************************************************
* defines
******************************************************************************/
//#define VIDEO_PERF_MEASURE
//#define CAMERA_PERF_MEASURE

#define TCC_PERFORMANCE_MAX_STRING 256
#define TCC_PERFORMANCE_MAX_FILENAME_LEN 256

#ifdef VIDEO_PERF_MEASURE

#define MEASURE_COUNT_FILEIO 20000
#define MEASURE_COUNT_CDMX 2000
#define MEASURE_COUNT_VDEC 2000
#define MEASURE_COUNT_ADEC 2000
#define MEASURE_COUNT_SCALER 2000

#define SUFFIX_FILEIO "_FILEIO.TXT"
#define SUFFIX_CDMX "_CDMX.TXT"
#define SUFFIX_VDEC "_VDEC.TXT"
#define SUFFIX_ADEC "_ADEC.TXT"
#define SUFFIX_SCALER "_SCALER.TXT"
#endif


#ifdef CAMERA_PERF_MEASURE
#define SUFFIX_CAMSRC_0 "_CAMSRC_0.TXT"
#define SUFFIX_CAMSRC_1 "_CAMSRC_1.TXT"
#define SUFFIX_CAMSRC_2 "_CAMSRC_2.TXT"
#define SUFFIX_CAMSRC_3 "_CAMSRC_3.TXT"
#define SUFFIX_CAMAUDIOSRC "_CAMAUDIOSRC.TXT"
#define SUFFIX_CAMVENC_0 "_CAMVENC_0.TXT"
#define SUFFIX_CAMVENC_1 "_CAMVENC_1.TXT"
#define SUFFIX_CAMVENC_2 "_CAMVENC_2.TXT"
#define SUFFIX_CAMVENC_3 "_CAMVENC_3.TXT"
#define SUFFIX_CAMMUX_0 "_CAMMUX_0.TXT"
#define SUFFIX_CAMMUX_1 "_CAMMUX_1.TXT"
#define SUFFIX_CAMMUX_2 "_CAMMUX_2.TXT"
#define SUFFIX_CAMMUX_3 "_CAMMUX_3.TXT"

#define MEASURE_COUNT_FILEIO 2000000
#define MEASURE_COUNT_CAMSRC 200000
#define MEASURE_COUNT_CAMAUDIOSRC 200000
#define MEASURE_COUNT_CAMVENC 200000
#define MEASURE_COUNT_CAMMUX 200000
#endif //#ifdef CAMERA_PERF_MEASURE


#if (defined VIDEO_PERF_MEASURE) || (defined CAMERA_PERF_MEASURE)
#if 0
#define PERF_MEASURE_CHECK_GLOBAL_DEFINE(mm) static PerfMeaure_t (mm)
#define PERF_MEASURE_CHECK_IS_STARTED(mm) TCC_Performance_Measure_Check_Is_Start(&mm)

#define PERF_MEASURE_CHECK_OPEN(mm, filename, suffix) TCC_Performance_Measure_Check_Open(&mm, filename, suffix)
#define PERF_MEASURE_CHECK_OPEN(mm, filename, suffix, buffersize) TCC_Performance_Measure_Check_Open2(&mm, filename, suffix, buffersize)

#define PERF_MEASURE_CHECK_CLOSE(mm) TCC_Performance_Measure_Check_Close(&mm)

#define PERF_MEASURE_CHECK_OPEN_CLOSE(mm, filename, suffix) TCC_Performance_Measure_Check_Open_Close(&mm, filename, suffix)

#define PERF_MEASURE_CHECK_START(mm) TCC_Performance_Measure_Check_Start(&mm)

#define PERF_MEASURE_CHECK_END(mm) TCC_Performance_Measure_Check_End(&mm)
#define PERF_MEASURE_CHECK_END1(mm, val1) TCC_Performance_Measure_Check_End1(&mm, val1)
#define PERF_MEASURE_CHECK_END2(mm, val1, val2) TCC_Performance_Measure_Check_End2(&mm, val1, val2)
#else
#define PERF_MEASURE_CHECK_GLOBAL_DEFINE(mm) static PerfMeaure_t (mm)

#define PERF_MEASURE_CHECK_IS_STARTED(mm) ((mm).m_bStart == 1 ? 1 : 0)

#define PERF_MEASURE_CHECK_OPEN(mm, filename, suffix) \
			do { \
				if((mm).m_bStart == 0) \
				{ \
					sprintf((mm).m_strFilename, "%s%s", filename, suffix); \
					(mm).m_fp = TCC_fopen((mm).m_strFilename, "w"); \
					if((mm).m_fp != NULL) \
					{ \
						(mm).m_bStart = 1; \
						printf("Measurement start : %s\n", (mm).m_strFilename); \
					} \
					else \
					{ \
						printf("Failed to open PerfMeaseure, file open error : %s\n", (mm).m_strFilename); \
					} \
				} \
			} while(0)

#define PERF_MEASURE_CHECK_CLOSE(mm) \
			do { \
				if((mm).m_bStart == 1) \
				{ \
					(mm).m_bStart = 0; \
					TCC_fflush(stdout); \
					TCC_fclose((mm).m_fp); \
					sync(); \
					printf("Stop Measurement : %s\n", (mm).m_strFilename); \
				} \
			} while(0)

#define PERF_MEASURE_CHECK_OPEN_CLOSE(mm, filename, suffix) \
			do { \
				if((mm).m_bStart == 0) \
				{ \
					sprintf((mm).m_strFilename, "%s%s", filename, suffix); \
					(mm).m_fp = TCC_fopen((mm).m_strFilename, "w"); \
					if((mm).m_fp != NULL) \
					{ \
						(mm).m_bStart = 1; \
						printf("Measurement start: %s\n", (mm).m_strFilename); \
					} \
					else \
					{ \
						printf("Failed to open PerfMeaseure, file open error : %s\n", (mm).m_strFilename); \
					} \
				} \
				else \
				{ \
					(mm).m_bStart = 0; \
					TCC_fflush(stdout); \
					TCC_fclose((mm).m_fp); \
					sync(); \
					printf("Stop measurement : %s\n", (mm).m_strFilename); \
				} \
			} while(0)

#define PERF_MEASURE_CHECK_START(mm) \
			do { \
				struct timeval tv; \
				if((mm).m_bStart)	\
				{ \
					gettimeofday(&tv,NULL);	\
					(mm).m_ullBaseTime =  ((unsigned long long )tv.tv_sec)*1000000 + ((unsigned long long )tv.tv_usec); \
				} \
			} while(0)

#define PERF_MEASURE_CHECK_END(mm) \
			do { \
				if((mm).m_bStart && ((mm).m_ullBaseTime != 0)) \
				{ \
					struct timeval tv; \
					unsigned char buf[TCC_PERFORMANCE_MAX_STRING]; \
					gettimeofday(&tv,NULL); \
					(mm).m_ullNewTime =  ((unsigned long long )tv.tv_sec)*1000000 + ((unsigned long long )tv.tv_usec); \
					memset(buf, 0x00, TCC_PERFORMANCE_MAX_STRING); \
					sprintf(buf, "%lld \n", (mm).m_ullNewTime - (mm).m_ullBaseTime); \
					fprintf((mm).m_fp, buf, TCC_PERFORMANCE_MAX_STRING, 1); \
				} \
			} while(0)

#define PERF_MEASURE_CHECK_END1(mm, val1) \
			do { \
				if((mm).m_bStart && ((mm).m_ullBaseTime != 0)) \
				{ \
					struct timeval tv; \
					unsigned char buf[TCC_PERFORMANCE_MAX_STRING]; \
					gettimeofday(&tv,NULL); \
					(mm).m_ullNewTime =  ((unsigned long long )tv.tv_sec)*1000000 + ((unsigned long long )tv.tv_usec); \
					memset(buf, 0x00, TCC_PERFORMANCE_MAX_STRING); \
					sprintf(buf, "%lld, %d\n", (mm).m_ullNewTime - (mm).m_ullBaseTime, (val1)); \
					fprintf((mm).m_fp, buf, TCC_PERFORMANCE_MAX_STRING, 1); \
				} \
			} while(0)

#define PERF_MEASURE_CHECK_END2(mm, val1, val2) \
			do { \
				if((mm).m_bStart && ((mm).m_ullBaseTime != 0)) \
				{ \
					struct timeval tv; \
					unsigned char buf[TCC_PERFORMANCE_MAX_STRING]; \
					gettimeofday(&tv,NULL); \
					(mm).m_ullNewTime =  ((unsigned long long )tv.tv_sec)*1000000 + ((unsigned long long )tv.tv_usec); \
					memset(buf, 0x00, TCC_PERFORMANCE_MAX_STRING); \
					sprintf(buf, "%lld, %d, %d\n", (mm).m_ullNewTime - (mm).m_ullBaseTime, (val1), (val2)); \
					fprintf((mm).m_fp, buf, TCC_PERFORMANCE_MAX_STRING, 1); \
				} \
			} while(0)

//////////////////////////////////////////////////////////////////////////////////////////////////
//  TMEASUREMENT 2
//////////////////////////////////////////////////////////////////////////////////////////////////

#define PERF_MEASURE_CHECK_OPEN(mm, filename, suffix, buffersize) \
			do { \
				if((mm).m_bStart == 0) \
				{ \
					(mm).m_ulCount = 0; \
					(mm).m_nMaxCount = buffersize; \
					(mm).m_pBuffer = NULL; \
					(mm).m_pBuffer = (int)malloc(sizeof(int) * buffersize); \
					if((mm).m_pBuffer == NULL) \
						printf("Malloc error\n"); \
					sprintf((mm).m_strFilename, "%s"suffix, (filename)); \
					(mm).m_fp = fopen((mm).m_strFilename, "w"); \
					if((mm).m_fp != NULL) \
					{ \
						(mm).m_bStart = 1; \
						printf("Measurement start: %s\n", (mm).m_strFilename); \
					} \
					else \
					{ \
						printf("Failed to open PerfMeaseure, file open error : %s\n", (mm).m_strFilename); \
					} \
				} \
			} while(0)

#define PERF_MEASURE_CHECK_CLOSE(mm) \
			do { \
				if((mm).m_bStart == 1) \
				{ \
					int ii; \
					unsigned char buf[TCC_PERFORMANCE_MAX_STRING]; \
					(mm).m_bStart = 0; \
					printf("Stop measurement : %s, count:%d \n", (mm).m_strFilename, (mm).m_ulCount); \
					for(ii=0; ii<(mm).m_ulCount; ii++) \
					{ \
						memset(buf, 0x00, TCC_PERFORMANCE_MAX_STRING); \
						sprintf(buf, "%d \n", (mm).m_pBuffer[ii]); \
						fprintf((mm).m_fp, buf, TCC_PERFORMANCE_MAX_STRING, 1); \
					}\
					free((mm).m_pBuffer); \
					fclose((mm).m_fp); \
					sync(); \
				} \
			} while(0)

#define PERF_MEASURE_CHECK_START(mm) \
			do { \
				struct timeval tv; \
				if((mm).m_bStart)	\
				{ \
					gettimeofday(&tv,NULL);	\
					(mm).m_ullBaseTime =  ((unsigned long long )tv.tv_sec)*1000000 + ((unsigned long long )tv.tv_usec); \
				} \
			} while(0)

#define PERF_MEASURE_CHECK_END(mm) \
			do { \
				if((mm).m_bStart && ((mm).m_ullBaseTime != 0)) \
				{ \
					struct timeval tv; \
					gettimeofday(&tv,NULL); \
					(mm).m_ullNewTime =  ((unsigned long long )tv.tv_sec)*1000000 + ((unsigned long long )tv.tv_usec); \
					if((mm).m_nMaxCount > (mm).m_ulCount ) \
						(mm).m_pBuffer[(mm).m_ulCount++] = (int)((mm).m_ullNewTime - (mm).m_ullBaseTime); \
				} \
			} while(0)
#endif
#else //#if (defined VIDEO_PERF_MEASURE) || (defined CAMERA_PERF_MEASURE)
#define PERF_MEASURE_CHECK_GLOBAL_DEFINE(mm)
#define PERF_MEASURE_CHECK_IS_STARTED(mm) (0)
#define PERF_MEASURE_CHECK_OPEN(mm, filename, suffix)
#define PERF_MEASURE_CHECK_OPEN(mm, filename, suffix, buffersize)
#define PERF_MEASURE_CHECK_CLOSE(mm)
#define PERF_MEASURE_CHECK_OPEN_CLOSE(mm, filename, suffix)
#define PERF_MEASURE_CHECK_START(mm)
#define PERF_MEASURE_CHECK_END(mm)
#define PERF_MEASURE_CHECK_END1(mm, val1)
#define PERF_MEASURE_CHECK_END2(mm, val1, val2)
#endif //#if (defined VIDEO_PERF_MEASURE) || (defined CAMERA_PERF_MEASURE)

/******************************************************************************
* Enum
******************************************************************************/


/******************************************************************************
* typedefs & structure
******************************************************************************/
typedef struct tagPerfMeasure
{
	FILE *m_fp;
	int m_bStart;
	unsigned char m_strFilename[TCC_PERFORMANCE_MAX_FILENAME_LEN];
	unsigned long m_ulCount;
	unsigned long long m_ullBaseTime;
	unsigned long long m_ullNewTime;
	int *m_pBuffer;
	int m_nMaxCount;
} PerfMeaure_t;


/******************************************************************************
* globals
******************************************************************************/


/******************************************************************************
* locals
******************************************************************************/


/******************************************************************************
* declarations
******************************************************************************/

#endif //_TCC_PERFORMANCE_H__
