/****************************************************************************
*   FileName    : omx_tcc_thread.c
*   Description : 
****************************************************************************
*
*   TCC Version 1.0
*   Copyright (c) Telechips, Inc.
*   ALL RIGHTS RESERVED
*
****************************************************************************/

/****************************************************************************

Revision History

****************************************************************************

****************************************************************************/
#ifdef HAVE_ANDROID_OS
#define LOG_TAG	"TCC_THREAD"
#include <utils/Log.h>
#else
#include <stdio.h>
#define	LOGD		printf
#define	LOGE		printf
#define	LOGI		printf
#define	LOGW		printf
#endif


#include <pthread.h>
#include <stdlib.h>
#include "omx_tcc_thread.h"

//#define	 NOPRIORITY_SUPPORT
#define	 DBG_PRINTF(msg...)		//LOGD(msg)

typedef struct TCCTHREAD_COUSTOM_DEF
{
	unsigned char *pucname;
	int sched_type;
	E_TCCTHREAD_PRIORITY epriority;
} ST_TCCTHREAD_COUSTOM_DEF;


ST_TCCTHREAD_COUSTOM_DEF stTCCThreadTable[] = 
{
	{	"default"      ,					SCHED_RR,	LOW_PRIORITY_11		},		
	{	"OMX.tcc.broadcast.isdbt",			SCHED_RR,	LOW_PRIORITY_11		},	
	{	NULL,								0,			0 					},
};

int TCCTHREAD_Create(void *pHandle, pThreadFunc Func, char *pcName, E_TCCTHREAD_PRIORITY ePriority, void *arg)
{
	int i, ret = 0;
	int iSchedType, iPriorityOffset, iMaxPriority;
	struct sched_param param;
	pthread_attr_t p_tattr;
	
#ifdef		NOPRIORITY_SUPPORT
	return pthread_create(pHandle, NULL, Func,  arg);
#else

	iSchedType = -1;
	if( ePriority >= PREDEF_PRIORITY )
		ePriority = HIGH_PRIORITY_9;
	
	if(pcName)
	{		
		for( i=0; stTCCThreadTable[i].pucname != NULL; i++ )
		{
			if( !strcmp(stTCCThreadTable[i].pucname, pcName) )
			{				
				iPriorityOffset = stTCCThreadTable[i].epriority;	
				iSchedType = stTCCThreadTable[i].sched_type;	
				DBG_PRINTF("%s %d %d", pcName, ePriority, iSchedType);
				break;
			}
		}
	}
	
	if(iSchedType == -1)
	{
		if(ePriority >= LOW_PRIORITY_0 )
		{
			#if 1
			iSchedType = SCHED_OTHER;		
			//iPriorityOffset = ePriority-LOW_PRIORITY_0;		
			iPriorityOffset = 0;
			#else
			iSchedType = SCHED_RR;
			iPriorityOffset = ePriority;
			#endif
		}
		else
		{
			iSchedType = SCHED_RR;
			iPriorityOffset = ePriority;
		}	
	}	
	/* initialized with default attributes */
	ret |= pthread_attr_init (&p_tattr);
	
	//ret |= pthread_attr_setinheritsched(p_tattr, PTHREAD_EXPLICIT_SCHED);  
	ret |= pthread_attr_setscope(&p_tattr, PTHREAD_SCOPE_SYSTEM);	
	ret |= pthread_attr_setschedpolicy(&p_tattr, iSchedType);
	/* safe to get existing scheduling param */
	ret |= pthread_attr_getschedparam (&p_tattr, &param);
	
	/* set the priority; others are unchanged */
	iMaxPriority = sched_get_priority_max(iSchedType);
	if( iMaxPriority > iPriorityOffset )
		param.sched_priority = iMaxPriority - iPriorityOffset;
	else
		param.sched_priority = iMaxPriority;
	//LOGE("MAX %d,Offset %d, Type %d", sched_get_priority_max(iSchedType), iPriorityOffset, iSchedType);
	/* setting the new scheduling param */
	ret |= pthread_attr_setschedparam (&p_tattr, &param);
	if(ret != 0)
	{
		DBG_PRINTF("%s::%d thread(%s) error %d", __func__,__LINE__, pcName, ret);
		return ret;
	}
	
	ret = pthread_create(pHandle, &p_tattr, Func,  arg);	
	if(ret != 0)
	{
		DBG_PRINTF("%s::%d thread(%s) error %d", __func__,__LINE__, pcName, ret);
		return ret;
	}
	
	DBG_PRINTF("%s::%s Success!! Priority(%d)Type(%d)", __func__, pcName, param.sched_priority, iSchedType);
	return ret;
#endif	
}

int TCCTHREAD_Join(void *pHandle, void **pThreadRet)
{
	return pthread_join(pHandle, pThreadRet);
}
