/*
  @file component/openmax_wfdadec_component/omx_wfdadec_component.c

  OpenMAX audio decoder component. This component implements AAC / LPCM audio decoder.

  Copyright (C) 2008  STMicroelectronics
  Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
  Copyright (C) 2015 Telechips Inc.

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  $Date: 2009/03/10 13:33:28 $
  Revision $Rev: 554 $
  Author $Author: B060934 $
  revised by B060955.
*/

#include <omxcore.h>
#include <omx_base_audio_port.h>

#include <omx_wfdadec_component.h>
#include <tcc_audio_common.h>
#include <tcc_video_common.h>
#include <OMX_TCC_Index.h>

#include <dlfcn.h>


typedef cdk_core_t*(*tcc_cdk_get_CoreAddr)(void);
typedef int(*tcc_cdk_get_video_codec_type)(int);
typedef int(*tcc_cdk_get_audio_codec_type)(int);
typedef cdk_result_t(*tcc_cdk_adec_decode)( cdk_core_t* , cdmx_output_t *, int ,int , unsigned char *,unsigned int *,int  ,int);
typedef cdk_result_t(*tcc_cdk_init_adec)( cdk_core_t* , cdmx_info_t *, int , int );
typedef cdk_result_t(*tcc_cdk_close_adec)( cdk_core_t*);

static tcc_cdk_get_CoreAddr sym_tcc_cdk_get_core_addr = NULL;
static tcc_cdk_get_audio_codec_type sym_tcc_cdk_get_audio_codec_type = NULL;
static tcc_cdk_get_video_codec_type sym_tcc_cdk_get_video_codec_type = NULL;
static tcc_cdk_init_adec sym_tcc_cdk_init_adec = NULL;
static tcc_cdk_close_adec sym_tcc_cdk_close_adec = NULL;
static tcc_cdk_adec_decode sym_tcc_cdk_adec_decode = NULL;

static void tcc_cdk_adec_symbol_loading(void)
{
    void *pModule = NULL;

    pModule = dlopen(LIBTCCCDKCORE_SO, RTLD_LAZY); 

    if(pModule == NULL)
        printf("Could not find libcdkcore.so.");

    sym_tcc_cdk_get_core_addr = dlsym(pModule,"TCC_CDK_GetCoreAddr");
    if(sym_tcc_cdk_get_core_addr == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_get_core_addr \n");

    sym_tcc_cdk_get_audio_codec_type = dlsym(pModule,"get_audio_codec_type");
    if(sym_tcc_cdk_get_audio_codec_type == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_get_audio_codec_type \n");

    sym_tcc_cdk_get_video_codec_type = dlsym(pModule,"get_video_codec_type");
    if(sym_tcc_cdk_get_video_codec_type == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_get_video_codec_type \n");

    sym_tcc_cdk_init_adec = dlsym(pModule,"cdk_adec_init");
    if(sym_tcc_cdk_init_adec == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_init_adec \n");

    sym_tcc_cdk_adec_decode = dlsym(pModule,"cdk_adec_decode2");
    if(sym_tcc_cdk_adec_decode == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_adec_decode \n");

    sym_tcc_cdk_close_adec = dlsym(pModule,"cdk_adec_close");
    if(sym_tcc_cdk_close_adec == NULL)
        printf("Could not find Symbol.. cdk_adec_close \n");

    dlclose(pModule);
}

void omx_wfdadec_component_BufferMgmtCallback(OMX_COMPONENTTYPE *openmaxStandComp, OMX_BUFFERHEADERTYPE* inputbuffer, OMX_BUFFERHEADERTYPE* outputbuffer)
	{
		omx_wfdadec_component_PrivateType* omx_wfdadec_component_Private = openmaxStandComp->pComponentPrivate;  
		int ret;
		mblk_t	*pAudiostream;
		OMX_U8 *input_src_ptr;
		
		outputbuffer->nFilledLen = 0;
		outputbuffer->nOffset=0;
	
		if(omx_wfdadec_component_Private->fast_stop)
		{
			inputbuffer->nFilledLen = 0;
			outputbuffer->nFilledLen = 0;
			usleep(1000);
			return;
		}
	
		if(omx_wfdadec_component_Private->open_audido_codec == OMX_FALSE)
		{
			omx_wfdadec_component_LibInit(openmaxStandComp);
			omx_wfdadec_component_Private->open_audido_codec = OMX_TRUE;
		}
	
		pthread_mutex_lock(&omx_wfdadec_component_Private->audiodec_off_mutex);

		input_src_ptr = (OMX_U8 *) (inputbuffer->pBuffer);	
		pAudiostream = (mblk_t	*)((*(input_src_ptr+3)<<24) | (*(input_src_ptr+2)<<16) | (*(input_src_ptr+1)<<8) | *(input_src_ptr)); 

		unsigned char* audio_stream = pAudiostream->b_rptr;
		unsigned int audio_len = pAudiostream->frame_size;

		if(audio_len > 0)
		{
			omx_wfdadec_component_Private->CdmxOut.m_pPacketData = audio_stream;
			omx_wfdadec_component_Private->CdmxOut.m_iPacketSize = audio_len;
	
			outputbuffer->nTimeStamp = inputbuffer->nTimeStamp;
	
			ret = sym_tcc_cdk_adec_decode(omx_wfdadec_component_Private->pCdk, 
								   &omx_wfdadec_component_Private->CdmxOut,
								   omx_wfdadec_component_Private->audio_codec_type,
								   0,
								   outputbuffer->pBuffer,
								   &outputbuffer->nFilledLen, 
								   0, 
								   0);
				
			if(ret >= 0)
			{
				inputbuffer->nOffset=0;
				inputbuffer->nFilledLen = 0;
				outputbuffer->audio_codec_type = omx_wfdadec_component_Private->audio_codec_type;	
			}
			else /* ret < 0*/
			{	
				inputbuffer->nFilledLen = 0;
				outputbuffer->nFilledLen = 0;
			}
		}

		freemsg((mblk_t *)pAudiostream);
	
		pthread_mutex_unlock(&omx_wfdadec_component_Private->audiodec_off_mutex);
	
		if(omx_wfdadec_component_Private->fast_stop)
		{
			inputbuffer->nFilledLen = 0;
			outputbuffer->nFilledLen = 0;
		}
	}


/** setting configurations */
OMX_ERRORTYPE omx_wfdadec_component_SetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_IN  OMX_PTR pComponentConfigStructure)
{

  OMX_TIME_CONFIG_TIMESTAMPTYPE* sTimeStamp;
  OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *)hComponent;
  omx_wfdadec_component_PrivateType* omx_wfdadec_component_Private = openmaxStandComp->pComponentPrivate;
  OMX_ERRORTYPE err = OMX_ErrorNone;
  omx_base_audio_PortType *pPort;

  switch (nIndex)
  {
	case OMX_IndexVendorConfigFastStop:
	{
		OMX_AUDIO_CONFIG_INFOTYPE *info;
		info = (OMX_AUDIO_CONFIG_INFOTYPE*)pComponentConfigStructure;
		omx_wfdadec_component_Private->fast_stop = info->faststop;
	}
	break;
		
	default: // delegate to superclass
	  return omx_base_component_SetConfig(hComponent, nIndex, pComponentConfigStructure);
  }
  return OMX_ErrorNone;
}

/** setting configurations */
OMX_ERRORTYPE omx_wfdadec_component_GetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_IN  OMX_PTR pComponentConfigStructure)
{

	OMX_VENDOR_EXTRADATATYPE sExtraData;
	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *)hComponent;
	omx_wfdadec_component_PrivateType* omx_wfdadec_component_Private = openmaxStandComp->pComponentPrivate;
	OMX_AUDIO_SPECTRUM_INFOTYPE *power;
	OMX_AUDIO_ENERGY_VOLUME_INFOTYPE *energyvolume;
	OMX_TIME_CONFIG_TIMESTAMPTYPE *Current_Time;
	int i;
	
	switch (nIndex)
	{
		default: // delegate to superclass
			return omx_base_component_GetConfig(hComponent, nIndex, pComponentConfigStructure);
	}
	return OMX_ErrorNone;
}

/** this function sets the parameter values regarding audio format & index */
OMX_ERRORTYPE omx_wfdadec_component_SetParameter(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nParamIndex,
  OMX_IN  OMX_PTR ComponentParameterStructure)
{
	OMX_ERRORTYPE err = OMX_ErrorNone;
	OMX_AUDIO_PARAM_PORTFORMATTYPE *pAudioPortFormat;
	OMX_AUDIO_PARAM_PCMMODETYPE* pAudioPcmMode;
	OMX_PARAM_COMPONENTROLETYPE * pComponentRole;
	OMX_AUDIO_CONFIG_INFOTYPE *info;

	OMX_U32 portIndex;
	OMX_U32 nFileNameLength;

	/* Check which structure we are being fed and make control its header */
	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *)hComponent;
	omx_wfdadec_component_PrivateType* omx_wfdadec_component_Private = openmaxStandComp->pComponentPrivate;
	omx_base_audio_PortType *port;

	if (ComponentParameterStructure == NULL) 
	{
		return OMX_ErrorBadParameter;
	}

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "   Setting parameter %i\n", nParamIndex);

	switch(nParamIndex) 
	{
		case OMX_IndexParamAudioPortFormat:
			pAudioPortFormat = (OMX_AUDIO_PARAM_PORTFORMATTYPE*)ComponentParameterStructure;
			portIndex = pAudioPortFormat->nPortIndex;
			/*Check Structure Header and verify component state*/
			err = omx_base_component_ParameterSanityCheck(hComponent, portIndex, pAudioPortFormat, sizeof(OMX_AUDIO_PARAM_PORTFORMATTYPE));
			if(err!=OMX_ErrorNone)
			{ 
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Parameter Check Error=%x\n",__func__,err); 
				break;
			}
			if (portIndex <= 1) 
			{
				port = (omx_base_audio_PortType *) omx_wfdadec_component_Private->ports[portIndex];
				memcpy(&port->sAudioParam, pAudioPortFormat, sizeof(OMX_AUDIO_PARAM_PORTFORMATTYPE));
			} 
			else
			{
				return OMX_ErrorBadPortIndex;
			}
		break;

		case OMX_IndexParamAudioPcm:
			pAudioPcmMode = (OMX_AUDIO_PARAM_PCMMODETYPE*)ComponentParameterStructure;
			portIndex = pAudioPcmMode->nPortIndex;
			/*Check Structure Header and verify component state*/
			err = omx_base_component_ParameterSanityCheck(hComponent, portIndex, pAudioPcmMode, sizeof(OMX_AUDIO_PARAM_PCMMODETYPE));
			if(err!=OMX_ErrorNone)
			{ 
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Parameter Check Error=%x\n",__func__,err); 
				break;
			}
			memcpy(&omx_wfdadec_component_Private->pAudioPcmMode_input, pAudioPcmMode, sizeof(OMX_AUDIO_PARAM_PCMMODETYPE));		  
		break;

		case OMX_IndexParamStandardComponentRole:
			pComponentRole = (OMX_PARAM_COMPONENTROLETYPE*)ComponentParameterStructure;

			if (!strcmp( (char*) pComponentRole->cRole, AUDIO_DEC_PCM_ROLE)) 
			{
				omx_wfdadec_component_Private->audio_coding_type = OMX_AUDIO_CodingPCM;
			}  
			else
			{
				return OMX_ErrorBadParameter;
			}
		break;

		case OMX_IndexVendorParamMediaInfo:
			info = (OMX_AUDIO_CONFIG_INFOTYPE*) ComponentParameterStructure;
			break;

		case OMX_IndexVendorParamCDKDMXInfo:
			memcpy(&omx_wfdadec_component_Private->sCDmxInfo,ComponentParameterStructure,sizeof(cdmx_info_t));	
			omx_wfdadec_component_Private->audio_codec_type = sym_tcc_cdk_get_audio_codec_type( omx_wfdadec_component_Private->sCDmxInfo.m_sAudioInfo.m_iFormatId );
			omx_wfdadec_component_Private->video_codec_type = sym_tcc_cdk_get_video_codec_type(omx_wfdadec_component_Private->sCDmxInfo.m_sVideoInfo.m_iFourCC);
			break;
			
		case OMX_IndexVendorParamCDKContainerType:
			info = (OMX_AUDIO_CONFIG_INFOTYPE*) ComponentParameterStructure;
			omx_wfdadec_component_Private->container_type =  info->container_type;//info->codec_type;
			break;

		default: /*Call the base component function*/
			return omx_base_component_SetParameter(hComponent, nParamIndex, ComponentParameterStructure);

	}

	return err;
}  

/** this function gets the parameters regarding audio formats and index */
OMX_ERRORTYPE omx_wfdadec_component_GetParameter(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nParamIndex,
  OMX_INOUT OMX_PTR ComponentParameterStructure)
{

	OMX_AUDIO_PARAM_PORTFORMATTYPE *pAudioPortFormat;  
	OMX_AUDIO_PARAM_PCMMODETYPE *pAudioPcmMode;
	OMX_PARAM_COMPONENTROLETYPE * pComponentRole;

	omx_base_audio_PortType *port;
	OMX_ERRORTYPE err = OMX_ErrorNone;

	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *)hComponent;
	omx_wfdadec_component_PrivateType* omx_wfdadec_component_Private = openmaxStandComp->pComponentPrivate;

	if (ComponentParameterStructure == NULL)
	{
		return OMX_ErrorBadParameter;
	}
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "   Getting parameter %i\n", nParamIndex);
	/* Check which structure we are being fed and fill its header */

	switch(nParamIndex) 
	{
		case OMX_IndexParamAudioInit:
			if ((err = checkHeader(ComponentParameterStructure, sizeof(OMX_PORT_PARAM_TYPE))) != OMX_ErrorNone) 
			{ 
			  break;
			}
			memcpy(ComponentParameterStructure, &omx_wfdadec_component_Private->sPortTypesParam, sizeof(OMX_PORT_PARAM_TYPE));
		break;	  

		case OMX_IndexParamAudioPortFormat:
			pAudioPortFormat = (OMX_AUDIO_PARAM_PORTFORMATTYPE*)ComponentParameterStructure;
			if ((err = checkHeader(ComponentParameterStructure, sizeof(OMX_AUDIO_PARAM_PORTFORMATTYPE))) != OMX_ErrorNone) 
			{ 
				break;
			}
			if (pAudioPortFormat->nPortIndex <= 1) 
			{
				port = (omx_base_audio_PortType *)omx_wfdadec_component_Private->ports[pAudioPortFormat->nPortIndex];
				memcpy(pAudioPortFormat, &port->sAudioParam, sizeof(OMX_AUDIO_PARAM_PORTFORMATTYPE));
			} 
			else 
			{
				return OMX_ErrorBadPortIndex;
			}
		break;	  

		case OMX_IndexParamAudioPcm:
			pAudioPcmMode = (OMX_AUDIO_PARAM_PCMMODETYPE*)ComponentParameterStructure;
			if ((err = checkHeader(ComponentParameterStructure, sizeof(OMX_AUDIO_PARAM_PCMMODETYPE))) != OMX_ErrorNone) 
			{ 
				break;
			}
			if (pAudioPcmMode->nPortIndex > 1) 
			{
				return OMX_ErrorBadPortIndex;
			}
			memcpy(pAudioPcmMode, &omx_wfdadec_component_Private->pAudioPcmMode_input, sizeof(OMX_AUDIO_PARAM_PCMMODETYPE));
		break;

		case OMX_IndexParamStandardComponentRole:
			pComponentRole = (OMX_PARAM_COMPONENTROLETYPE*)ComponentParameterStructure;
			if ((err = checkHeader(ComponentParameterStructure, sizeof(OMX_PARAM_COMPONENTROLETYPE))) != OMX_ErrorNone) 
			{ 
				break;
			}
			if (omx_wfdadec_component_Private->audio_coding_type == OMX_AUDIO_CodingPCM) 
			{
				strcpy( (char*) pComponentRole->cRole, AUDIO_DEC_PCM_ROLE);
			}
			else
			{
				strcpy( (char*) pComponentRole->cRole,"\0");;
			}
		break;

		default: /*Call the base component function*/
			return omx_base_component_GetParameter(hComponent, nParamIndex, ComponentParameterStructure);

	}

	return OMX_ErrorNone;

}

OMX_ERRORTYPE omx_wfdadec_decoder_MessageHandler(OMX_COMPONENTTYPE* openmaxStandComp, internalRequestMessageType *message)	
{

	omx_wfdadec_component_PrivateType* omx_wfdadec_component_Private = (omx_wfdadec_component_PrivateType*)openmaxStandComp->pComponentPrivate;  
	OMX_ERRORTYPE err;
	OMX_STATETYPE eCurrentState = omx_wfdadec_component_Private->state;
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "In %s\n", __func__);

	if (message->messageType == OMX_CommandStateSet)
	{
		if ((message->messageParam == OMX_StateIdle) && (eCurrentState == OMX_StateLoaded)) 
		{
			omx_wfdadec_component_buffer_Init(openmaxStandComp);
		}
		else if ((message->messageParam == OMX_StateLoaded) && (eCurrentState == OMX_StateIdle)) 
		{
			omx_wfdadec_component_buffer_Deinit(openmaxStandComp);
			omx_wfdadec_component_LibDeinit(openmaxStandComp);
		}
		else if ((message->messageParam == OMX_StateIdle) && (eCurrentState == OMX_StateExecuting))
		{ 
		
		}
	}

	/** Execute the base message handling */
	err = omx_base_component_MessageHandler(openmaxStandComp, message);

	return err;  
}


OMX_ERRORTYPE omx_wfdadec_component_Constructor(OMX_COMPONENTTYPE *openmaxStandComp, OMX_STRING cComponentName) 
{

	OMX_ERRORTYPE err = OMX_ErrorNone;	
	omx_wfdadec_component_PrivateType* omx_wfdadec_component_Private;
	omx_base_audio_PortType *inPort,*outPort;
	OMX_U32 i;
	
	if (!openmaxStandComp->pComponentPrivate) 
	{
		openmaxStandComp->pComponentPrivate = TCC_calloc(1, sizeof(omx_wfdadec_component_PrivateType));

		if(openmaxStandComp->pComponentPrivate==NULL)  
		{
			return OMX_ErrorInsufficientResources;
		}
	} 
	else 
	{
	
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME, "In %s, Error Component %x Already Allocated\n", 
				  __func__, (int)openmaxStandComp->pComponentPrivate);
	}

	omx_wfdadec_component_Private = openmaxStandComp->pComponentPrivate;
	omx_wfdadec_component_Private->ports = NULL;

	/** we could create our own port structures here
	* fixme maybe the base class could use a "port factory" function pointer?  
	*/
	err = omx_base_filter_Constructor(openmaxStandComp, cComponentName);

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "constructor of cdk audio decoder component is called\n");

	/** Domain specific section for the ports. */  
	/** first we set the parameter common to both formats
	/** parameters related to input port which does not depend upon input audio format	  */
	/** Allocate Ports and call port constructor. */  

	omx_wfdadec_component_Private->sPortTypesParam[OMX_PortDomainAudio].nStartPortNumber = 0;
	omx_wfdadec_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts = 2;


	if (omx_wfdadec_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts && !omx_wfdadec_component_Private->ports) 
	{
		omx_wfdadec_component_Private->ports = TCC_calloc(omx_wfdadec_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts, sizeof(omx_base_PortType *));
		if (!omx_wfdadec_component_Private->ports) 
		{
			return OMX_ErrorInsufficientResources;
		}
		for (i=0; i < omx_wfdadec_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts; i++) 
		{
			  omx_wfdadec_component_Private->ports[i] = TCC_calloc(1, sizeof(omx_base_audio_PortType));
			  if (!omx_wfdadec_component_Private->ports[i]) 
			  {
					return OMX_ErrorInsufficientResources;
			  }
		}
	}

	base_audio_port_Constructor(openmaxStandComp, &omx_wfdadec_component_Private->ports[0], 0, OMX_TRUE); // outport..
	base_audio_port_Constructor(openmaxStandComp, &omx_wfdadec_component_Private->ports[1], 1, OMX_FALSE); // outport..

	/** parameters related to input port */
	inPort = (omx_base_audio_PortType *) omx_wfdadec_component_Private->ports[OMX_BASE_FILTER_INPUTPORT_INDEX];

	inPort->sPortParam.nBufferSize = AUDIO_DEC_IN_BUFFER_SIZE; //DEFAULT_OUT_BUFFER_SIZE;
	strcpy(inPort->sPortParam.format.audio.cMIMEType, "audio/pcm");
	inPort->sPortParam.format.audio.eEncoding = OMX_AUDIO_CodingPCM;
	inPort->sPortParam.nBufferCountMin = 4;
	inPort->sPortParam.nBufferCountActual = 4;
	inPort->sAudioParam.eEncoding = OMX_AUDIO_CodingPCM;

	setHeader(&omx_wfdadec_component_Private->pAudioPcmMode_input, sizeof(OMX_AUDIO_PARAM_PCMMODETYPE));

	omx_wfdadec_component_Private->pAudioPcmMode_input.nPortIndex = 0;
	omx_wfdadec_component_Private->pAudioPcmMode_input.nChannels = 2;
	omx_wfdadec_component_Private->pAudioPcmMode_input.eNumData = OMX_NumericalDataSigned;
	omx_wfdadec_component_Private->pAudioPcmMode_input.eEndian = OMX_EndianLittle;
	omx_wfdadec_component_Private->pAudioPcmMode_input.bInterleaved = OMX_TRUE;
	omx_wfdadec_component_Private->pAudioPcmMode_input.nBitPerSample = 16;
	omx_wfdadec_component_Private->pAudioPcmMode_input.nSamplingRate = 48000;
	omx_wfdadec_component_Private->pAudioPcmMode_input.ePCMMode = OMX_AUDIO_PCMModeLinear;
	omx_wfdadec_component_Private->pAudioPcmMode_input.eChannelMapping[0] = OMX_AUDIO_ChannelLF;
	omx_wfdadec_component_Private->pAudioPcmMode_input.eChannelMapping[1] = OMX_AUDIO_ChannelRF;

	/** parameters related to output port */
	outPort = (omx_base_audio_PortType *) omx_wfdadec_component_Private->ports[OMX_BASE_FILTER_OUTPUTPORT_INDEX];
	outPort->sPortParam.format.audio.eEncoding = OMX_AUDIO_CodingAAC;
	outPort->sPortParam.nBufferSize = AUDIO_DEC_OUT_BUFFER_SIZE; //DEFAULT_OUT_BUFFER_SIZE;
	outPort->sPortParam.nBufferCountMin = 4;
	outPort->sPortParam.nBufferCountActual = 4;
	outPort->sAudioParam.eEncoding = OMX_AUDIO_CodingPCM;
	
	/** settings of output port audio format - pcm */
	setHeader(&omx_wfdadec_component_Private->pAudioPcmMode_output, sizeof(OMX_AUDIO_PARAM_PCMMODETYPE));
	omx_wfdadec_component_Private->pAudioPcmMode_output.nPortIndex = 1;
	omx_wfdadec_component_Private->pAudioPcmMode_output.nChannels = 2;
	omx_wfdadec_component_Private->pAudioPcmMode_output.eNumData = OMX_NumericalDataSigned;
	omx_wfdadec_component_Private->pAudioPcmMode_output.eEndian = OMX_EndianLittle;
	omx_wfdadec_component_Private->pAudioPcmMode_output.bInterleaved = OMX_TRUE;
	omx_wfdadec_component_Private->pAudioPcmMode_output.nBitPerSample = 16;
	omx_wfdadec_component_Private->pAudioPcmMode_output.nSamplingRate = 48000;
	omx_wfdadec_component_Private->pAudioPcmMode_output.ePCMMode = OMX_AUDIO_PCMModeLinear;
	omx_wfdadec_component_Private->pAudioPcmMode_output.eChannelMapping[0] = OMX_AUDIO_ChannelLF;
	omx_wfdadec_component_Private->pAudioPcmMode_output.eChannelMapping[1] = OMX_AUDIO_ChannelRF;
	omx_wfdadec_component_Private->audio_coding_type = OMX_AUDIO_CodingUnused;

	/** initialise the semaphore to be used for mad decoder access synchronization */

	/** general configuration irrespective of any audio formats */
	/**  setting values of other fields of omx_maddec_component_Private structure */
	
	omx_wfdadec_component_Private->BufferMgmtCallback = omx_wfdadec_component_BufferMgmtCallback;
	omx_wfdadec_component_Private->messageHandler = omx_wfdadec_decoder_MessageHandler;
	omx_wfdadec_component_Private->destructor = omx_wfdadec_component_Destructor;
	openmaxStandComp->SetParameter = omx_wfdadec_component_SetParameter;
	openmaxStandComp->GetParameter = omx_wfdadec_component_GetParameter;
	openmaxStandComp->SetConfig= omx_wfdadec_component_SetConfig;
	openmaxStandComp->GetConfig= omx_wfdadec_component_GetConfig;
			
	openmaxStandComp->GetExtensionIndex = omx_wfdadec_component_GetExtensionIndex;

	omx_wfdadec_component_Private->pcm_out_ptr = TCC_calloc(1,sizeof(char)*AUDIO_PCM_OUTPUT_MAX_LENGTH);
	memset(&omx_wfdadec_component_Private->sCDmxInfo,0,sizeof(cdmx_info_t));
	
	omx_wfdadec_component_Private->fast_stop = OMX_FALSE;

    tcc_cdk_adec_symbol_loading();

	pthread_mutex_init(&omx_wfdadec_component_Private->audiodec_off_mutex, NULL);
	return err;

}


OMX_ERRORTYPE omx_wfdadec_component_GetExtensionIndex(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_STRING cParameterName,
  OMX_OUT OMX_INDEXTYPE* pIndexType)
{	 
	if(strcmp(cParameterName,TCC_AUDIO_FILE_OPEN_STRING) == 0) {
		*pIndexType = OMX_IndexVendorParamFileOpen;  
	} else if(strcmp(cParameterName,TCC_AUDIO_POWERSPECTUM_STRING) == 0){
		*pIndexType = OMX_IndexVendorConfigPowerSpectrum;  
	} else if(strcmp(cParameterName,TCC_AUDIO_MEDIA_INFO_STRING) == 0){
		*pIndexType = OMX_IndexVendorParamMediaInfo;
	}else if(strcmp(cParameterName,TCC_AUDIO_ENERGYVOLUME_STRING) == 0){
		*pIndexType = OMX_IndexVendorConfigEnergyVolume;
	}else if( strcmp(cParameterName,TCC_AUDIO_DEC_SET_DYNAMICRANGE_STRING)== 0){
		*pIndexType = OMX_IndexVendorConfigDynamic_Range;
	}else if( strcmp(cParameterName,TCC_AUDIO_DEC_SET_OUTPUT_STRING)== 0){
		*pIndexType = OMX_IndexVendorParamSetOutputMode;
	}else{
		return OMX_ErrorBadParameter;
	}

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME,"In  %s \n",__func__);
	return OMX_ErrorNone;  
}


/** The destructor */
OMX_ERRORTYPE omx_wfdadec_component_Destructor(OMX_COMPONENTTYPE *openmaxStandComp) 
{

	omx_wfdadec_component_PrivateType* omx_wfdadec_component_Private = openmaxStandComp->pComponentPrivate;
	OMX_U32 i;
	
	/* frees port/s */
	if (omx_wfdadec_component_Private->ports) 
	{
		for (i=0; i < omx_wfdadec_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts; i++) 
		{
			if(omx_wfdadec_component_Private->ports[i])
				omx_wfdadec_component_Private->ports[i]->PortDestructor(omx_wfdadec_component_Private->ports[i]);
		}

		TCC_free(omx_wfdadec_component_Private->ports);
		omx_wfdadec_component_Private->ports=NULL;
	}

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME, "Destructor of wav decoder component is called\n");


	if(omx_wfdadec_component_Private->pcm_out_ptr)
	{
		TCC_free(omx_wfdadec_component_Private->pcm_out_ptr);
		omx_wfdadec_component_Private->pcm_out_ptr = NULL;
	}

	pthread_mutex_destroy(&omx_wfdadec_component_Private->audiodec_off_mutex);

	omx_base_filter_Destructor(openmaxStandComp);

	//omx_cdkadec_component_buffer_Deinit(openmaxStandComp);

	return OMX_ErrorNone;

}


/** The Initialization function  */
OMX_ERRORTYPE omx_wfdadec_component_Init(OMX_COMPONENTTYPE *openmaxStandComp)
{
	return omx_wfdadec_component_Constructor(openmaxStandComp,WFDSINK_ADEC_NAME);
}

/** The Deinitialization function  */
OMX_ERRORTYPE omx_wfdadec_component_Deinit(OMX_COMPONENTTYPE *openmaxStandComp)
{
	return omx_wfdadec_component_Destructor(openmaxStandComp);
}

OMX_ERRORTYPE omx_wfdadec_component_LibInit(OMX_COMPONENTTYPE *openmaxStandComp) {

	omx_wfdadec_component_PrivateType* omx_wfdadec_component_Private = openmaxStandComp->pComponentPrivate;
	OMX_ERRORTYPE eError = OMX_ErrorNone;

	int ret;
	int iAdecType;
	cdk_func_t* adec_func;

	cdk_core_t * pCdk;
	cdmx_info_t *pCdmxInfo;
	
	omx_wfdadec_component_Private->pCdk = sym_tcc_cdk_get_core_addr();

	omx_wfdadec_component_Private->audio_codec_type = sym_tcc_cdk_get_audio_codec_type( omx_wfdadec_component_Private->sCDmxInfo.m_sAudioInfo.m_iFormatId );
	ret = sym_tcc_cdk_init_adec(omx_wfdadec_component_Private->pCdk, &omx_wfdadec_component_Private->sCDmxInfo, omx_wfdadec_component_Private->audio_codec_type, omx_wfdadec_component_Private->container_type );

	if(ret != CDK_ERR_NONE)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR,"cdk audio init fail!!!! %d\n",ret);
		return ret;
	}

	return eError;
}

OMX_ERRORTYPE  omx_wfdadec_component_LibDeinit(OMX_COMPONENTTYPE *openmaxStandComp) {

	omx_wfdadec_component_PrivateType* omx_wfdadec_component_Private = openmaxStandComp->pComponentPrivate;
	OMX_ERRORTYPE eError = OMX_ErrorNone;

	if(omx_wfdadec_component_Private->open_audido_codec)
	sym_tcc_cdk_close_adec(	omx_wfdadec_component_Private->pCdk );
	return eError;
}	
/** The Initialization function  */
OMX_ERRORTYPE omx_wfdadec_component_buffer_Init(OMX_COMPONENTTYPE *openmaxStandComp)
{

  omx_wfdadec_component_PrivateType* omx_wfdadec_component_Private = openmaxStandComp->pComponentPrivate;
  OMX_ERRORTYPE err = OMX_ErrorNone;

  /** initializing omx_maddec_component_Private->temporary_buffer with 2k memory space*/
  omx_wfdadec_component_Private->temporary_buffer = TCC_malloc(sizeof(OMX_BUFFERHEADERTYPE));
  omx_wfdadec_component_Private->temporary_buffer->pBuffer = TCC_malloc(AUDIO_DEC_IN_BUFFER_SIZE*2);
  memset(omx_wfdadec_component_Private->temporary_buffer->pBuffer, 0, AUDIO_DEC_IN_BUFFER_SIZE*2);

  omx_wfdadec_component_Private->temp_input_buffer = omx_wfdadec_component_Private->temporary_buffer->pBuffer;
  omx_wfdadec_component_Private->temporary_buffer->nFilledLen=0;
  omx_wfdadec_component_Private->temporary_buffer->nOffset=0;

  omx_wfdadec_component_Private->isNewBuffer = 1;

  return err;
}

/** The Deinitialization function  */
OMX_ERRORTYPE omx_wfdadec_component_buffer_Deinit(OMX_COMPONENTTYPE *openmaxStandComp)
{

  omx_wfdadec_component_PrivateType* omx_wfdadec_component_Private = openmaxStandComp->pComponentPrivate;
  OMX_ERRORTYPE err = OMX_ErrorNone;


  /*Restore temporary input buffer pointer*/
  omx_wfdadec_component_Private->temporary_buffer->pBuffer = omx_wfdadec_component_Private->temp_input_buffer;
  TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Freeing Temporary Buffer\n");
  /* freeing temporary memory allocation */
  if(omx_wfdadec_component_Private->temporary_buffer->pBuffer) {
	TCC_free(omx_wfdadec_component_Private->temporary_buffer->pBuffer);
	omx_wfdadec_component_Private->temporary_buffer->pBuffer = NULL;
  }
  if(omx_wfdadec_component_Private->temporary_buffer) {
	TCC_free(omx_wfdadec_component_Private->temporary_buffer);
	omx_wfdadec_component_Private->temporary_buffer = NULL;
  }
  
  return err;
}
