/*
  @file component/openmax_wfdadec_component/omx_wfdadec_component.h

  OpenMAX audio decoder component. This component implements AAC / LPCM audio decoder.

  Copyright (C) 2008  STMicroelectronics
  Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
  Copyright (C) 2015 Telechips Inc.

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  $Date: 2009/03/10 13:33:28 $
  Revision $Rev: 554 $
  Author $Author: B060934 $
  revised by B060955.
*/

#ifndef _OMX_WFDADEC_COMPONENT_H_
#define _OMX_WFDADEC_COMPONENT_H_

#include <OMX_Types.h>
#include <OMX_Component.h>
#include <OMX_Core.h>
#include <pthread.h>
#include <stdlib.h>
#include <string.h>
#include <omx_base_filter.h>

#include "tcc_memory.h"

#include "tcc_video_common.h"

#include "cdk_core.h"
#include "cdk_audio.h"
#include <str_utils.h>

DERIVEDCLASS(omx_wfdadec_component_PrivateType, omx_base_filter_PrivateType)
#define omx_wfdadec_component_PrivateType_FIELDS omx_base_filter_PrivateType_FIELDS \
	tsem_t* tccwavDecSyncSem; \
	OMX_AUDIO_PARAM_PCMMODETYPE pAudioPcmMode_input; \
	OMX_AUDIO_PARAM_PCMMODETYPE pAudioPcmMode_output; \
  	OMX_U32 audio_coding_type; \
	OMX_BUFFERHEADERTYPE *temporary_buffer; \
	OMX_U8* temp_input_buffer; \
	OMX_U8* pcm_out_ptr; \
	short*	pcm_left_ptr; \
	short*	pcm_right_ptr; \
	int isNewBuffer; \
	unsigned long time_pos; \
	int *Power_Spectrum_info; \
	int *Energy_Volume_level; \
	int open_audido_codec; \
	OMX_TICKS nTimeStamp; \
	int first_timestamp_send; \
	int first_time_backupok; \
	OMX_TICKS first_nTimeStamp; \
	OMX_TIME_CONFIG_TIMESTAMPTYPE sTimeStamp; \
	adec_input_t sADecInput; \
	adec_output_t sADecOutput; \
	adec_init_t sADecInit; \
	cdmx_info_t sCDmxInfo; \
	cdk_core_t *pCdk; \
	int container_type; \
	int audio_codec_type; \
	int video_codec_type; \
	cdmx_output_t CdmxOut; \
	int	date_consumed; \
	int alsa_open_status; \
	int spdif_mode; \
	int dynamic_range_mode; \
	int outputmode; \
	int extramode; \
	int	measure_audio_time; \
	int multi_language_mode; \
	int previous_codec_open; \
	OMX_U32 is_notsupportaudio; \
	OMX_U32 ErrorDetectMode; \
	OMX_U32 ErrorDetect; \
	OMX_U32 nErrorcount; \
	int fast_stop; \
	int seek_start; \
	int flush_input_buffer; \
	int dec_fail_resync_cnt; \
	int	play_type; \
	tsem_t* StopSyncSem; \
	cdk_func_t* dec_func; \
	int audio_skip_start; \
	pthread_mutex_t audiodec_off_mutex; \
	long long audio_skipframe_pts;
ENDCLASS(omx_wfdadec_component_PrivateType)

//-------------------------------------------------------------------------------------------------------------------

/* Component private entry points declaration */
OMX_ERRORTYPE omx_wfdadec_component_Constructor(OMX_COMPONENTTYPE *openmaxStandComp,OMX_STRING cComponentName);
OMX_ERRORTYPE omx_wfdadec_component_Destructor(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdadec_decoder_MessageHandler(OMX_COMPONENTTYPE*,internalRequestMessageType*);
OMX_ERRORTYPE omx_wfdadec_component_Init(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdadec_component_Deinit(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdadec_component_buffer_Init(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdadec_component_buffer_Deinit(OMX_COMPONENTTYPE *openmaxStandComp);

void omx_wfdadec_component_BufferMgmtCallback(
  OMX_COMPONENTTYPE *openmaxStandComp,
	OMX_BUFFERHEADERTYPE* inputbuffer,
  OMX_BUFFERHEADERTYPE* outputbuffer);

OMX_ERRORTYPE omx_wfdadec_component_GetParameter(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nParamIndex,
  OMX_INOUT OMX_PTR ComponentParameterStructure);

OMX_ERRORTYPE omx_wfdadec_component_SetParameter(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nParamIndex,
  OMX_IN  OMX_PTR ComponentParameterStructure);

OMX_ERRORTYPE omx_cdkadec_component_SetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_IN  OMX_PTR pComponentConfigStructure);

OMX_ERRORTYPE omx_wfdadec_component_GetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_IN  OMX_PTR pComponentConfigStructure);


OMX_ERRORTYPE omx_wfdadec_component_GetExtensionIndex(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_STRING cParameterName,
  OMX_OUT OMX_INDEXTYPE* pIndexType);


#endif /* _OMX_WFDADEC_COMPONENT_H_ */




