/**
  @file component/openmax_wfdalsasink_component/omx_wfdalsasink_component.c
 
  OpenMAX ALSA sink component. This component is an audio sink that uses ALSA library.

  Copyright (C) 2007-2008  STMicroelectronics
  Copyright (C) 2007-2008 Nokia Corporation and/or its subsidiary(-ies).
  Copyright (C) 2015 Telechips Inc.

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  $Date: 2009/03/10 13:33:28 $
  Revision $Rev: 558 $
  Author $Author: B060934 $
  revised by B060955.
*/

#include <omxcore.h>
#include <omx_base_audio_port.h>
#include <omx_base_component.h>
#include <omx_base_clock_port.h>
#include <omx_base_sink.h>
#include <omx_wfdalsasink_component.h>
#include <tcc_video_common.h>
#include <tcc_audio_common.h>
#include <OMX_TCC_Index.h>

//#define SW_VOLUME_CONTROL 
#define DROP_FRAME_NUM		20
/* gain value */
#define GAIN_VALUE 1.0f

#define IRREGULAR_PTS_COUNT		0

/** Maximum Number of AlsaSink Instance*/
#define MAX_COMPONENT_ALSASINK 1

/** Number of AlsaSink Instance*/
static OMX_U32 noAlsasinkInstance = 0;

int omx_wfdalsasink_dynamix_setting (OMX_COMPONENTTYPE * openmaxStandComp)
{
	int 	  err;
	omx_wfdalsasink_component_PrivateType *omx_wfdalsasink_component_Private = openmaxStandComp->pComponentPrivate;
	snd_pcm_t *playback_handle;
	snd_pcm_hw_params_t *hw_params;

	if (omx_wfdalsasink_component_Private->hw_params)
	{
		snd_pcm_hw_params_free (omx_wfdalsasink_component_Private->hw_params);
	}
	if (omx_wfdalsasink_component_Private->playback_handle)
	{
		snd_pcm_close (omx_wfdalsasink_component_Private->playback_handle);
	}

	if(omx_wfdalsasink_component_Private->pcm_handle_name == NULL){
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "pcm_handle_name is NULL...\n");
		return OMX_ErrorHardware;
	}
	
	/*ALSA Open*/
   if ((err = snd_pcm_open (&omx_wfdalsasink_component_Private->playback_handle,
							  omx_wfdalsasink_component_Private->pcm_handle_name, 
							  SND_PCM_STREAM_PLAYBACK, 0)) < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot open audio device %s (%s)\n", 
															omx_wfdalsasink_component_Private->pcm_handle_name, 
															snd_strerror (err));
		return OMX_ErrorHardware;
	}
	else
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Got playback handle at %p %p in %i\n",
			   omx_wfdalsasink_component_Private->playback_handle, &omx_wfdalsasink_component_Private->playback_handle,
			   getpid ());


	 /*ALSA Malloc*/
	if (snd_pcm_hw_params_malloc (&omx_wfdalsasink_component_Private->hw_params) < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "%s: failed allocating input pPort hw parameters\n", __func__);
		return OMX_ErrorHardware;
	}
	else
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Got hw parameters at %p\n", omx_wfdalsasink_component_Private->hw_params);

	playback_handle = omx_wfdalsasink_component_Private->playback_handle;
	hw_params = omx_wfdalsasink_component_Private->hw_params;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ,"alsa configure setting!!!! %d %d %d\n", 
			omx_wfdalsasink_component_Private->sPCMModeParam.nSamplingRate,
			omx_wfdalsasink_component_Private->sPCMModeParam.nChannels,
			omx_wfdalsasink_component_Private->sPCMModeParam.nBitPerSample);

	if ((err = snd_pcm_hw_params_any (omx_wfdalsasink_component_Private->playback_handle,
								omx_wfdalsasink_component_Private->hw_params)) < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot initialize hardware parameter structure (%s)\n", snd_strerror (err));
		return OMX_ErrorHardware;
	}


	// Set the interleaved read and write format.
	if ((err = snd_pcm_hw_params_set_access (playback_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot set access type intrleaved (%s)\n", snd_strerror (err));
		return OMX_ErrorHardware;
	}

	if (omx_wfdalsasink_component_Private->sPCMModeParam.eEndian == OMX_EndianLittle)
	{
		switch (omx_wfdalsasink_component_Private->sPCMModeParam.nBitPerSample)
		{
		case 24:
			if (snd_pcm_hw_params_set_format (playback_handle, hw_params, SND_PCM_FORMAT_S24_3LE) < 0)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "snd_pcm_hw_params_set_format error\n");
			}
			break;
		case 16:
		default:
			if (snd_pcm_hw_params_set_format (playback_handle, hw_params, SND_PCM_FORMAT_S16_LE) < 0)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "snd_pcm_hw_params_set_format error\n");
			}
			break;
		}
	}
	else
	{
		switch (omx_wfdalsasink_component_Private->sPCMModeParam.nBitPerSample)
		{
		case 24:
			if (snd_pcm_hw_params_set_format (playback_handle, hw_params, SND_PCM_FORMAT_S24_3BE) < 0)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "snd_pcm_hw_params_set_format error\n");
			}
			break;
		case 16:
		default:
			if (snd_pcm_hw_params_set_format (playback_handle, hw_params, SND_PCM_FORMAT_S16_BE) < 0)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "snd_pcm_hw_params_set_format error\n");
			}
			break;
		}
	}
	
	// sample
	if (snd_pcm_hw_params_set_rate_near(playback_handle, hw_params, &omx_wfdalsasink_component_Private->sPCMModeParam.nSamplingRate, 0) < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "snd_pcm_hw_params_set_rate_near error\n");
	}
	// channel
	if (snd_pcm_hw_params_set_channels(playback_handle, hw_params, omx_wfdalsasink_component_Private->sPCMModeParam.nChannels) < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "snd_pcm_hw_params_set_channels error\n");
	}


	snd_pcm_uframes_t handle_bufferSize = 2*1024;
	unsigned int handle_sampleRate;
	unsigned int handle_latency = 10000;

	snd_pcm_uframes_t bufferSize = handle_bufferSize;
	unsigned int requestedRate = handle_sampleRate;
	unsigned int latency = handle_latency;


	// Make sure we have at least the size we originally wanted
	err = snd_pcm_hw_params_set_buffer_size(playback_handle, hw_params, bufferSize);
	if (err < 0) 
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Unable to set buffer size to %d:  %s \n",(int)bufferSize, snd_strerror(err));
	}
	
	// Setup buffers for latency
	err = snd_pcm_hw_params_set_buffer_time_near(playback_handle, hw_params, &latency, NULL);
	if (err < 0) 
	{
		printf("[Error] Setup buffers for latency.. \n");	// debug code, B090183
	
		/* That didn't work, set the period instead */
		unsigned int periodTime = latency / 4;
		err = snd_pcm_hw_params_set_period_time_near(playback_handle, hw_params, &periodTime, NULL);
		if (err < 0) 
		{
			printf("Unable to set the period time for latency: %s \n", snd_strerror(err));
		}
		
		snd_pcm_uframes_t periodSize;
		err = snd_pcm_hw_params_get_period_size(hw_params, &periodSize, NULL);
		if (err < 0) 
		{
			printf("Unable to get the period size for latency: %s \n", snd_strerror(err));
		}

		bufferSize = periodSize * 4;
		printf("ALSA bufferSize %d handle_bufferSize %d.............\n",bufferSize);
		if (bufferSize < handle_bufferSize)
		{ 
			bufferSize = handle_bufferSize;		/* MP3 case: buffersize(1024*10) */
		}
		err = snd_pcm_hw_params_set_buffer_size_near(playback_handle, hw_params, &bufferSize);
		if (err < 0) 
		{
			printf("Unable to set the buffer size for latency: %s\n", snd_strerror(err));
		}
	}
	else 
	{
		// OK, we got buffer time near what we expect. See what that did for bufferSize.
		err = snd_pcm_hw_params_get_buffer_size(hw_params, &bufferSize);
		if (err < 0) 
		{
			printf("Unable to get the buffer size for latency: %s\n", snd_strerror(err));
		}

		// Does set_buffer_time_near change the passed value? It should.
		err = snd_pcm_hw_params_get_buffer_time(hw_params, &latency, NULL);
		if (err < 0) 
		{
			printf("Unable to get the buffer time for latency: %s\n", snd_strerror(err));
		}

		unsigned int periodTime = latency / 4;
		
		err = snd_pcm_hw_params_set_period_time_near(playback_handle, hw_params, &periodTime, NULL);
		if (err < 0) 
		{
			printf("Unable to set the period time for latency: %s\n", snd_strerror(err));
		}
	}

	if ((err = snd_pcm_hw_params (playback_handle, hw_params)) < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot set parameters (%s)\n", snd_strerror (err));
		return OMX_ErrorHardware;
	}

	if ((err = snd_pcm_prepare (playback_handle)) < 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot prepare audio interface for use (%s)\n", snd_strerror (err));
		return OMX_ErrorHardware;
	}
	omx_wfdalsasink_component_Private->alsa_configure_request = 0;
	omx_wfdalsasink_component_Private->alsa_status = 1;
}

OMX_ERRORTYPE omx_wfdalsasink_component_Init (OMX_COMPONENTTYPE * openmaxStandComp)
{
	return (omx_wfdalsasink_component_Constructor (openmaxStandComp, WFDSINK_ALSASINK_NAME));
}

/** The Constructor 
 */
OMX_ERRORTYPE omx_wfdalsasink_component_Constructor (OMX_COMPONENTTYPE * openmaxStandComp, OMX_STRING cComponentName)
{
	int 	  err;
	int 	  omxErr;
	omx_base_audio_PortType *pPort;
	omx_wfdalsasink_component_PrivateType *omx_wfdalsasink_component_Private;

	if (!openmaxStandComp->pComponentPrivate)
	{
		openmaxStandComp->pComponentPrivate = TCC_calloc (1, sizeof (omx_wfdalsasink_component_PrivateType));
		if (openmaxStandComp->pComponentPrivate == NULL)
		{
			return OMX_ErrorInsufficientResources;
		}
	}

	omx_wfdalsasink_component_Private = openmaxStandComp->pComponentPrivate;
	omx_wfdalsasink_component_Private->ports = NULL;

	omxErr = omx_base_sink_Constructor (openmaxStandComp, cComponentName);
	if (omxErr != OMX_ErrorNone)
	{
		return OMX_ErrorInsufficientResources;
	}

	omx_wfdalsasink_component_Private->sPortTypesParam[OMX_PortDomainAudio].nStartPortNumber = 0;
	omx_wfdalsasink_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts = 1;

  /** Allocate Ports and call port constructor. */
	if (omx_wfdalsasink_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts && !omx_wfdalsasink_component_Private->ports)
	{
		omx_wfdalsasink_component_Private->ports =
			TCC_calloc ((omx_wfdalsasink_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts), sizeof (omx_base_PortType *));
		if (!omx_wfdalsasink_component_Private->ports)
		{
			return OMX_ErrorInsufficientResources;
		}
		omx_wfdalsasink_component_Private->ports[0] = TCC_calloc (1, sizeof (omx_base_audio_PortType));
		if (!omx_wfdalsasink_component_Private->ports[0])
		{
			return OMX_ErrorInsufficientResources;
		}
		base_audio_port_Constructor (openmaxStandComp, &omx_wfdalsasink_component_Private->ports[0], 0, OMX_TRUE);
	}

	pPort = (omx_base_audio_PortType *) omx_wfdalsasink_component_Private->ports[OMX_BASE_SINK_INPUTPORT_INDEX];

	// set the pPort params, now that the ports exist  
    /** Domain specific section for the ports. */
	pPort->sPortParam.format.audio.eEncoding = OMX_AUDIO_CodingPCM;
	/*Input pPort buffer size is equal to the size of the output buffer of the previous component */
	pPort->sPortParam.nBufferSize = AUDIO_DEC_OUT_BUFFER_SIZE; //DEFAULT_OUT_BUFFER_SIZE;
	pPort->sPortParam.nBufferCountMin = 4;
	pPort->sPortParam.nBufferCountActual = 4;
	/* Initializing the function pointers */
	omx_wfdalsasink_component_Private->BufferMgmtCallback = omx_wfdalsasink_component_BufferMgmtCallback;
	omx_wfdalsasink_component_Private->messageHandler = omx_wfdalsasink_component_MessageHandler;

	omx_wfdalsasink_component_Private->destructor = omx_wfdalsasink_component_Destructor;

	setHeader (&pPort->sAudioParam, sizeof (OMX_AUDIO_PARAM_PORTFORMATTYPE));
	pPort->sAudioParam.nPortIndex = 0;
	pPort->sAudioParam.nIndex = 0;
	pPort->sAudioParam.eEncoding = OMX_AUDIO_CodingPCM;

	/* OMX_AUDIO_PARAM_PCMMODETYPE */
	setHeader (&omx_wfdalsasink_component_Private->sPCMModeParam, sizeof (OMX_AUDIO_PARAM_PCMMODETYPE));
	omx_wfdalsasink_component_Private->sPCMModeParam.nPortIndex = 0;
	omx_wfdalsasink_component_Private->sPCMModeParam.nChannels = 2;
	omx_wfdalsasink_component_Private->sPCMModeParam.eNumData = OMX_NumericalDataSigned;
	omx_wfdalsasink_component_Private->sPCMModeParam.eEndian = OMX_EndianLittle;
	omx_wfdalsasink_component_Private->sPCMModeParam.bInterleaved = OMX_TRUE;
	omx_wfdalsasink_component_Private->sPCMModeParam.nBitPerSample = 16;
	omx_wfdalsasink_component_Private->sPCMModeParam.nSamplingRate = 48000;
	omx_wfdalsasink_component_Private->sPCMModeParam.ePCMMode = OMX_AUDIO_PCMModeLinear;
	omx_wfdalsasink_component_Private->sPCMModeParam.eChannelMapping[0] = OMX_AUDIO_ChannelNone;
	omx_wfdalsasink_component_Private->after_pause = 0;


	pthread_mutex_init (&omx_wfdalsasink_component_Private->audio_alsa_mutex, NULL);
	pthread_mutex_init (&omx_wfdalsasink_component_Private->get_audio_pts_mutex, NULL);

	if(!omx_wfdalsasink_component_Private->StopSyncSem)
	{
		omx_wfdalsasink_component_Private->StopSyncSem = TCC_calloc(1,sizeof(tsem_t));
		if(omx_wfdalsasink_component_Private->StopSyncSem == NULL) return OMX_ErrorInsufficientResources;
			tsem_init(omx_wfdalsasink_component_Private->StopSyncSem, 0);
	}

	openmaxStandComp->SetParameter = omx_wfdalsasink_component_SetParameter;
	openmaxStandComp->GetParameter = omx_wfdalsasink_component_GetParameter;
	openmaxStandComp->GetConfig = omx_wfdalsasink_component_GetConfig;
	openmaxStandComp->SetConfig = omx_wfdalsasink_component_SetConfig;

	/* Write in the default parameters */
	omx_wfdalsasink_component_Private->AudioPCMConfigured = 0;
	omx_wfdalsasink_component_Private->eState = OMX_TIME_ClockStateStopped;
	omx_wfdalsasink_component_Private->xScale = 1 << 16;
	omx_wfdalsasink_component_Private->alsa_reopen_request = 0;
	omx_wfdalsasink_component_Private->alsa_open_mode = ALSA_OPEN_SRC;
	omx_wfdalsasink_component_Private->hw_params = 0;
	omx_wfdalsasink_component_Private->playback_handle = 0;
	omx_wfdalsasink_component_Private->alsa_status = 0;
	omx_wfdalsasink_component_Private->fast_stop = OMX_FALSE;
	omx_wfdalsasink_component_Private->underrun_count = 0;
	omx_wfdalsasink_component_Private->underrun_time = 0;
	omx_wfdalsasink_component_Private->alsa_configure_request = 0;
	omx_wfdalsasink_component_Private->broadcastflag =0;
	omx_wfdalsasink_component_Private->alsa_setting = OMX_FALSE;
	omx_wfdalsasink_component_Private->alsa_write_start = OMX_FALSE;
	omx_wfdalsasink_component_Private->Use_Buffering_Mode = OMX_TRUE;
    omx_wfdalsasink_component_Private->gain = 50.0f / 100.0f; // default gain
    omx_wfdalsasink_component_Private->supprot_volume_control = OMX_FALSE;
	return OMX_ErrorNone;
}

/** The Destructor 
 */
OMX_ERRORTYPE omx_wfdalsasink_component_Destructor (OMX_COMPONENTTYPE * openmaxStandComp)
{
	omx_wfdalsasink_component_PrivateType *omx_wfdalsasink_component_Private = openmaxStandComp->pComponentPrivate;
	OMX_U32   i;

    free((OMX_S16*)omx_wfdalsasink_component_Private->pcm_buffer_for_volume_ctrl);
    
	if (omx_wfdalsasink_component_Private->hw_params)
	{
		snd_pcm_hw_params_free (omx_wfdalsasink_component_Private->hw_params);
	}
	if (omx_wfdalsasink_component_Private->playback_handle)
	{
		snd_pcm_close (omx_wfdalsasink_component_Private->playback_handle);
	}

	/* frees port/s */
	if (omx_wfdalsasink_component_Private->ports)
	{
		for (i = 0; i < (omx_wfdalsasink_component_Private->sPortTypesParam[OMX_PortDomainAudio].nPorts +
						 omx_wfdalsasink_component_Private->sPortTypesParam[OMX_PortDomainOther].nPorts); i++)
		{
			if (omx_wfdalsasink_component_Private->ports[i])
				omx_wfdalsasink_component_Private->ports[i]->PortDestructor (omx_wfdalsasink_component_Private->ports[i]);
		}
		TCC_free (omx_wfdalsasink_component_Private->ports);
		omx_wfdalsasink_component_Private->ports = NULL;
	}

	noAlsasinkInstance--;

	pthread_mutex_destroy (&omx_wfdalsasink_component_Private->audio_alsa_mutex);
	pthread_mutex_destroy (&omx_wfdalsasink_component_Private->get_audio_pts_mutex);

	if(omx_wfdalsasink_component_Private->StopSyncSem) {
		tsem_deinit(omx_wfdalsasink_component_Private->StopSyncSem);
		TCC_free(omx_wfdalsasink_component_Private->StopSyncSem);
		omx_wfdalsasink_component_Private->StopSyncSem=NULL;
	}
	return omx_base_sink_Destructor (openmaxStandComp);

}

/** @brief Releases buffers under processing.
 * This function must be implemented in the derived classes, for the
 * specific processing
 */
OMX_ERRORTYPE omx_wfdalsasink_component_port_FlushProcessingBuffers (omx_base_PortType * openmaxStandPort)
{
	omx_base_component_PrivateType *omx_base_component_Private;
	omx_wfdalsasink_component_PrivateType *omx_wfdalsasink_component_Private;
	OMX_BUFFERHEADERTYPE *pBuffer;
	omx_base_clock_PortType *pClockPort;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME, "In %s\n", __func__);
	omx_base_component_Private =
		(omx_base_component_PrivateType *) openmaxStandPort->standCompContainer->pComponentPrivate;
	omx_wfdalsasink_component_Private = (omx_wfdalsasink_component_PrivateType *) omx_base_component_Private;

	pClockPort = (omx_base_clock_PortType *) omx_wfdalsasink_component_Private->ports[OMX_BASE_SINK_CLOCKPORT_INDEX];

	if (openmaxStandPort->sPortParam.eDomain != OMX_PortDomainOther)
	{	/* clock buffers not used in the clients buffer managment function */
		pthread_mutex_lock (&omx_base_component_Private->flush_mutex);
		openmaxStandPort->bIsPortFlushed = OMX_TRUE;
		/*Signal the buffer management thread of port flush,if it is waiting for buffers */
		if (omx_base_component_Private->bMgmtSem->semval == 0)
		{
			tsem_up (omx_base_component_Private->bMgmtSem);
		}

		if (omx_base_component_Private->state == OMX_StatePause)
		{
			/*Waiting at paused state */
			tsem_signal (omx_base_component_Private->bStateSem);
		}
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ, "In %s waiting for flush all condition port index =%d\n", __func__,
			   (int) openmaxStandPort->sPortParam.nPortIndex);
		/* Wait until flush is completed */
		pthread_mutex_unlock (&omx_base_component_Private->flush_mutex);

		/*Dummy signal to clock port */
		if (pClockPort->pBufferSem->semval == 0)
		{
			tsem_up (pClockPort->pBufferSem);
		}
		tsem_down (omx_base_component_Private->flush_all_condition);
		if (pClockPort->pBufferSem->semval > 0)
		{
			tsem_down (pClockPort->pBufferSem);
		}
	}

	tsem_reset (omx_base_component_Private->bMgmtSem);

	/* Flush all the buffers not under processing */
	while (openmaxStandPort->pBufferSem->semval > 0)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ, "In %s TFlag=%x Flusing Port=%d,Semval=%d Qelem=%d\n",
			   __func__, (int) openmaxStandPort->nTunnelFlags, (int) openmaxStandPort->sPortParam.nPortIndex,
			   (int) openmaxStandPort->pBufferSem->semval, (int) openmaxStandPort->pBufferQueue->nelem);

		tsem_down (openmaxStandPort->pBufferSem);
		pBuffer = dequeue (openmaxStandPort->pBufferQueue);
		if (PORT_IS_TUNNELED (openmaxStandPort) && !PORT_IS_BUFFER_SUPPLIER (openmaxStandPort))
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"In %s: Comp %s is returning io:%d buffer\n",
				   __func__, omx_base_component_Private->name, (int) openmaxStandPort->sPortParam.nPortIndex);
			if (openmaxStandPort->sPortParam.eDir == OMX_DirInput)
			{
				((OMX_COMPONENTTYPE *) (openmaxStandPort->hTunneledComponent))->FillThisBuffer (openmaxStandPort->
																								hTunneledComponent,
																								pBuffer);
			}
			else
			{
				((OMX_COMPONENTTYPE *) (openmaxStandPort->hTunneledComponent))->EmptyThisBuffer (openmaxStandPort->
																								 hTunneledComponent,
																								 pBuffer);
			}
		}
		else if (PORT_IS_TUNNELED_N_BUFFER_SUPPLIER (openmaxStandPort))
		{
			queue (openmaxStandPort->pBufferQueue, pBuffer);
		}
		else
		{
			(*(openmaxStandPort->BufferProcessedCallback)) (openmaxStandPort->standCompContainer,
															omx_base_component_Private->callbackData, pBuffer);
		}
	}
	/*Port is tunneled and supplier and didn't received all it's buffer then wait for the buffers */
	if (PORT_IS_TUNNELED_N_BUFFER_SUPPLIER (openmaxStandPort))
	{
		while (openmaxStandPort->pBufferQueue->nelem != openmaxStandPort->nNumAssignedBuffers)
		{
			tsem_down (openmaxStandPort->pBufferSem);
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ, "In %s Got a buffer qelem=%d\n", __func__, openmaxStandPort->pBufferQueue->nelem);
		}
		tsem_reset (openmaxStandPort->pBufferSem);
	}

	pthread_mutex_lock (&omx_base_component_Private->flush_mutex);
	openmaxStandPort->bIsPortFlushed = OMX_FALSE;
	pthread_mutex_unlock (&omx_base_component_Private->flush_mutex);

	tsem_up (omx_base_component_Private->flush_condition);

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"Out %s Port Index=%d bIsPortFlushed=%d Component %s\n", __func__,
		   (int) openmaxStandPort->sPortParam.nPortIndex, (int) openmaxStandPort->bIsPortFlushed,
		   omx_base_component_Private->name);

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"In %s TFlag=%x Qelem=%d BSem=%d bMgmtsem=%d component=%s\n", __func__,
		   (int) openmaxStandPort->nTunnelFlags,
		   (int) openmaxStandPort->pBufferQueue->nelem,
		   (int) openmaxStandPort->pBufferSem->semval,
		   (int) omx_base_component_Private->bMgmtSem->semval, omx_base_component_Private->name);

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME, "Out %s Port Index=%d\n", __func__, (int) openmaxStandPort->sPortParam.nPortIndex);

	return OMX_ErrorNone;
}

/** 
 * This function plays the input buffer. When fully consumed it returns.
 */

unsigned long long GetTime_MSec2(void) // milli second
{
	struct timeval now;
	gettimeofday (&now, NULL);
	return ((unsigned long long) now.tv_sec) * 1000 + ((unsigned long long) now.tv_usec) / 1000;
}


void omx_wfdalsasink_component_BufferMgmtCallback (OMX_COMPONENTTYPE * openmaxStandComp,
												OMX_BUFFERHEADERTYPE * inputbuffer)
{
	OMX_U32   frameSize;
	OMX_S32   written;
	OMX_S32   totalBuffer;
	OMX_S32   offsetBuffer;
	OMX_BOOL  allDataSent;
	OMX_BOOL  SendFrame;
	OMX_HANDLETYPE hclkComponent;
	omx_base_clock_PortType *pClockPort;
	omx_base_audio_PortType *pPort;
	OMX_TIME_CONFIG_MEDIATIMEREQUESTSPEEDTYPE sMediaTimeNotify; 
	omx_wfdalsasink_component_PrivateType *omx_wfdalsasink_component_Private = openmaxStandComp->pComponentPrivate;
 
	if(omx_wfdalsasink_component_Private->fast_stop)
	{
		usleep(10000);
		inputbuffer->nFilledLen = 0;
		return;
	}	

	if(inputbuffer->nFilledLen > inputbuffer->nAllocLen)
	{
		DEBUG (DEB_LEV_TCC_ERR,"inputbuffer->nFilledLen %d inputbuffer->nAllocLen %d\n",inputbuffer->nFilledLen,inputbuffer->nAllocLen);
		inputbuffer->nFilledLen = inputbuffer->nAllocLen;
	}
	
	if(omx_wfdalsasink_component_Private->alsa_setting == OMX_FALSE)
	{
		int ret = omx_wfdalsasink_dynamix_setting(openmaxStandComp);
		if(ret == OMX_ErrorHardware)
		{
			(*(omx_wfdalsasink_component_Private->callbacks->EventHandler))
													(openmaxStandComp,
													omx_wfdalsasink_component_Private->callbackData,
													OMX_EventVendorALSASettingFailure, /* The command was completed */
													NULL, /* Format Not Detected */
													AUDIO_PORT_INDEX, /* This is the output port index*/
													NULL);
			return;
		}
		
		omx_wfdalsasink_component_Private->alsa_setting = OMX_TRUE;
        omx_wfdalsasink_component_Private->pcm_buffer_for_volume_ctrl = malloc(inputbuffer->nFilledLen); /*It is for Volume control*/
	}
    /* Feed it to ALSA */
	frameSize =
		(omx_wfdalsasink_component_Private->sPCMModeParam.nChannels *
		 omx_wfdalsasink_component_Private->sPCMModeParam.nBitPerSample) >> 3;
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ, "Framesize is %u chl=%d bufSize=%d\n", (int) frameSize,
		   (int) omx_wfdalsasink_component_Private->sPCMModeParam.nChannels, (int) inputbuffer->nFilledLen);

	if (inputbuffer->nFilledLen < frameSize)
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Ouch!! In %s input buffer filled len(%d) less than frame size(%d)\n", __func__,
			   (int) inputbuffer->nFilledLen, (int) frameSize);
		return;
	}

	allDataSent = OMX_FALSE;
	
	totalBuffer = inputbuffer->nFilledLen / frameSize;
	offsetBuffer = 0;

	if(omx_wfdalsasink_component_Private->Use_Buffering_Mode == OMX_TRUE)
	{
		pthread_mutex_lock(&omx_wfdalsasink_component_Private->get_audio_pts_mutex);
		omx_wfdalsasink_component_Private->current_pts = inputbuffer->nTimeStamp;
		pthread_mutex_unlock(&omx_wfdalsasink_component_Private->get_audio_pts_mutex);
	}
	
	while (!allDataSent)
	{
		if(omx_wfdalsasink_component_Private->alsa_write_start == OMX_FALSE && omx_wfdalsasink_component_Private->Use_Buffering_Mode == OMX_TRUE)
		{
			unsigned long long audio_start_time = GetTime_MSec2();
			(*(omx_wfdalsasink_component_Private->callbacks->EventHandler))(openmaxStandComp, 
																	omx_wfdalsasink_component_Private->callbackData,
																	OMX_EventVendorAlsaWriteStart,	/* The command was completed */
																	NULL,	/* port Format Detected */
																	NULL,	/* This is the output port index */
																	&audio_start_time);

			omx_wfdalsasink_component_Private->alsa_write_start = OMX_TRUE;
		}
#ifdef SW_VOLUME_CONTROL 
        if((omx_wfdalsasink_component_Private->gain != GAIN_VALUE) && (omx_wfdalsasink_component_Private->supprot_volume_control == OMX_TRUE)){
            int cnt;
            int sampleCount = inputbuffer->nFilledLen / 2; // signed 16 bit samples assumed
            for (cnt = 0; cnt < sampleCount; cnt++){
                ((OMX_S16*) omx_wfdalsasink_component_Private->pcm_buffer_for_volume_ctrl)[cnt] = 
                (OMX_S16)(((OMX_S16*) inputbuffer->pBuffer)[cnt] * omx_wfdalsasink_component_Private->gain);
            }
            written = snd_pcm_writei (omx_wfdalsasink_component_Private->playback_handle,
                                      omx_wfdalsasink_component_Private->pcm_buffer_for_volume_ctrl + (offsetBuffer * frameSize), totalBuffer);
        }
        else
#endif
        {
            written = snd_pcm_writei (omx_wfdalsasink_component_Private->playback_handle, inputbuffer->pBuffer + (offsetBuffer * frameSize), totalBuffer);
        }

		if (written < 0)
		{
			if (written == -EPIPE)
			{			  
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ, "ALSA Underrun.. %d\n", totalBuffer);
				//audio_adma_contorl_onoff(0);
				//pthread_mutex_lock(&omx_wfdalsasink_component_Private->audio_alsa_mutex);
				written = snd_pcm_recover(omx_wfdalsasink_component_Private->playback_handle, written, 0);
				if (written < 0) 
				{
					printf("snd_pcm_writei failed: %s\n", snd_strerror(written));
				}
				//snd_pcm_prepare (omx_wfdalsasink_component_Private->playback_handle);
				//pthread_mutex_unlock(&omx_wfdalsasink_component_Private->audio_alsa_mutex);
				//audio_adma_contorl_onoff(1);

				written = 0;
			}
			else
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Cannot send any data to the audio device %s (%s)\n", "default",
					   snd_strerror (written));
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "IB FilledLen=%d,totalBuffer=%d,frame size=%d,offset=%d\n",
					   (int) inputbuffer->nFilledLen, (int) totalBuffer, (int) frameSize, (int) offsetBuffer);
				break;
				return;
			}
		}


		if (written != totalBuffer)
		{
			totalBuffer = totalBuffer - written;
			if(written != 0)
			{
				offsetBuffer = written;
			}
		}
		else
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"Buffer successfully sent to ALSA. Length was %i\n",
				   (int) inputbuffer->nFilledLen);
			allDataSent = OMX_TRUE;
			omx_wfdalsasink_component_Private->after_pause = 0;
		}
	}
    inputbuffer->nFilledLen = 0;
	inputbuffer->nFlags = 0;
}

OMX_ERRORTYPE omx_wfdalsasink_component_MessageHandler (OMX_COMPONENTTYPE * openmaxStandComp,
													 internalRequestMessageType * message)
{

	omx_wfdalsasink_component_PrivateType *omx_wfdalsasink_component_Private =
		(omx_wfdalsasink_component_PrivateType *) openmaxStandComp->pComponentPrivate;
	OMX_ERRORTYPE err;
	OMX_STATETYPE eCurrentState = omx_wfdalsasink_component_Private->state;
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "In %s\n", __func__);



		if ((message->messageParam == OMX_StateIdle) && (eCurrentState == OMX_StateLoaded)) 
		{

		}  
		else if ((message->messageParam == OMX_StateLoaded) && (eCurrentState == OMX_StateIdle)) 
		{

		}  
		else if ((message->messageParam == OMX_StateIdle) && (eCurrentState == OMX_StateExecuting))
		{  

		}

	/** Execute the base message handling */
	err = omx_base_component_MessageHandler (openmaxStandComp, message);

	return err;
}

OMX_ERRORTYPE omx_wfdalsasink_component_SetConfig (OMX_IN OMX_HANDLETYPE hComponent,
												OMX_IN OMX_INDEXTYPE nIndex, OMX_IN OMX_PTR pComponentConfigStructure)
{
	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *) hComponent;

	omx_wfdalsasink_component_PrivateType *omx_wfdalsasink_component_Private = openmaxStandComp->pComponentPrivate;
#if 0
	if (pComponentConfigStructure == NULL)
	{
		return OMX_ErrorBadParameter;
	}
#endif
	switch (nIndex)
	{
		case OMX_IndexVendorConfigExtendDisplayModeSet:
		{
			OUTPUT_INFO *info;

			info = (OUTPUT_INFO*)pComponentConfigStructure;
			if(info->mode == LINUX_SNOR_DONGLE)
			{
				omx_wfdalsasink_component_Private->platform = LINUX_SNOR_DONGLE;
				omx_wfdalsasink_component_Private->pcm_handle_name = info->pcm_handle_name;
			}
			else if(info->mode == LINUX_CCM)
			{
				omx_wfdalsasink_component_Private->platform = LINUX_CCM;
				omx_wfdalsasink_component_Private->pcm_handle_name = info->pcm_handle_name;
			}
			else if(info->mode == LINUX_PLATFORM)
			{
				omx_wfdalsasink_component_Private->platform = LINUX_PLATFORM;
				omx_wfdalsasink_component_Private->pcm_handle_name = info->pcm_handle_name;
			}else
			{
				 omx_wfdalsasink_component_Private->platform = LINUX_SNOR_DONGLE;
				 omx_wfdalsasink_component_Private->pcm_handle_name = info->pcm_handle_name;
				
			}
		}
		break;
		
		case OMX_IndexVendorConfigAlsaConfigureRequest:
			{
				OMX_AUDIO_PARAM_PCMMODETYPE *sPCMModeParam = (OMX_AUDIO_PARAM_PCMMODETYPE *) pComponentConfigStructure;
				memcpy (&omx_wfdalsasink_component_Private->sPCMModeParam, sPCMModeParam,
						sizeof (OMX_AUDIO_PARAM_PCMMODETYPE));
				omx_wfdalsasink_component_Private->alsa_configure_request = 1;
			}
			break;

	case OMX_IndexVendorSetSlow:
		{
			omx_wfdalsasink_component_Private->slow_times = *(int *) pComponentConfigStructure;
		}
		break;

	case OMX_IndexVendorConfigDynamic_AlsaOpen:
		{
			OMX_AUDIO_CONFIG_INFOTYPE *info = (OMX_AUDIO_CONFIG_INFOTYPE *) pComponentConfigStructure;
			omx_wfdalsasink_component_Private->alsa_open_mode = info->dynamic_alsa_open_mode;
			omx_wfdalsasink_component_Private->alsa_reopen_request = 1;
			omx_wfdalsasink_component_Private->alsa_configure_request = 1;
			omx_wfdalsasink_component_Private->sPCMModeParam.nChannels = info->nChannels;
			omx_wfdalsasink_component_Private->sPCMModeParam.nBitPerSample = info->nBitPerSample;
			omx_wfdalsasink_component_Private->sPCMModeParam.nSamplingRate = info->nSamplingRate;
		}
		break;

	case OMX_IndexVendorConfigFastStop:
	{
		OMX_AUDIO_CONFIG_INFOTYPE *info;
		info = (OMX_AUDIO_CONFIG_INFOTYPE*)pComponentConfigStructure;
		omx_wfdalsasink_component_Private->fast_stop = info->faststop;
	}
	break;

	case OMX_IndexVendorSetWFDBaseTimeReset:
	{
		omx_wfdalsasink_component_Private->alsa_write_start = OMX_FALSE;
	}
	break;	

	case OMX_IndexVendorSetBufferingMode:
	{
		if(omx_wfdalsasink_component_Private->Use_Buffering_Mode == OMX_FALSE)
		{
			printf("Audio Sink Enable Buffering Mode................\n");
			omx_wfdalsasink_component_Private->Use_Buffering_Mode = OMX_TRUE; 
		}
		else		
		{
			printf("Audio Sink Disable Buffering Mode................\n");
			omx_wfdalsasink_component_Private->Use_Buffering_Mode = OMX_FALSE;	
		}
	}
	break;

    case OMX_IndexVendorSetVolumeControlEnable:
        printf("[WFD-ALSASINK] Enable Volume Control \n");
        omx_wfdalsasink_component_Private->supprot_volume_control = OMX_TRUE;
    break;    

    case OMX_IndexVendorSetVolumeControl:
    {
        float gain_value = *(float *) pComponentConfigStructure;

        if(100 < gain_value){
            printf("[WFD-ALSASINK] MAX gain Value is 100  gain= %d \n", gain_value );
        }
        else{
            omx_wfdalsasink_component_Private->gain = gain_value / 100.0f;
        }
    }
    break;  
   
   case OMX_IndexVendorSetVolumeMute:
   {

   }
   break;  

	case OMX_IndexVendorConfigClockBufferClear:
	{
		omx_base_clock_PortType *pClockPort;
		pClockPort = (omx_base_clock_PortType *) omx_wfdalsasink_component_Private->ports[OMX_BASE_SINK_CLOCKPORT_INDEX];
		OMX_BUFFERHEADERTYPE *clockBuffer;

		/* check for any scale change information from the clock component */
		while(pClockPort->pBufferSem->semval > 0)
		{
			tsem_down (pClockPort->pBufferSem);
			if (pClockPort->pBufferQueue->nelem > 0)
			{
				clockBuffer = dequeue (pClockPort->pBufferQueue);
				if (checkHeader (clockBuffer, sizeof (OMX_BUFFERHEADERTYPE)))
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "crashed output buffer!!\n", __func__);
				}
				pClockPort->ReturnBufferFunction ((omx_base_PortType *) pClockPort, clockBuffer);
			}
		}
	}

	{
		omx_base_audio_PortType *pAudioPort;
		pAudioPort = (omx_base_audio_PortType *) omx_wfdalsasink_component_Private->ports[OMX_BASE_SINK_INPUTPORT_INDEX];
		OMX_BUFFERHEADERTYPE *audioBuffer;

		if(pAudioPort->pBufferSem->semval > 0)
		{
			if(omx_wfdalsasink_component_Private->playback_handle)
			{
				if(tsem_down_timewait(omx_wfdalsasink_component_Private->StopSyncSem,4) == 0)
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "%s alsa sync stop timeout!!\n", __func__);
				}
			}
		}
	}
	
#if 0
	{
		omx_base_audio_PortType *pAudioPort;
		pAudioPort = (omx_base_audio_PortType *) omx_wfdalsasink_component_Private->ports[OMX_BASE_SINK_INPUTPORT_INDEX];
		OMX_BUFFERHEADERTYPE *audioBuffer;
		
		/* check for any scale change information from the clock component */
		while(pAudioPort->pBufferSem->semval > 0)
		{
			tsem_down (pAudioPort->pBufferSem);
			if (pAudioPort->pBufferQueue->nelem > 0)
			{
				audioBuffer = dequeue (pAudioPort->pBufferQueue);
				if (checkHeader (audioBuffer, sizeof (OMX_BUFFERHEADERTYPE)))
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "crashed Audio Input buffer!!\n", __func__);
				}
				pAudioPort->ReturnBufferFunction ((omx_base_PortType *) pAudioPort, audioBuffer);
			}
		}

	}
#endif
	break;

	case OMX_IndexVendorConfigFlushDecoderInputBuffer:
		omx_wfdalsasink_component_Private->flush_input_buffer = *(OMX_U32*)pComponentConfigStructure;
	break;

	///////////////////////////for BroadCast ///////////////////////////////
	case OMX_IndexVendorConfigVideotoBroadCastFlagSet:
		omx_wfdalsasink_component_Private->broadcastflag =	TRUE;
	break;

	case OMX_IndexVendorConfigPtsView:
		omx_wfdalsasink_component_Private->ptsview = *(int*)pComponentConfigStructure;
		//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_INFO,"audio pts view : %d\n", omx_wfdalsasink_component_Private->ptsview);
		break;
			
	default:	// delegate to superclass
		return omx_base_component_SetConfig (hComponent, nIndex, pComponentConfigStructure);
	}
	return OMX_ErrorNone;

}

OMX_ERRORTYPE omx_wfdalsasink_component_GetConfig (OMX_IN OMX_HANDLETYPE hComponent,
												OMX_IN OMX_INDEXTYPE nIndex,
												OMX_INOUT OMX_PTR pComponentConfigStructure)
{

	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *) hComponent;
	omx_wfdalsasink_component_PrivateType *omx_wfdalsasink_component_Private = openmaxStandComp->pComponentPrivate;

	if (pComponentConfigStructure == NULL)
	{
		return OMX_ErrorBadParameter;
	}

	switch (nIndex)
	{

	case OMX_IndexVendorConfigAlsaStatus:
		{
			OMX_AUDIO_CONFIG_INFOTYPE *info = (OMX_AUDIO_CONFIG_INFOTYPE *) pComponentConfigStructure;
			info->alsa_status = omx_wfdalsasink_component_Private->alsa_status;
		}
		break;

	case OMX_IndexVendorConfigCurrentPTS:
		{
			pthread_mutex_lock(&omx_wfdalsasink_component_Private->get_audio_pts_mutex);
			*(OMX_TICKS*)pComponentConfigStructure = omx_wfdalsasink_component_Private->current_pts;
			pthread_mutex_unlock(&omx_wfdalsasink_component_Private->get_audio_pts_mutex);
		}
	break;

	case OMX_Index_Vendor_ConfigAlsaRemainTime:
		{
			OMX_AUDIO_CONFIG_INFOTYPE *info = (OMX_AUDIO_CONFIG_INFOTYPE *) pComponentConfigStructure;
		
			snd_pcm_status_t *p_status;

			snd_pcm_sframes_t delay;
			snd_pcm_uframes_t frames;
			int resume_delay_time;
		
			if(omx_wfdalsasink_component_Private->playback_handle)
			{
				p_status = (snd_pcm_status_t *)TCC_malloc(snd_pcm_status_sizeof());
				snd_pcm_status(omx_wfdalsasink_component_Private->playback_handle, p_status);
				delay = snd_pcm_status_get_delay(p_status);
				frames = snd_pcm_status_get_avail(p_status);
				if(delay < 0)
				{
					resume_delay_time = ((frames/2)*1000)/omx_wfdalsasink_component_Private->sPCMModeParam.nSamplingRate;

					if(resume_delay_time > 1000)
					{
						resume_delay_time = 50;
					}
					
					info->remain_delay_time = resume_delay_time;
				}else{
					resume_delay_time = ((frames/2)*1000)/omx_wfdalsasink_component_Private->sPCMModeParam.nSamplingRate;
					info->remain_delay_time = resume_delay_time;
				}
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_INFO,"[A] remain delay : %d frames : %d resume_delay_time %d\n", delay, frames,resume_delay_time);
				TCC_free(p_status);
				pthread_mutex_lock(&omx_wfdalsasink_component_Private->audio_alsa_mutex);
				snd_pcm_reset(omx_wfdalsasink_component_Private->playback_handle);
				pthread_mutex_unlock(&omx_wfdalsasink_component_Private->audio_alsa_mutex);
			}else{
				resume_delay_time = 0;
				info->remain_delay_time = 0;
			}
		}
		break;


	default:	// delegate to superclass
		return omx_base_component_GetConfig (hComponent, nIndex, pComponentConfigStructure);
	}
	return OMX_ErrorNone;

}

OMX_ERRORTYPE omx_wfdalsasink_component_SetParameter (OMX_IN OMX_HANDLETYPE hComponent,
												   OMX_IN OMX_INDEXTYPE nParamIndex,
												   OMX_IN OMX_PTR ComponentParameterStructure)
{
	int 	  err;
	int 	  omxErr = OMX_ErrorNone;
	OMX_AUDIO_PARAM_PORTFORMATTYPE *pAudioPortFormat;
	OMX_OTHER_PARAM_PORTFORMATTYPE *pOtherPortFormat;
	OMX_AUDIO_PARAM_MP3TYPE *pAudioMp3;
	OMX_U32   portIndex;

	snd_pcm_uframes_t periodsize;
	snd_pcm_uframes_t period;
	snd_pcm_uframes_t buffersize;

	/* Check which structure we are being fed and make control its header */
	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *) hComponent;
	omx_wfdalsasink_component_PrivateType *omx_wfdalsasink_component_Private = openmaxStandComp->pComponentPrivate;
	omx_base_audio_PortType *pPort =
		(omx_base_audio_PortType *) omx_wfdalsasink_component_Private->ports[OMX_BASE_SINK_INPUTPORT_INDEX];
	omx_base_clock_PortType *pClockPort;
	snd_pcm_t *playback_handle = omx_wfdalsasink_component_Private->playback_handle;
	snd_pcm_hw_params_t *hw_params = omx_wfdalsasink_component_Private->hw_params;

	if (ComponentParameterStructure == NULL)
	{
		return OMX_ErrorBadParameter;
	}

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ, "	Setting parameter %i\n", nParamIndex);

  /** Each time we are (re)configuring the hw_params thing
  * we need to reinitialize it, otherwise previous changes will not take effect.
  * e.g.: changing a previously configured sampling rate does not have
  * any effect if we are not calling this each time.
  */
	//err = snd_pcm_hw_params_any (playback_handle, hw_params);

	switch (nParamIndex)
	{
	case OMX_IndexParamAudioPortFormat:
		pAudioPortFormat = (OMX_AUDIO_PARAM_PORTFORMATTYPE *) ComponentParameterStructure;
		portIndex = pAudioPortFormat->nPortIndex;
		/*Check Structure Header and verify component state */
		omxErr =
			omx_base_component_ParameterSanityCheck (hComponent, portIndex, pAudioPortFormat,
													 sizeof (OMX_AUDIO_PARAM_PORTFORMATTYPE));
		if (omxErr != OMX_ErrorNone)
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Parameter Check Error=%x\n", __func__, omxErr);
			break;
		}
		if (portIndex < 1)
		{
			memcpy (&pPort->sAudioParam, pAudioPortFormat, sizeof (OMX_AUDIO_PARAM_PORTFORMATTYPE));
		}
		else
		{
			return OMX_ErrorBadPortIndex;
		}
		break;
	case OMX_IndexParamOtherPortFormat:
		pOtherPortFormat = (OMX_OTHER_PARAM_PORTFORMATTYPE *) ComponentParameterStructure;
		portIndex = pOtherPortFormat->nPortIndex;
		err =
			omx_base_component_ParameterSanityCheck (hComponent, portIndex, pOtherPortFormat,
													 sizeof (OMX_OTHER_PARAM_PORTFORMATTYPE));
		if (err != OMX_ErrorNone)
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Parameter Check Error=%x\n", __func__, err);
			break;
		}
		if (portIndex != 1)
		{
			return OMX_ErrorBadPortIndex;
		}
		pClockPort = (omx_base_clock_PortType *) omx_wfdalsasink_component_Private->ports[portIndex];

		pClockPort->sOtherParam.eFormat = pOtherPortFormat->eFormat;
		break;

	case OMX_IndexParamAudioPcm:
		{
			unsigned int rate;
			OMX_AUDIO_PARAM_PCMMODETYPE *sPCMModeParam = (OMX_AUDIO_PARAM_PCMMODETYPE *) ComponentParameterStructure;

			portIndex = sPCMModeParam->nPortIndex;
#if 0
			/*Check Structure Header and verify component state */
			omxErr =
				omx_base_component_ParameterSanityCheck (hComponent, portIndex, sPCMModeParam,
														 sizeof (OMX_AUDIO_PARAM_PCMMODETYPE));
			if (omxErr != OMX_ErrorNone)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Parameter Check Error=%x\n", __func__, omxErr);
				break;
			}
#endif

			omx_wfdalsasink_component_Private->AudioPCMConfigured = 1;
			if (sPCMModeParam->nPortIndex != omx_wfdalsasink_component_Private->sPCMModeParam.nPortIndex)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Error setting input pPort index\n");
				omxErr = OMX_ErrorBadParameter;
				break;
			}
#if 0
			if (snd_pcm_hw_params_set_channels (playback_handle, hw_params, sPCMModeParam->nChannels))
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Error setting number of channels\n");
				return OMX_ErrorBadParameter;
			}
#endif

			if ((err =
				 snd_pcm_hw_params_any (omx_wfdalsasink_component_Private->playback_handle,
										omx_wfdalsasink_component_Private->hw_params)) < 0)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot initialize hardware parameter structure (%s)\n", snd_strerror (err));
				return OMX_ErrorHardware;
			}

			if (sPCMModeParam->bInterleaved == OMX_TRUE)
			{
				if ((err =
					 snd_pcm_hw_params_set_access (playback_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0)
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot set access type intrleaved (%s)\n", snd_strerror (err));
					return OMX_ErrorHardware;
				}
			}
			else
			{
				if ((err =
					 snd_pcm_hw_params_set_access (playback_handle, hw_params, SND_PCM_ACCESS_RW_NONINTERLEAVED)) < 0)
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot set access type non interleaved (%s)\n", snd_strerror (err));
					return OMX_ErrorHardware;
				}
			}
#if 0
			rate = sPCMModeParam->nSamplingRate;
			if ((err = snd_pcm_hw_params_set_rate_near (playback_handle, hw_params, &rate, 0)) < 0)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot set sample rate (%s)\n", snd_strerror (err));
				return OMX_ErrorHardware;
			}
			else
			{
				sPCMModeParam->nSamplingRate = rate;
				DEBUG (DEB_LEV_PARAMS, "Set correctly sampling rate to %lu\n", sPCMModeParam->nSamplingRate);
			}

			if (sPCMModeParam->ePCMMode == OMX_AUDIO_PCMModeLinear)
			{
				snd_pcm_format_t snd_pcm_format = SND_PCM_FORMAT_UNKNOWN;
				DEBUG (DEB_LEV_PARAMS, "Bit per sample %i, signed=%i, little endian=%i\n",
					   (int) sPCMModeParam->nBitPerSample,
					   (int) sPCMModeParam->eNumData == OMX_NumericalDataSigned,
					   (int) sPCMModeParam->eEndian == OMX_EndianLittle);

				switch (sPCMModeParam->nBitPerSample)
				{
				case 8:
					if (sPCMModeParam->eNumData == OMX_NumericalDataSigned)
					{
						snd_pcm_format = SND_PCM_FORMAT_S8;
					}
					else
					{
						snd_pcm_format = SND_PCM_FORMAT_U8;
					}
					break;
				case 16:
					if (sPCMModeParam->eNumData == OMX_NumericalDataSigned)
					{
						if (sPCMModeParam->eEndian == OMX_EndianLittle)
						{
							snd_pcm_format = SND_PCM_FORMAT_S16_LE;
						}
						else
						{
							snd_pcm_format = SND_PCM_FORMAT_S16_BE;
						}
					}
					if (sPCMModeParam->eNumData == OMX_NumericalDataUnsigned)
					{
						if (sPCMModeParam->eEndian == OMX_EndianLittle)
						{
							snd_pcm_format = SND_PCM_FORMAT_U16_LE;
						}
						else
						{
							snd_pcm_format = SND_PCM_FORMAT_U16_BE;
						}
					}
					break;
				case 24:
					if (sPCMModeParam->eNumData == OMX_NumericalDataSigned)
					{
						if (sPCMModeParam->eEndian == OMX_EndianLittle)
						{
							snd_pcm_format = SND_PCM_FORMAT_S24_LE;
						}
						else
						{
							snd_pcm_format = SND_PCM_FORMAT_S24_BE;
						}
					}
					if (sPCMModeParam->eNumData == OMX_NumericalDataUnsigned)
					{
						if (sPCMModeParam->eEndian == OMX_EndianLittle)
						{
							snd_pcm_format = SND_PCM_FORMAT_U24_LE;
						}
						else
						{
							snd_pcm_format = SND_PCM_FORMAT_U24_BE;
						}
					}
					break;

				case 32:
					if (sPCMModeParam->eNumData == OMX_NumericalDataSigned)
					{
						if (sPCMModeParam->eEndian == OMX_EndianLittle)
						{
							snd_pcm_format = SND_PCM_FORMAT_S32_LE;
						}
						else
						{
							snd_pcm_format = SND_PCM_FORMAT_S32_BE;
						}
					}
					if (sPCMModeParam->eNumData == OMX_NumericalDataUnsigned)
					{
						if (sPCMModeParam->eEndian == OMX_EndianLittle)
						{
							snd_pcm_format = SND_PCM_FORMAT_U32_LE;
						}
						else
						{
							snd_pcm_format = SND_PCM_FORMAT_U32_BE;
						}
					}
					break;
				default:
					omxErr = OMX_ErrorBadParameter;
					break;
				}

				if (snd_pcm_format != SND_PCM_FORMAT_UNKNOWN)
				{
					if ((err = snd_pcm_hw_params_set_format (playback_handle, hw_params, snd_pcm_format)) < 0)
					{
						TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot set sample format (%s)\n", snd_strerror (err));
						return OMX_ErrorHardware;
					}
					memcpy (&omx_wfdalsasink_component_Private->sPCMModeParam, ComponentParameterStructure,
							sizeof (OMX_AUDIO_PARAM_PCMMODETYPE));
				}
				else
				{
					DEBUG (DEB_LEV_SIMPLE_SEQ, "ALSA OMX_IndexParamAudioPcm configured\n");
					memcpy (&omx_wfdalsasink_component_Private->sPCMModeParam, ComponentParameterStructure,
							sizeof (OMX_AUDIO_PARAM_PCMMODETYPE));
				}
			}
			else if (sPCMModeParam->ePCMMode == OMX_AUDIO_PCMModeALaw)
			{
				DEBUG (DEB_LEV_SIMPLE_SEQ, "Configuring ALAW format\n\n");
				if ((err = snd_pcm_hw_params_set_format (playback_handle, hw_params, SND_PCM_FORMAT_A_LAW)) < 0)
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot set sample format (%s)\n", snd_strerror (err));
					return OMX_ErrorHardware;
				}
				memcpy (&omx_wfdalsasink_component_Private->sPCMModeParam, ComponentParameterStructure,
						sizeof (OMX_AUDIO_PARAM_PCMMODETYPE));
			}
			else if (sPCMModeParam->ePCMMode == OMX_AUDIO_PCMModeMULaw)
			{
				DEBUG (DEB_LEV_SIMPLE_SEQ, "Configuring ALAW format\n\n");
				if ((err = snd_pcm_hw_params_set_format (playback_handle, hw_params, SND_PCM_FORMAT_MU_LAW)) < 0)
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot set sample format (%s)\n", snd_strerror (err));
					return OMX_ErrorHardware;
				}
				memcpy (&omx_wfdalsasink_component_Private->sPCMModeParam, ComponentParameterStructure,
						sizeof (OMX_AUDIO_PARAM_PCMMODETYPE));
			}
#else

			memcpy (&omx_wfdalsasink_component_Private->sPCMModeParam, ComponentParameterStructure,
					sizeof (OMX_AUDIO_PARAM_PCMMODETYPE));

			if (snd_pcm_hw_params_set_format (playback_handle, hw_params, SND_PCM_FORMAT_S16_LE) < 0)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "snd_pcm_hw_params_set_format error\n");
			}
			// sample
			if (snd_pcm_hw_params_set_rate_near (playback_handle, hw_params, &sPCMModeParam->nSamplingRate, 0) < 0)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "snd_pcm_hw_params_set_rate_near error\n");
			}
			// channel
			if (snd_pcm_hw_params_set_channels (playback_handle, hw_params, sPCMModeParam->nChannels) < 0)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "snd_pcm_hw_params_set_channels error\n");
			}
#endif

	  /** Configure and prepare the ALSA handle */
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ, "Configuring the PCM interface\n");
			if ((err = snd_pcm_hw_params (playback_handle, hw_params)) < 0)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot set parameters (%s)\n", snd_strerror (err));
				return OMX_ErrorHardware;
			}

			if ((err = snd_pcm_prepare (playback_handle)) < 0)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "cannot prepare audio interface for use (%s)\n", snd_strerror (err));
				return OMX_ErrorHardware;
			}
		}
		break;
	case OMX_IndexParamAudioMp3:
		pAudioMp3 = (OMX_AUDIO_PARAM_MP3TYPE *) ComponentParameterStructure;
		/*Check Structure Header and verify component state */
		omxErr =
			omx_base_component_ParameterSanityCheck (hComponent, pAudioMp3->nPortIndex, pAudioMp3,
													 sizeof (OMX_AUDIO_PARAM_MP3TYPE));
		if (omxErr != OMX_ErrorNone)
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Parameter Check Error=%x\n", __func__, omxErr);
			break;
		}
		break;


	default:	/*Call the base component function */
		return omx_base_component_SetParameter (hComponent, nParamIndex, ComponentParameterStructure);
	}
	return omxErr;
}

OMX_ERRORTYPE omx_wfdalsasink_component_GetParameter (OMX_IN OMX_HANDLETYPE hComponent,
												   OMX_IN OMX_INDEXTYPE nParamIndex,
												   OMX_INOUT OMX_PTR ComponentParameterStructure)
{
	OMX_AUDIO_PARAM_PORTFORMATTYPE *pAudioPortFormat;
	OMX_OTHER_PARAM_PORTFORMATTYPE *pOtherPortFormat;
	OMX_ERRORTYPE err = OMX_ErrorNone;
	OMX_AUDIO_CONFIG_INFOTYPE *info;

	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *) hComponent;
	omx_wfdalsasink_component_PrivateType *omx_wfdalsasink_component_Private = openmaxStandComp->pComponentPrivate;
	omx_base_audio_PortType *pPort =
		(omx_base_audio_PortType *) omx_wfdalsasink_component_Private->ports[OMX_BASE_SINK_INPUTPORT_INDEX];
	omx_base_clock_PortType *pClockPort = (omx_base_clock_PortType *) omx_wfdalsasink_component_Private->ports[1];
	if (ComponentParameterStructure == NULL)
	{
		return OMX_ErrorBadParameter;
	}
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "   Getting parameter %i\n", nParamIndex);
	/* Check which structure we are being fed and fill its header */
	switch (nParamIndex)
	{
	case OMX_IndexParamAudioInit:
		if ((err = checkHeader (ComponentParameterStructure, sizeof (OMX_PORT_PARAM_TYPE))) != OMX_ErrorNone)
		{
			break;
		}
		memcpy (ComponentParameterStructure, &omx_wfdalsasink_component_Private->sPortTypesParam[OMX_PortDomainAudio],
				sizeof (OMX_PORT_PARAM_TYPE));
		break;
	case OMX_IndexParamOtherInit:
		if ((err = checkHeader (ComponentParameterStructure, sizeof (OMX_PORT_PARAM_TYPE))) != OMX_ErrorNone)
		{
			break;
		}
		memcpy (ComponentParameterStructure, &omx_wfdalsasink_component_Private->sPortTypesParam[OMX_PortDomainOther],
				sizeof (OMX_PORT_PARAM_TYPE));
		break;
	case OMX_IndexParamAudioPortFormat:
		pAudioPortFormat = (OMX_AUDIO_PARAM_PORTFORMATTYPE *) ComponentParameterStructure;
		if ((err = checkHeader (ComponentParameterStructure, sizeof (OMX_AUDIO_PARAM_PORTFORMATTYPE))) != OMX_ErrorNone)
		{
			break;
		}
		if (pAudioPortFormat->nPortIndex < 1)
		{
			memcpy (pAudioPortFormat, &pPort->sAudioParam, sizeof (OMX_AUDIO_PARAM_PORTFORMATTYPE));
		}
		else
		{
			return OMX_ErrorBadPortIndex;
		}
		break;
	case OMX_IndexParamAudioPcm:
		if (((OMX_AUDIO_PARAM_PCMMODETYPE *) ComponentParameterStructure)->nPortIndex !=
			omx_wfdalsasink_component_Private->sPCMModeParam.nPortIndex)
		{
			return OMX_ErrorBadParameter;
		}
		if ((err = checkHeader (ComponentParameterStructure, sizeof (OMX_AUDIO_PARAM_PCMMODETYPE))) != OMX_ErrorNone)
		{
			break;
		}
		memcpy (ComponentParameterStructure, &omx_wfdalsasink_component_Private->sPCMModeParam,
				sizeof (OMX_AUDIO_PARAM_PCMMODETYPE));
		break;
	case OMX_IndexParamOtherPortFormat:

		pOtherPortFormat = (OMX_OTHER_PARAM_PORTFORMATTYPE *) ComponentParameterStructure;

		if ((err = checkHeader (ComponentParameterStructure, sizeof (OMX_OTHER_PARAM_PORTFORMATTYPE))) != OMX_ErrorNone)
		{
			break;
		}
		if (pOtherPortFormat->nPortIndex == 1)
		{
			memcpy (pOtherPortFormat, &pClockPort->sOtherParam, sizeof (OMX_OTHER_PARAM_PORTFORMATTYPE));
		}
		else
		{
			return OMX_ErrorBadPortIndex;
		}
		break;

	default:	/*Call the base component function */
		return omx_base_component_GetParameter (hComponent, nParamIndex, ComponentParameterStructure);
	}
	return err;
}

void audio_adma_contorl_onoff(int onoff)
{
	int fd = -1;
	struct tcc_audio_ioctl a_ioctl;

        fd = open(DEV_NAME, 0);
        if (fd == -1) {
            TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR,"Can't open device file : %s\n", DEV_NAME);
            return;
        }

	if(onoff)
	{
		/* ADMA START (PLAYBACK) */
		a_ioctl.audio_num = AUDIO_PLAYBACK_CTRL_ID;
		ioctl(fd, IOCTL_ADMA_START, &a_ioctl);
	}else{
		/* ADMA STOP (PLAYBACK) */
		a_ioctl.audio_num = AUDIO_PLAYBACK_CTRL_ID;
		ioctl(fd, IOCTL_ADMA_STOP, &a_ioctl);
	}
	close(fd);
}


