/**
  @file component/openmax_wfdalsasink_component/omx_wfdalsasink_component.h
 
  OpenMAX ALSA sink component. This component is an audio sink that uses ALSA library.

  Copyright (C) 2007-2008  STMicroelectronics
  Copyright (C) 2007-2008 Nokia Corporation and/or its subsidiary(-ies).
  Copyright (C) 2015 Telechips Inc.

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  $Date: 2009/03/10 13:33:28 $
  Revision $Rev: 558 $
  Author $Author: B060934 $
  revised by B060955.
*/


#ifndef _OMX_WFDALSASINK_COMPONENT_H_
#define _OMX_WFDALSASINK_COMPONENT_H_

#include <OMX_Types.h>
#include <OMX_Component.h>
#include <OMX_Core.h>
#include <OMX_Audio.h>
#include <pthread.h>
#include <omx_base_sink.h>
#include <alsa/asoundlib.h>
#include "tcc_file.h"
#include "tcc_memory.h"

#include <sys/ioctl.h>
#include "tcc_audio_ioctl.h"


/** Alsasinkport component private structure.
 * see the define above
 * @param sPCMModeParam Audio PCM specific OpenMAX parameter  
 * @param AudioPCMConfigured boolean flag to check if the audio has been configured 
 * @param playback_handle ALSA specif handle for audio player
 * @param xScale the scale of the media clock
 * @param eState the state of the media clock
 * @param hw_params ALSA specif hardware parameters 
 */
DERIVEDCLASS(omx_wfdalsasink_component_PrivateType, omx_base_sink_PrivateType)
#define omx_wfdalsasink_component_PrivateType_FIELDS omx_base_sink_PrivateType_FIELDS \
  OMX_AUDIO_PARAM_PCMMODETYPE   sPCMModeParam; \
  char                          AudioPCMConfigured;  \
  snd_pcm_t*                    playback_handle;  \
  OMX_S32                       xScale; \
  OMX_TIME_CLOCKSTATE           eState; \
  snd_pcm_hw_params_t*          hw_params;\
  snd_pcm_sw_params_t*          sw_params; \ 
  int						    alsa_configure_request; \
  int						    alsa_reopen_request; \
  int						    alsa_open_mode; \
  volatile   unsigned int  		checkseekend; \
  int							alsa_status; \
  int 							fast_stop; \
  int 							flush_input_buffer; \ 
  int						    broadcastflag;\
  int						    slow_times; \
  int						    underrun_count; \
  OMX_TICKS					    underrun_time; \
  OMX_TICKS					    prev_pts; \
  OMX_TICKS					    current_pts; \
  OMX_U32					    pcm_sum; \
  OMX_U32				        sync_abort; \
  OMX_TICKS					    old_pts; \
  int						    irregular_pts_count; \  
  OMX_U32					    after_pause; \
  pthread_mutex_t 			    get_audio_pts_mutex;  \
  pthread_mutex_t 			    audio_alsa_mutex;  \
  tsem_t*       			    StopSyncSem; \
  OMX_U32				        platform; \
  OMX_U32				        alsa_setting; \
  const char*				    pcm_handle_name; \
  OMX_BOOL						alsa_write_start;\
  OMX_BOOL                      Use_Buffering_Mode;\
  OMX_BOOL						supprot_volume_control;\
  float                         gain;\
  OMX_S16*                      pcm_buffer_for_volume_ctrl;\  
  int						    ptsview;
ENDCLASS(omx_wfdalsasink_component_PrivateType)

/* Component private entry points declaration */
OMX_ERRORTYPE omx_wfdalsasink_component_Constructor(OMX_COMPONENTTYPE *openmaxStandComp,OMX_STRING cComponentName);
OMX_ERRORTYPE omx_wfdalsasink_component_Destructor(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdalsasink_component_MessageHandler(OMX_COMPONENTTYPE* openmaxStandComp, internalRequestMessageType *message);
OMX_ERRORTYPE omx_wfdalsasink_component_Init(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdalsasink_component_Deinit(OMX_COMPONENTTYPE *openmaxStandComp);


void omx_wfdalsasink_component_BufferMgmtCallback(
  OMX_COMPONENTTYPE *openmaxStandComp,
  OMX_BUFFERHEADERTYPE* inputbuffer);

OMX_ERRORTYPE omx_wfdalsasink_component_port_SendBufferFunction(
  omx_base_PortType *openmaxStandPort, 
  OMX_BUFFERHEADERTYPE* pBuffer);

/* to handle the communication at the clock port */
OMX_BOOL omx_wfdalsasink_component_ClockPortHandleFunction(
  omx_wfdalsasink_component_PrivateType* omx_wfdalsasink_component_Private,
  OMX_BUFFERHEADERTYPE* inputbuffer);

OMX_ERRORTYPE omx_wfdalsasink_component_GetParameter(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nParamIndex,
  OMX_INOUT OMX_PTR ComponentParameterStructure);

OMX_ERRORTYPE omx_wfdalsasink_component_SetParameter(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nParamIndex,
  OMX_IN  OMX_PTR ComponentParameterStructure);

OMX_ERRORTYPE omx_wfdalsasink_component_GetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_INOUT OMX_PTR pComponentConfigStructure) ;

  OMX_ERRORTYPE omx_wfdalsasink_component_SetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_INOUT OMX_PTR pComponentConfigStructure) ;

void* omx_base_sink_alsa_BufferMgmtFunction (void* param);
OMX_ERRORTYPE omx_wfdalsasink_component_port_FlushProcessingBuffers(omx_base_PortType *openmaxStandPort);
void omx_alsasink_dynamix_setting(OMX_COMPONENTTYPE *openmaxStandComp);
void audio_adma_contorl_onoff(int onoff);

#endif
