/**
  @file component/openmax_wfdfbdev_component/omx_wfdfbdev_component.h

  OpenMAX FBDEV sink component. This component is a video sink that copies
  data to a Linux framebuffer device.

  Originally developed by Peter Littlefield
  Copyright (C) 2007-2008  STMicroelectronics and Agere Systems
  Copyright (C) 2007-2008 Nokia Corporation and/or its subsidiary(-ies)
  Copyright (C) 2009-2010 Telechips Inc

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  $Date: 2009/03/10 13:33:28 $
  Revision $Rev: 554 $
  Author $Author: B060934 $
  revised by B060955.
*/


#ifndef _OMX_WFDFBDEV_SINK_COMPONENT_H_
#define _OMX_WFDFBDEV_SINK_COMPONENT_H_

#include <OMX_Types.h>
#include <OMX_Component.h>
#include <OMX_Core.h>
#include <OMX_Video.h>
#include <OMX_IVCommon.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <sys/poll.h>

#include <tcc_video_common.h>
#include <omx_base_video_port.h>
#include <omx_base_clock_port.h>
#include <omx_base_sink.h>
#include <linux/fb.h>
#include "tcc_file.h"
#include "tcc_memory.h"

#include <linux/fb.h>
#include <linux/videodev2.h>
#include "tccfb_ioctrl.h"

#include "tcc_overlay_ioctl.h"
#include "pmap.h"

#include "pmap.h"
#include "tcc_aspect_ratio.h"


#define TCC_MEMORY_DEV_NAME			"/dev/mem"




/**  Filename of devnode for framebuffer device
  *  Should somehow be passed from client
  */

/* for scannig mode */
enum {
	FBSINK_PROGRESSIVE_SCAN = 1,
	FBSINK_INTERLACED_SCAN,
};

/* for dec output color format */
enum {
	DEC_OUT_YUV420INT = 0,
	DEC_OUT_YUV420SEP = 1,
	DEC_OUT_MAX
};

/** FBDEV sink port component port structure.
  */
DERIVEDCLASS(omx_wfdfbdev_sink_component_PortType, omx_base_video_PortType)
#define omx_wfdfbdev_sink_component_PortType_FIELDS omx_base_video_PortType_FIELDS \
  /** @param omxConfigCrop Crop rectangle of image */ \
  OMX_CONFIG_RECTTYPE omxConfigCrop; \
  /** @param omxConfigRotate Set rotation angle of image */ \
  OMX_CONFIG_ROTATIONTYPE omxConfigRotate; \
  /** @param omxConfigMirror Set mirroring of image */ \
  OMX_CONFIG_MIRRORTYPE omxConfigMirror; \
  /** @param omxConfigScale Set scale factors */ \
  OMX_CONFIG_SCALEFACTORTYPE omxConfigScale; \
  /** @param omxConfigOutputPosition Top-Left offset from intermediate buffer to output buffer */ \
  OMX_CONFIG_POINTTYPE omxConfigOutputPosition;
ENDCLASS(omx_wfdfbdev_sink_component_PortType)

/** FBDEV sink port component private structure.
  * see the define above
  * @param fd The file descriptor for the framebuffer 
  * @param vscr_info The fb_var_screeninfo structure for the framebuffer 
  * @param fscr_info The fb_fix_screeninfo structure for the framebuffer
  * @param scr_data Pointer to the mmapped memory for the framebuffer 
  * @param fbpxlfmt frame buffer pixel format
  * @param fbwidth frame buffer display width 
  * @param fbheight frame buffer display height 
  * @param fbbpp frame buffer pixel depth
  * @param fbstride frame buffer display stride 
  * @param xScale the scale of the media clock
  * @param eState the state of the media clock
  * @param product frame buffer memory area 
  * @param frameDropFlag the flag active on scale change indicates that frames are to be dropped 
  * @param dropFrameCount counts the number of frames dropped 
  */
DERIVEDCLASS(omx_wfdfbdev_sink_component_PrivateType, omx_base_sink_PrivateType)
#define omx_wfdfbdev_sink_component_PrivateType_FIELDS omx_base_sink_PrivateType_FIELDS \
  OMX_S32			fd; \
  struct                       fb_var_screeninfo vscr_info; \
  struct                       fb_fix_screeninfo fscr_info; \
  unsigned char                *scr_ptr; \
  OMX_COLOR_FORMATTYPE         fbpxlfmt; \
  OMX_U32                      fbstartX; \
  OMX_U32                      fbstartY; \  
  OMX_U32                      fbwidth; \
  OMX_U32                      fbheight; \
  OMX_U32                      fbbpp; \
  OMX_S32                      fbstride; \
  OMX_S32                      xScale; \
  OMX_TIME_CLOCKSTATE          eState; \
  OMX_U32                      product;\
  OMX_BOOL                     frameDropFlag;\
  tsem_t*       			compSyncSem; \
  OMX_BOOL      		compReady; \  
  OMX_S32               dropFrameCount; \
  OMX_U32				v_offset;\
  OMX_U32		  		h_offset;\
  OMX_U32				fbscreenmode;  \
  OMX_U32				fbzoomscale;  \
  OMX_U32				fbpanscan_direction;  \
  OMX_U32				fbscanmode; \
  OMX_U32  				dec_color_mode; \
  OMX_U32  				video_codec; \
  OMX_S32				is_actual_resolution;\  
  OMX_U32				doDiscardFrame; \
  OMX_U32				doSendKeyFrameCallback; \ 
  OMX_U32				is_video_player; \
  OMX_U32				is_fbinit; \
  OMX_U32 				fast_stop; \
  OMX_S32				is_seek_start; \
  OMX_U32				step_count; \
  OMX_U32				discard_count;\
  OMX_U32				pause_scrmod; \
  OMX_U32				fps; \
  float					fRealSpeedLimit; \
  OMX_S32				fb0_fd; \
  OMX_S32				scaler0_fd; \
  OMX_S32				overlay_fd; \
  OMX_S32				g2d_fd; \
  OMX_S32				viqe_fd; \
  OMX_S32				composite_fd; \
  OMX_S32				memory_fd;	\
  pmap_t 				pmap_g2d; \
  pmap_t				pmap_overlay; \
  pmap_t				pmap_viqe; \
  void *				overlay_map_ptr; \
  void *				viqe_map_ptr; \
  overlay_config_t 		overlay_info;	\  
  OMX_U32				overlay_config_check; \
  OMX_S32				zoom_enable;	\
  OMX_CONFIG_RECTTYPE	zoom_config;	\
  OMX_U32				cable_posx; \
  OMX_U32				cable_posy; \
  OMX_U32				cable_width; \
  OMX_U32				cable_height; \
  OMX_U32				cable_mode; \
  OMX_S32				ptsview; \
  OMX_U32				overlay_used;	\
  OMX_U32				vsync_output_mode; \
  pthread_mutex_t 		fb_stop_mutex; \  
  OMX_U32				output_type; \
  OMX_BOOL 			    display_enable; \
  OMX_U32				overlay_setting_done; \
  OMX_TICKS  			TimeSourceDelta; \
  OMX_TICKS  			LastAudioTime; \
  OMX_TICKS  			LastVideoDisplayTime; \
  OMX_BOOL  			VideoDelay; \
  OMX_TICKS             First_Video_arrive_time;\
  OMX_TICKS             Syatem_base_clocks;\  
  OMX_TICKS			    Media_base_clocks;\
  OMX_BOOL              Use_Buffering_Mode;\
  OMX_TICKS             delta_value;\  
  OMX_U32				vsync_seek_start;
ENDCLASS(omx_wfdfbdev_sink_component_PrivateType)

/* Component private entry points declaration */
OMX_ERRORTYPE omx_wfdfbdev_sink_component_Init(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdfbdev_sink_component_Constructor(OMX_COMPONENTTYPE *openmaxStandComp,OMX_STRING cComponentName);
OMX_ERRORTYPE omx_wfdfbdev_sink_component_Destructor(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdfbdev_sink_component_Initialize(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdfbdev_sink_component_Deinit(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdfbdev_sink_component_MessageHandler(OMX_COMPONENTTYPE* , internalRequestMessageType*);

void omx_wfdfbdev_sink_component_BufferMgmtCallback(
  OMX_COMPONENTTYPE *openmaxStandComp,
  OMX_BUFFERHEADERTYPE* pInputBuffer);

OMX_ERRORTYPE omx_wfdfbdev_sink_component_SetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_IN  OMX_PTR pComponentConfigStructure);

OMX_ERRORTYPE omx_wfdfbdev_sink_component_GetParameter(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nParamIndex,
  OMX_INOUT OMX_PTR ComponentParameterStructure);

OMX_ERRORTYPE omx_wfdfbdev_sink_component_SetParameter(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nParamIndex,
  OMX_IN  OMX_PTR ComponentParameterStructure);

OMX_ERRORTYPE omx_wfdfbdev_sink_component_GetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_INOUT OMX_PTR pComponentConfigStructure);

/** function prototypes of some internal functions */

/** finds OpenMAX standard pixel format from screen info */
OMX_COLOR_FORMATTYPE find_omx_pxlfmt(struct fb_var_screeninfo *vscr_info);

/** finds FFmpeg supported pixel format from input OpenMAX standard pixel format */
enum PixelFormat find_ffmpeg_pxlfmt(OMX_COLOR_FORMATTYPE omx_pxlfmt);

/** finds video stride  from input dimension and color format */
OMX_S32 fb_calcStride(OMX_U32 width, OMX_COLOR_FORMATTYPE omx_pxlfmt);

/** image copy function */
void fb_omx_img_copy(OMX_U8* src_ptr, OMX_S32 src_stride, OMX_U32 src_width, OMX_U32 src_height, 
                  OMX_S32 src_offset_x, OMX_S32 src_offset_y,
                  OMX_U8* dest_ptr, OMX_S32 dest_stride, OMX_U32 dest_width,  OMX_U32 dest_height, 
                  OMX_S32 dest_offset_x, OMX_S32 dest_offset_y, 
                  OMX_S32 cpy_width, OMX_U32 cpy_height, OMX_COLOR_FORMATTYPE colorformat,OMX_COLOR_FORMATTYPE fbpxlfmt);

#endif
