/****************************************************************************
 *   FileName    : tcc_aspect_ratio.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/

/******************************************************************************
* include 
******************************************************************************/
#include "tcc_aspect_ratio.h"
#include "main.h"

/******************************************************************************
* locals
******************************************************************************/


/******************************************************************************
*	FUNCTIONS			: TCC_Aspect_Ratio_Calculate_Image
*	SYNOPSIS			:
*	EXTERNAL EFFECTS	:
*	PARAMETERS			:
*	RETURNS				:
*	ERRNO				:
******************************************************************************/
void TCC_Aspect_Ratio_Calculate_Image (int OriginImg_Width, int OriginImg_Height, int OutBuff_Width, int OutBuff_Height,
						   int *Out_Width, int *Out_Height, int *Out_Offset_Horizontal, int *Out_Offset_Vertical,
						   unsigned int ScreenMode)
{

	int DecodedImg_Offset_Horizontal, DecodedImg_Offset_Vertical;
	int DecodedImg_Width, DecodedImg_Height;

	float src_aspect_ratio;
	float dst_aspect_ratio;
	float temp_ratio;

	/*Source*/
	src_aspect_ratio =  (float)OriginImg_Width/(float)OriginImg_Height;
	/*Destination*/
	dst_aspect_ratio =  (float)OutBuff_Width/(float)OutBuff_Height;

	if (ScreenMode == FBSINK_MODE_STANDARD)
	{
		if(src_aspect_ratio>dst_aspect_ratio)
		{
			DecodedImg_Offset_Horizontal = 0;
			DecodedImg_Width = OutBuff_Width;
			DecodedImg_Height = (DecodedImg_Width* OriginImg_Height)/ OriginImg_Width;
			DecodedImg_Height = (DecodedImg_Height >> 4 )<< 4; // 16 pixel unit
			DecodedImg_Offset_Vertical = (OutBuff_Height - DecodedImg_Height)>>1;
		}	
		else
		{
			DecodedImg_Offset_Vertical = 0;
			DecodedImg_Height = OutBuff_Height;
			DecodedImg_Width = (DecodedImg_Height*OriginImg_Width)/OriginImg_Height;
			DecodedImg_Width = (DecodedImg_Width >> 4 )<< 4; // 16 pixel unit	
			DecodedImg_Offset_Horizontal = (OutBuff_Width -DecodedImg_Width)>>1;
		}	
	}
	else if(ScreenMode == FBSINK_MODE_ZOOM)
	{
		if(src_aspect_ratio>dst_aspect_ratio)
		{
			DecodedImg_Offset_Vertical = 0;
			DecodedImg_Height = OriginImg_Height;
			DecodedImg_Width = (OutBuff_Width*OriginImg_Height)/OutBuff_Height;
			DecodedImg_Width = (DecodedImg_Width >> 4 )<< 4; // 16 pixel unit			
			if(OriginImg_Width>DecodedImg_Width)
				DecodedImg_Offset_Horizontal = (OriginImg_Width-DecodedImg_Width)>>1;
			else
				DecodedImg_Offset_Horizontal = (DecodedImg_Width-OriginImg_Width)>>1;

		}	
		else
		{
			DecodedImg_Offset_Horizontal = 0;
			DecodedImg_Width = OriginImg_Width;
			DecodedImg_Height = (OutBuff_Height* OriginImg_Width)/ OutBuff_Width;
			DecodedImg_Height = (DecodedImg_Height >> 4 )<< 4; // 16 pixel unit
			if(OriginImg_Height>DecodedImg_Height)
				DecodedImg_Offset_Vertical = (OriginImg_Height - DecodedImg_Height)>>1;
			else
				DecodedImg_Offset_Vertical = (DecodedImg_Height - OriginImg_Height)>>1;
		}
	}
	else if(ScreenMode == FBSINK_MODE_ORIGINAL)
	{
		if(src_aspect_ratio>dst_aspect_ratio)
		{
			DecodedImg_Width = OutBuff_Width;
			DecodedImg_Width = (DecodedImg_Width >> 4 )<< 4; // 16 pixel unit
			DecodedImg_Height = (DecodedImg_Width* OriginImg_Height)/ OriginImg_Width;
			DecodedImg_Height = (DecodedImg_Height +1) & 0xfffffffe;
			DecodedImg_Offset_Horizontal = 0;
			DecodedImg_Offset_Vertical = (OutBuff_Height - DecodedImg_Height)>>1;
		}
		else if(src_aspect_ratio==dst_aspect_ratio)
		{
			DecodedImg_Width = OutBuff_Width;
			DecodedImg_Width = (DecodedImg_Width >> 4 )<< 4; // 16 pixel unit		
			DecodedImg_Height = OutBuff_Height;
			DecodedImg_Height = (DecodedImg_Height +1) & 0xfffffffe;	
			DecodedImg_Offset_Horizontal = (OutBuff_Width -DecodedImg_Width)>>1;
			DecodedImg_Offset_Vertical = (OutBuff_Height -DecodedImg_Height)>>1;
		}
		else if(src_aspect_ratio<dst_aspect_ratio)
		{
			DecodedImg_Height = OutBuff_Height;
			DecodedImg_Height = (DecodedImg_Height +1) & 0xfffffffe;
			DecodedImg_Width = (DecodedImg_Height*OriginImg_Width)/OriginImg_Height;
			DecodedImg_Width = (DecodedImg_Width >> 4 )<< 4; // 16 pixel unit	
			DecodedImg_Offset_Vertical = 0;
			DecodedImg_Offset_Horizontal = (OutBuff_Width -DecodedImg_Width)>>1;
		}
		else
		{
			;
		}
	}
	else if(ScreenMode == FBSINK_MODE_16_9)
	{
		DecodedImg_Offset_Horizontal = 0;
		DecodedImg_Width = OutBuff_Width;
		DecodedImg_Height = (DecodedImg_Width*9)>>4;
		DecodedImg_Height = (DecodedImg_Height >> 4 )<< 4; // 16 pixel unit
		DecodedImg_Offset_Vertical = (OutBuff_Height - DecodedImg_Height)>>1;
	}
	else if(ScreenMode == FBSINK_MODE_4_3)
	{
		if(dst_aspect_ratio > (4.0f/3.0f))
		{
			DecodedImg_Offset_Vertical = 0;
			DecodedImg_Height = OutBuff_Height;
			DecodedImg_Width = (DecodedImg_Height<<2)/3;
			DecodedImg_Width = (DecodedImg_Width >> 4 )<< 4; // 16 pixel unit	
			DecodedImg_Offset_Horizontal = (OutBuff_Width -DecodedImg_Width)>>1;
		}
		else
		{
			DecodedImg_Offset_Horizontal = 0;
			DecodedImg_Width = OutBuff_Width;
			DecodedImg_Height = (DecodedImg_Width*3)>>2;
			DecodedImg_Height = (DecodedImg_Height >> 4 )<< 4; // 16 pixel unit
			DecodedImg_Offset_Vertical = (OutBuff_Height - DecodedImg_Height)>>1;
		}	
	}
	else if(ScreenMode == FBSINK_MODE_FULL)
	{
		DecodedImg_Offset_Vertical = 0;
		DecodedImg_Height = OutBuff_Height;
		DecodedImg_Width = OutBuff_Width;
		DecodedImg_Width = (DecodedImg_Width >> 4 )<< 4; // 16 pixel unit	
		DecodedImg_Offset_Horizontal = (OutBuff_Width -DecodedImg_Width)>>1;
	}
	else
	{
		TCC_DEBUG_M(TCC_DEBUG_COMMON, TCC_DEBUG_LEV_ERR,"[%s:%d] : Not Available Screen Mode Index!!! ",__FUNCTION__,__LINE__);
	}	
	
	*Out_Width = DecodedImg_Width;
	*Out_Height = DecodedImg_Height;
	*Out_Offset_Horizontal = DecodedImg_Offset_Horizontal;
	*Out_Offset_Vertical = DecodedImg_Offset_Vertical;
}


#if 0
static unsigned int g_current_ZoomScale=0;
static unsigned int g_current_offsetH=0;
static unsigned int g_current_offsetV=0;
static unsigned int g_current_CalculatedW=0;
static unsigned int g_current_CalculatedH=0;
/******************************************************************************
*	FUNCTIONS			: Get_dvdzoom_Resizeimage
*	SYNOPSIS			:
*	EXTERNAL EFFECTS	:
*	PARAMETERS			:
*	RETURNS				:
*	ERRNO				:
******************************************************************************/
void Get_dvdzoom_Resizeimage(int OriginImg_Width,int OriginImg_Height, int ZoomScale,
									int *Out_width, int *Out_Height, int *Out_Offset_H, int *Out_Offset_V)
{
	unsigned int temp_imgsize_H;
	unsigned int temp_imgsize_V;
	
	if(ZoomScale==2)  // X2 Zoom
	{	
		temp_imgsize_H=(float)OriginImg_Width/1.414;
		temp_imgsize_V=(float)OriginImg_Height/1.414;
		*Out_width= (temp_imgsize_H >> 4 )<< 4; // 16 pixel unit	
		*Out_Height= (temp_imgsize_V >> 4 )<< 4; // 16 pixel unit
		*Out_Offset_H=(OriginImg_Width-*Out_width)/2;	
		*Out_Offset_V=(OriginImg_Height-*Out_Height)/2;					
		return;
	}
	else if(ZoomScale==3)	// X3 Zoom
	{
		temp_imgsize_H=(float)OriginImg_Width/1.732;
		temp_imgsize_V=(float)OriginImg_Height/1.732;
		*Out_width= (temp_imgsize_H >> 4 )<< 4; // 16 pixel unit	
		*Out_Height= (temp_imgsize_V >> 4 )<< 4; // 16 pixel unit
		*Out_Offset_H=(OriginImg_Width-*Out_width)/2;	
		*Out_Offset_V=(OriginImg_Height-*Out_Height)/2;
		return;
	}
	else if(ZoomScale==4)  // X4 Zoom
	{
		temp_imgsize_H=OriginImg_Width/2;
		temp_imgsize_V=OriginImg_Height/2;
		*Out_width= (temp_imgsize_H >> 4 )<< 4; // 16 pixel unit	
		*Out_Height= (temp_imgsize_V >> 4 )<< 4; // 16 pixel unit
		*Out_Offset_H=(OriginImg_Width-*Out_width)/2;	
		*Out_Offset_V=(OriginImg_Height-*Out_Height)/2;
		return;
	}
	else if(ZoomScale==9)	// X9 Zoom
	{
		temp_imgsize_H=OriginImg_Width/3;
		temp_imgsize_V=OriginImg_Height/3;
		*Out_width= (temp_imgsize_H >> 4 )<< 4; // 16 pixel unit	
		*Out_Height= (temp_imgsize_V >> 4 )<< 4; // 16 pixel unit
		*Out_Offset_H=(OriginImg_Width-*Out_width)/2;	
		*Out_Offset_V=(OriginImg_Height-*Out_Height)/2;
		return;
	}
	else
	{	
		
		printf("[%s:%d] : --Invalid Scale Value--",__FUNCTION__,__LINE__);
		*Out_width=OriginImg_Width;
		*Out_Height=OriginImg_Height;
		*Out_Offset_H=0;
		*Out_Offset_V=0;
		return;
	}
}
/******************************************************************************
*	FUNCTIONS			: ClaculatePanscanOffset
*	SYNOPSIS			:
*	EXTERNAL EFFECTS	:
*	PARAMETERS			:
*	RETURNS				:
*	ERRNO				:
******************************************************************************/
void ClaculatePanscanOffset(int OriginImg_Width,int OriginImg_Height,int Calculated_width, int Calculated_Height,
								int scan_start_offsetH, int scan_start_offsetV, int  direction, int *Out_Offset_H, int *Out_Offset_V)
{
	switch(direction)
	{ 
		case 0: //UP
			*Out_Offset_H=g_current_offsetH;
			*Out_Offset_V=g_current_offsetV-((scan_start_offsetV/5)+2);
			if(*Out_Offset_V<0)
			{
				*Out_Offset_V=0;	
			}
		break;
			
		case 1://Down
			*Out_Offset_H=g_current_offsetH;
			*Out_Offset_V=g_current_offsetV+((scan_start_offsetV/5)+2);
			if((OriginImg_Height-Calculated_Height)<*Out_Offset_V)
			{
				*Out_Offset_V=OriginImg_Height-Calculated_Height;	
			}
		break;

		case 2://LEFT
			*Out_Offset_H=g_current_offsetH-(scan_start_offsetH/5);
			if(*Out_Offset_H<0)
			{
				*Out_Offset_H=0;
			}
			*Out_Offset_V=g_current_offsetV;
		break;

		case 3://Right
			*Out_Offset_H=g_current_offsetH+((scan_start_offsetH/5)+1);
			if((OriginImg_Width-Calculated_width)<*Out_Offset_H)
			{
				*Out_Offset_H=OriginImg_Width-Calculated_width;	
			}
			*Out_Offset_V=g_current_offsetV;
		break;
	}
}

#endif
