/*
  @file component/openmax_wfdvdec_component/omx_wfdvdec_component.c

  OpenMAX Video decoder component. This component implements H.264 video decoder.

  Copyright (C) 2008  STMicroelectronics
  Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
  Copyright (C) 2015 Telechips Inc.

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  $Date: 2009/03/10 13:33:28 $
  Revision $Rev: 554 $
  Author $Author: B060934 $
  revised by B060955.
*/

#include <omxcore.h>
#include <omx_base_video_port.h>
#include <omx_wfdvdec_component.h>
#include <OMX_Video.h>

#include <lcd_resolution.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/fb.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include "tcc_time.h"
#include <dlfcn.h>

#define TCC_MEMORY_DEV_NAME	"/dev/mem"

/** Maximum Number of Video Component Instance*/
#define MAX_COMPONENT_VIDEODEC 4

#define ABS(A)	( (A) > 0 ? (A) : (-A) )
/** Counter of Video Component Instance*/

static OMX_U32 noVideoDecInstance = 0;

/** The output decoded color format */
#define OUTPUT_DECODED_COLOR_FMT OMX_COLOR_FormatYUV420Planar

/** define the max input buffer size */
static int sequence_init_done=0;
static int vpu_error_state = 0;
static int g_buffer_full_count = 0;

static int		g_number_of_vpu_error = 0;
static int		g_number_of_vpu_macro_error = 0;
/** internal function to set codec related parameters in the private type structure
  */

static int gvdecerrcnt = 0;
static int gvdecerr_sendevent = 0;


pthread_mutex_t vpu_index_mutex;

#define	DISPLAY_INDEX_QUEUE_SIZE	64
static int g_disidx_queue[DISPLAY_INDEX_QUEUE_SIZE]; // 0~31
static int g_disidx_queue_read;
static int g_disidx_queue_write;
static int g_disidx_queue_entry = 0;

//#define	USE_DISP_INDEX_CHK_QUEUE	0

#ifdef	USE_DISP_INDEX_CHK_QUEUE
#define	DISPLAY_INDEX_CHK_QUEUE_SIZE	(DISPLAY_INDEX_QUEUE_SIZE*4)
static int g_disidx_chk_queue[DISPLAY_INDEX_QUEUE_SIZE*4]; // 0~31
static int g_disidx_chk_queue_read;
static int g_disidx_chk_queue_write;
static int g_disidx_chk_queue_entry = 0;
static int chk_dequeue_vpu_displayindex(void);
#endif


PERF_MEASURE_CHECK_GLOBAL_DEFINE(gs_PerfMeasureVDEC);

#define MIN_DISPLAY_QUEUE_SIZE			4 	/* [omx_base_port.c]DEFAULT_NUMBER_BUFFERS_PER_PORT */

enum CAPTURE_STATE_e
{
	CAPTURE_END = 0,
	CAPTURE_READY,
	CAPTURE_STARTED,
};

typedef void*(*tcc_cdk_vdec_alloc_instance)(int);
typedef cdk_core_t*(*tcc_cdk_get_CoreAddr)(void);
typedef int(*tcc_cdk_get_video_codec_type)(int);
typedef int(*tcc_cdk_get_audio_codec_type)(int);
typedef cdk_func_t* (*tcc_cdk_get_decode_function)(unsigned int);
typedef void(*tcc_cdk_init_ts_pts_control)(int);
typedef void(*tcc_cdk_display_pic_info)(int , void *, void *, void *);
typedef unsigned char* (*tcc_cdk_get_vpu_bitstreambuf_addr)(unsigned int, void* );
typedef void(*tcc_cdk_print_user_data)(unsigned char* );

static tcc_cdk_vdec_alloc_instance sym_tcc_cdk_vdec_alloc_instance = NULL;
static tcc_cdk_get_CoreAddr sym_tcc_cdk_get_core_addr = NULL;
static tcc_cdk_get_audio_codec_type sym_tcc_cdk_get_audio_codec_type = NULL;
static tcc_cdk_get_video_codec_type sym_tcc_cdk_get_video_codec_type = NULL;
static tcc_cdk_get_decode_function sym_tcc_cdk_get_decode_function = NULL;
static tcc_cdk_init_ts_pts_control sym_tcc_cdk_init_ts_pts_control = NULL;
static tcc_cdk_display_pic_info sym_tcc_cdk_display_pic_info = NULL;
static tcc_cdk_get_vpu_bitstreambuf_addr sym_tcc_cdk_get_vpu_bitstreambuf_addr = NULL;
static tcc_cdk_print_user_data sym_tcc_cdk_print_user_data = NULL;

static void tcc_cdk_vdec_symbol_loading(void)
{
    void *pModule = NULL;

    pModule = dlopen(LIBTCCCDKCORE_SO, RTLD_LAZY); 
    if(pModule == NULL)
        printf("Could not find libcdkcore.so.");

    sym_tcc_cdk_vdec_alloc_instance = dlsym(pModule,"vdec_alloc_instance");
    if(sym_tcc_cdk_vdec_alloc_instance == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_vdec_alloc_instance \n");

    sym_tcc_cdk_get_core_addr = dlsym(pModule,"TCC_CDK_GetCoreAddr");
    if(sym_tcc_cdk_get_core_addr == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_get_core_addr \n");

    sym_tcc_cdk_get_audio_codec_type = dlsym(pModule,"get_audio_codec_type");
    if(sym_tcc_cdk_get_audio_codec_type == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_get_audio_codec_type \n");

    sym_tcc_cdk_get_video_codec_type = dlsym(pModule,"get_video_codec_type");
    if(sym_tcc_cdk_get_video_codec_type == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_get_video_codec_type \n");

    sym_tcc_cdk_get_decode_function = dlsym(pModule,"TCC_CDK_GetDECFunc");
    if(sym_tcc_cdk_get_decode_function == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_get_decode_function \n");

    sym_tcc_cdk_init_ts_pts_control = dlsym(pModule,"TCC_CDK_InitTSPTSCtrl");
    if(sym_tcc_cdk_init_ts_pts_control == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_init_ts_pts_control \n");

    sym_tcc_cdk_display_pic_info = dlsym(pModule,"disp_pic_info");
    if(sym_tcc_cdk_display_pic_info == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_display_pic_info \n");

    sym_tcc_cdk_get_vpu_bitstreambuf_addr = dlsym(pModule,"vpu_getBitstreamBufAddr");
    if(sym_tcc_cdk_get_vpu_bitstreambuf_addr == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_get_vpu_bitstreambuf_addr \n");

    sym_tcc_cdk_print_user_data = dlsym(pModule,"print_user_data");
    if(sym_tcc_cdk_print_user_data == NULL)
        printf("Could not find Symbol.. sym_tcc_cdk_print_user_data \n"); 

    dlclose(pModule);
}

static void* omx_wfdvdec_component_BufferMgmtFunction (void* param);

void display_queue_destroy(display_index_queue_t* pQueue)
{

}

void display_queue_init(display_index_queue_t* pQueue)
{
	int ii;

	pQueue->gs_nQueueFrontIndex = 0;
	pQueue->gs_nQueueRearIndex = 0;
	pQueue->gs_nQueueNumOfIndex = 0;

	for(ii=0; ii<DISPLAY_QUEUE_MAX_SIZE; ii++)
	{
		pQueue->gs_nDisplayQueue[ii] = -1;
	}
}

void display_queue_push(display_index_queue_t* pQueue, const int index)
{
	pQueue->gs_nDisplayQueue[pQueue->gs_nQueueRearIndex] = index;

	//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"[%s][%d] queue rear:%d, push idx:%d, size:%d\n", __FUNCTION__, __LINE__, gs_nQueueRearIndex, index, gs_nQueueNumOfIndex+1);

	pQueue->gs_nQueueRearIndex = (++pQueue->gs_nQueueRearIndex) % DISPLAY_QUEUE_MAX_SIZE;

	pQueue->gs_nQueueNumOfIndex++;
}

int display_queue_pop(display_index_queue_t* pQueue)
{
	int ret;

	if(pQueue->gs_nQueueNumOfIndex <= 0)
	{
		//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"[%s][%d] current size, ret -1\n", __FUNCTION__, __LINE__, gs_nQueueFrontIndex);
		return -1;
	}

	ret = pQueue->gs_nDisplayQueue[pQueue->gs_nQueueFrontIndex];
	pQueue->gs_nDisplayQueue[pQueue->gs_nQueueFrontIndex] = -1;

	//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"[%s][%d] queue front:%d, pop idx:%d, size:%d\n", __FUNCTION__, __LINE__, gs_nQueueFrontIndex, ret, gs_nQueueNumOfIndex-1);

	pQueue->gs_nQueueFrontIndex = (++pQueue->gs_nQueueFrontIndex) % DISPLAY_QUEUE_MAX_SIZE;

	if(pQueue->gs_nQueueNumOfIndex > 0)
		pQueue->gs_nQueueNumOfIndex--;

	return ret;
}

int display_queue_num(display_index_queue_t* pQueue)
{
	return pQueue->gs_nQueueNumOfIndex;
}

int display_queue_debug(display_index_queue_t* pQueue)
{
	int ii;
	int idx;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ, "[display queue] : ");
	for(ii=0; ii<pQueue->gs_nQueueNumOfIndex; ii++)
	{
		idx = (pQueue->gs_nQueueFrontIndex + ii) % DISPLAY_QUEUE_MAX_SIZE;

		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"%2d ", pQueue->gs_nDisplayQueue[idx]);
	}
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"\n");
}

int display_queue_check(display_index_queue_t* pQueue, int index)
{
	int ii;
	int tmp_index = pQueue->gs_nQueueFrontIndex;

	for(ii=0; ii<pQueue->gs_nQueueNumOfIndex; ii++)
	{
		if(pQueue->gs_nDisplayQueue[tmp_index] == index)
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"[%s][%d]Display Index Corruption!! %d\n", __FUNCTION__, __LINE__, index);
			return 1;
		}

		tmp_index = (++tmp_index) % DISPLAY_QUEUE_MAX_SIZE;
	}

	return 0;
}

static unsigned g_curr_vc1_skip_index = -1;
static unsigned g_prev_vc1_skip_index = -1;

static int	is_full_vpu_displayindex_queue ()
{
	if ( g_disidx_queue_entry == DISPLAY_INDEX_QUEUE_SIZE ) {
		return (1);
	} else {
		return (0);
	}
}

static int	is_empty_vpu_displayindex_queue ()
{
	if ( g_disidx_queue_entry <= 0 ) {
		return (1);
	} else {
		return (0);
	}
}

static void queue_vpu_displayindex(int index)
{
#if 0
	if(g_disidx_queue_read==g_disidx_queue_write)
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"vpu display index queue full!!!!\n");
#endif
	if (is_full_vpu_displayindex_queue() ) {
//	if(g_disidx_queue_entry==DISPLAY_INDEX_QUEUE_SIZE)
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR,"vpu display index queue full!!!!\n");
	}

	g_disidx_queue[g_disidx_queue_write] = index;
	g_disidx_queue_write++;
	g_disidx_queue_entry++;
	if(DISPLAY_INDEX_QUEUE_SIZE==g_disidx_queue_write)
		g_disidx_queue_write = 0;
}

static int dequeue_vpu_displayindex(void)
{
	int ret = -1;

/*
	if(g_disidx_queue_read==g_disidx_queue_write)
		return ret;
		*/
	if ( is_empty_vpu_displayindex_queue() ) return ret;
//	if ( g_disidx_queue_entry == 0 ) return ret;

	ret = g_disidx_queue[g_disidx_queue_read];
	g_disidx_queue_read++;
	g_disidx_queue_entry--;
	if(DISPLAY_INDEX_QUEUE_SIZE==g_disidx_queue_read)
		g_disidx_queue_read = 0;

	#ifdef	USE_DISP_INDEX_CHK_QUEUE
	{
		int	nChkIdx;
		nChkIdx = chk_dequeue_vpu_displayindex ();
		if ( nChkIdx & 0x40000000 ) {
			// VC1 Skip Frame
			nChkIdx &= ~0x40000000;
			if ( ret != nChkIdx ) {
				//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"%s : %d : disp_queue read mismatch error\n",__func__, __LINE__);
				//while (1);
			} else {
				if ( nChkIdx == g_prev_vc1_skip_index ) {
					return ret;
				} else {
					g_prev_vc1_skip_index = nChkIdx;
					return (g_prev_vc1_skip_index);
				}
			}
		} else {
			if ( ret != nChkIdx ) {
				//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"%s : %d : disp_queue read mismatch error\n",__func__, __LINE__);
				//while (1);
			}
		}
	}
	#endif

	return ret;
}

void display_vpu_index_queue_entry (void)
{
	unsigned int new_idx;
	unsigned int i;

	#ifdef	USE_DISP_INDEX_CHK_QUEUE

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ," vpu queue entry: window [");
	for ( i = 0 ; i < g_disidx_chk_queue_entry ; i++ ) {
		new_idx = g_disidx_chk_queue_read + i;
		if ( new_idx >= DISPLAY_INDEX_CHK_QUEUE_SIZE ) {
			new_idx -= DISPLAY_INDEX_CHK_QUEUE_SIZE;
		}
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ," %02d", g_disidx_chk_queue[new_idx]);
	}
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"]\n");
	#endif
}

void	check_vpu_index_queue(int index)
{
	unsigned int i;
	unsigned int new_idx;
	unsigned int error;

	#ifdef	USE_DISP_INDEX_CHK_QUEUE

	if ( index < 0 ) return;

	error = 0;
	for ( i = 0 ; i < g_disidx_chk_queue_entry ; i++ ) {
		new_idx = g_disidx_chk_queue_read + i;
		if ( new_idx >= DISPLAY_INDEX_CHK_QUEUE_SIZE ) {
			new_idx -= DISPLAY_INDEX_CHK_QUEUE_SIZE;
		}
		if ( index == g_disidx_chk_queue[new_idx] ) {
			error = 1;
		}
	}
	if ( error == 0 ) return;

	//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ," vpu overwriting error : '%d' at window [", index);
	for ( i = 0 ; i < g_disidx_chk_queue_entry ; i++ ) {
		new_idx = g_disidx_chk_queue_read + i;
		if ( new_idx >= DISPLAY_INDEX_CHK_QUEUE_SIZE ) {
			new_idx -= DISPLAY_INDEX_CHK_QUEUE_SIZE;
		}
		//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ," %02d", g_disidx_chk_queue[new_idx]);
	}
	//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"]\n");
	#endif
}

#ifdef	USE_DISP_INDEX_CHK_QUEUE
static int	is_full_vpu_displayindex_chk_queue ()
{
	if ( g_disidx_chk_queue_entry == DISPLAY_INDEX_CHK_QUEUE_SIZE ) {
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"%s : %d : disp_queue check error\n",__func__, __LINE__);
		return (1);
	} else {
		return (0);
	}
}

static int	is_empty_vpu_displayindex_chk_queue ()
{
	if ( g_disidx_chk_queue_entry <= 0 ) {
		//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"%s : %d : disp_queue check error\n",__func__, __LINE__);
		return (1);
	} else {
		return (0);
	}
}

static void chk_queue_vpu_displayindex(int index)
{
	if (is_full_vpu_displayindex_chk_queue() ) {
		//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"%s : %d : disp_queue check full error\n",__func__, __LINE__);
	}

//	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"chk_queue : %d\n",index);
	g_disidx_chk_queue[g_disidx_chk_queue_write] = index;
	g_disidx_chk_queue_write++;
	g_disidx_chk_queue_entry++;
	if(DISPLAY_INDEX_CHK_QUEUE_SIZE==g_disidx_chk_queue_write)
		g_disidx_chk_queue_write = 0;
}

static int chk_dequeue_vpu_displayindex(void)
{
	int ret = -1;

	if ( is_empty_vpu_displayindex_chk_queue() ) {
		//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR,"%s : %d : disp_queue check error\n",__func__, __LINE__);
		return ret;
	}

	ret = g_disidx_chk_queue[g_disidx_chk_queue_read];
//	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"chk_dequeue : %d\n",ret);
	g_disidx_chk_queue_read++;
	g_disidx_chk_queue_entry--;
	if(DISPLAY_INDEX_CHK_QUEUE_SIZE==g_disidx_chk_queue_read)
		g_disidx_chk_queue_read = 0;

	return ret;
}

#endif

#ifdef USE_TEE_SECURE_MEMORY
extern int
TC_SecureMemoryCopy(
	unsigned int paTarget,
	unsigned int paSource,
	unsigned int nSize
);
#endif



static void SetInternalVideoParameters(OMX_COMPONENTTYPE *openmaxStandComp) {

  omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private;
  omx_base_video_PortType *inPort ;

  omx_wfdvdec_component_Private = openmaxStandComp->pComponentPrivate;;

  if (omx_wfdvdec_component_Private->video_coding_type == OMX_VIDEO_CodingMPEG4) {
    strcpy(omx_wfdvdec_component_Private->ports[OMX_BASE_FILTER_INPUTPORT_INDEX]->sPortParam.format.video.cMIMEType,"video/cdkvdec");
    omx_wfdvdec_component_Private->ports[OMX_BASE_FILTER_INPUTPORT_INDEX]->sPortParam.format.video.eCompressionFormat = OMX_VIDEO_CodingMPEG4;


    inPort = (omx_base_video_PortType *)omx_wfdvdec_component_Private->ports[OMX_BASE_FILTER_INPUTPORT_INDEX];
    inPort->sVideoParam.eCompressionFormat = OMX_VIDEO_CodingMPEG4;

  }
}
OMX_ERRORTYPE omx_wfdvdec_component_Init(OMX_COMPONENTTYPE *openmaxStandComp) {
	return (omx_wfdvdec_component_Constructor(openmaxStandComp, WFDSINK_VDEC_NAME));
}

/** The Constructor of the video decoder component
  * @param openmaxStandComp the component handle to be constructed
  * @param cComponentName is the name of the constructed component
  */
OMX_ERRORTYPE omx_wfdvdec_component_Constructor(OMX_COMPONENTTYPE *openmaxStandComp,OMX_STRING cComponentName) {

  OMX_ERRORTYPE eError = OMX_ErrorNone;
  omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private;
  omx_base_video_PortType *inPort,*outPort;
  OMX_U32 i;

  if (!openmaxStandComp->pComponentPrivate) {
    TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME, "In %s, allocating component\n", __func__);
    openmaxStandComp->pComponentPrivate = TCC_calloc(1, sizeof(omx_wfdvdec_component_PrivateType));
    if(openmaxStandComp->pComponentPrivate == NULL) {
      return OMX_ErrorInsufficientResources;
    }
  } else {
    TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME, "In %s, Error Component %x Already Allocated\n", __func__, (int)openmaxStandComp->pComponentPrivate);
  }

  omx_wfdvdec_component_Private = openmaxStandComp->pComponentPrivate;
  omx_wfdvdec_component_Private->ports = NULL;

  eError = omx_base_filter_Constructor(openmaxStandComp, cComponentName);

  omx_wfdvdec_component_Private->sPortTypesParam[OMX_PortDomainVideo].nStartPortNumber = 0;
  omx_wfdvdec_component_Private->sPortTypesParam[OMX_PortDomainVideo].nPorts = 2;
  omx_wfdvdec_component_Private->BufferMgmtFunction = omx_wfdvdec_component_BufferMgmtFunction;

  /** Allocate Ports and call port constructor. */
  if (omx_wfdvdec_component_Private->sPortTypesParam[OMX_PortDomainVideo].nPorts && !omx_wfdvdec_component_Private->ports) {
        omx_wfdvdec_component_Private->ports = TCC_calloc(omx_wfdvdec_component_Private->sPortTypesParam[OMX_PortDomainVideo].nPorts, sizeof(omx_base_PortType *));
    if (!omx_wfdvdec_component_Private->ports) {
      return OMX_ErrorInsufficientResources;
    }
    for (i=0; i < omx_wfdvdec_component_Private->sPortTypesParam[OMX_PortDomainVideo].nPorts; i++) {
      omx_wfdvdec_component_Private->ports[i] = TCC_calloc(1, sizeof(omx_base_video_PortType));
      if (!omx_wfdvdec_component_Private->ports[i]) {
        return OMX_ErrorInsufficientResources;
      }
    }
  }

  base_video_port_Constructor(openmaxStandComp, &omx_wfdvdec_component_Private->ports[VIDEO_DEC_INPUT_PORT_INDEX], VIDEO_DEC_INPUT_PORT_INDEX, OMX_TRUE);
  base_video_port_Constructor(openmaxStandComp, &omx_wfdvdec_component_Private->ports[VIDEO_DEC_FB_OUTPUT_PORT_INDEX], VIDEO_DEC_FB_OUTPUT_PORT_INDEX, OMX_FALSE);

  /** here we can override whatever defaults the base_component constructor set
    * e.g. we can override the function pointers in the private struct
    */

  /** Domain specific section for the ports.
    * first we set the parameter common to both formats
    */
  //common parameters related to input port.
  inPort = (omx_base_video_PortType *)omx_wfdvdec_component_Private->ports[VIDEO_DEC_INPUT_PORT_INDEX];
  inPort->sPortParam.nBufferSize = VIDEO_DEC_IN_BUFFER_SIZE; //DEFAULT_OUT_BUFFER_SIZE;
  inPort->sPortParam.nBufferCountMin = 4;
  inPort->sPortParam.nBufferCountActual = 4;
  inPort->sPortParam.format.video.xFramerate = 30;

  //common parameters related to output port
  outPort = (omx_base_video_PortType *)omx_wfdvdec_component_Private->ports[VIDEO_DEC_FB_OUTPUT_PORT_INDEX];
  outPort->sPortParam.format.video.eColorFormat = OUTPUT_DECODED_COLOR_FMT;
  outPort->sPortParam.nBufferSize =  VIDEO_DEC_OUT_BUFFER_SIZE;
  outPort->sPortParam.nBufferCountMin = 4;
  outPort->sPortParam.nBufferCountActual = 4;
  outPort->sPortParam.format.video.xFramerate = 30;

  /** settings of output port parameter definition */
  outPort->sVideoParam.eColorFormat = OUTPUT_DECODED_COLOR_FMT;
  outPort->sVideoParam.xFramerate = 30;

  omx_wfdvdec_component_Private->video_coding_type = OMX_VIDEO_CodingAVC;  

  if(!omx_wfdvdec_component_Private->avCodecSyncSem) {
    omx_wfdvdec_component_Private->avCodecSyncSem = TCC_malloc(sizeof(tsem_t));
    if(omx_wfdvdec_component_Private->avCodecSyncSem == NULL) {
      return OMX_ErrorInsufficientResources;
    }
    tsem_init(omx_wfdvdec_component_Private->avCodecSyncSem, 0);
  }

  omx_wfdvdec_component_Private->eOutFramePixFmt = PIX_FMT_YUV420P;

  /** general configuration irrespective of any video formats
    * setting other parameters of omx_videodec_component_private
    */
//  omx_wfdvdec_component_Private->avCodec = NULL;
//  omx_wfdvdec_component_Private->avCodecContext= NULL;
	omx_wfdvdec_component_Private->avcodecReady = OMX_FALSE;
	omx_wfdvdec_component_Private->extradata = NULL;
	omx_wfdvdec_component_Private->extradata_size = 0;
	omx_wfdvdec_component_Private->BufferMgmtCallback = omx_wfdvdec_component_BufferMgmtCallback;

	/** initializing the codec context etc that was done earlier by ffmpeglibinit function */
	omx_wfdvdec_component_Private->messageHandler = omx_wfdvdec_component_MessageHandler;
	omx_wfdvdec_component_Private->destructor = omx_wfdvdec_component_Destructor;
	openmaxStandComp->SetParameter = omx_wfdvdec_component_SetParameter;
	openmaxStandComp->GetParameter = omx_wfdvdec_component_GetParameter;
	openmaxStandComp->SetConfig    = omx_wfdvdec_component_SetConfig;
	openmaxStandComp->GetConfig    = omx_wfdvdec_component_GetConfig;
	openmaxStandComp->ComponentRoleEnum = omx_wfdvdec_component_ComponentRoleEnum;
	openmaxStandComp->GetExtensionIndex = omx_wfdvdec_component_GetExtensionIndex;

	omx_wfdvdec_component_Private->LastKeyFrameTime = 0;
	omx_wfdvdec_component_Private->is_vdecinit = 0;
	omx_wfdvdec_component_Private->fast_stop = OMX_FALSE;
	omx_wfdvdec_component_Private->avcinterlacedscanflag = OMX_TRUE;
	omx_wfdvdec_component_Private->isGetDecodingInfo = 0;
	omx_wfdvdec_component_Private->use_secured_memory = OMX_FALSE;
	display_queue_init(&omx_wfdvdec_component_Private->index_queue);
	pthread_mutex_init (&omx_wfdvdec_component_Private->video_stop_mutex, NULL);
    omx_wfdvdec_component_Private->isVPUClosed = OMX_FALSE;
    omx_wfdvdec_component_Private->Display_debug_Msg = OMX_FALSE; 

    tcc_cdk_vdec_symbol_loading();
	/** decoder instance */
	omx_wfdvdec_component_Private->pVDecInstance = sym_tcc_cdk_vdec_alloc_instance(STD_AVC);
    if (omx_wfdvdec_component_Private->pVDecInstance == NULL) {
        TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "vdec instance allocation failed" );
        return OMX_ErrorInsufficientResources;
    }

	g_number_of_vpu_error = 0;
	g_number_of_vpu_macro_error=0;
	noVideoDecInstance++;

    if(noVideoDecInstance > MAX_COMPONENT_VIDEODEC)
        return OMX_ErrorInsufficientResources;
    

	return eError;
}


/** The destructor of the video decoder component
  */
OMX_ERRORTYPE omx_wfdvdec_component_Destructor(OMX_COMPONENTTYPE *openmaxStandComp) {
  omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private = openmaxStandComp->pComponentPrivate;
  OMX_U32 i;

  if(omx_wfdvdec_component_Private->extradata) {
	TCC_free(omx_wfdvdec_component_Private->extradata);
	omx_wfdvdec_component_Private->extradata=NULL;
  }

  if(omx_wfdvdec_component_Private->avCodecSyncSem) {
	tsem_deinit(omx_wfdvdec_component_Private->avCodecSyncSem);
	TCC_free(omx_wfdvdec_component_Private->avCodecSyncSem);
	omx_wfdvdec_component_Private->avCodecSyncSem = NULL;
  }

  pthread_mutex_destroy (&omx_wfdvdec_component_Private->video_stop_mutex);

  /* frees port/s */
  if (omx_wfdvdec_component_Private->ports) {
	for (i=0; i < omx_wfdvdec_component_Private->sPortTypesParam[OMX_PortDomainVideo].nPorts; i++) {
	  if(omx_wfdvdec_component_Private->ports[i])
		omx_wfdvdec_component_Private->ports[i]->PortDestructor(omx_wfdvdec_component_Private->ports[i]);
	}
	TCC_free(omx_wfdvdec_component_Private->ports);
	omx_wfdvdec_component_Private->ports=NULL;
  }

  TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME, "Destructor of video decoder component is called\n");

  omx_base_filter_Destructor(openmaxStandComp);

  display_queue_destroy(&omx_wfdvdec_component_Private->index_queue);

  /* Release decoder instance */
  if (omx_wfdvdec_component_Private->pVDecInstance) 
  {
	  vdec_release_instance(omx_wfdvdec_component_Private->pVDecInstance);
	  omx_wfdvdec_component_Private->pVDecInstance = NULL;
  }

  noVideoDecInstance--;

  return OMX_ErrorNone;
}

/** It initializates the FFmpeg framework, and opens an FFmpeg videodecoder of type specified by IL client
  */
OMX_ERRORTYPE omx_wfdvdec_component_LibInit(omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private) {

 	int ret;
	int video_codec_type;

	int vdec_address;
	cdk_func_t* vdec_func;
	cdmx_info_t* psCDmxInfo;

	vdec_init_t* psVDecInit;
	vdec_info_t* psVDecInfo;
	vdec_user_info_t vdec_user_info;
	cdk_core_t* pCdk;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "library/codec initialized\n");

	pCdk = sym_tcc_cdk_get_core_addr();

	psCDmxInfo = &omx_wfdvdec_component_Private->sCDmxInfo;
	video_codec_type = sym_tcc_cdk_get_video_codec_type( psCDmxInfo->m_sVideoInfo.m_iFourCC );
	omx_wfdvdec_component_Private->video_codec_type = video_codec_type;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"omx_cdkvdec_component_LibInit FCC    \t: 0x%08x(%c%c%c%c/%c%c%c%c)  video_codec_type:%d\n",(unsigned int)psCDmxInfo->m_sVideoInfo.m_iFourCC,
		(unsigned int)(psCDmxInfo->m_sVideoInfo.m_iFourCC&0xFF000000)>>24,
		(unsigned int)(psCDmxInfo->m_sVideoInfo.m_iFourCC&0x00FF0000)>>16,
		(unsigned int)(psCDmxInfo->m_sVideoInfo.m_iFourCC&0x0000FF00)>>8,
		(unsigned int)(psCDmxInfo->m_sVideoInfo.m_iFourCC&0x000000FF),
		(unsigned int)(psCDmxInfo->m_sVideoInfo.m_iFourCC&0x000000FF),
		(unsigned int)(psCDmxInfo->m_sVideoInfo.m_iFourCC&0x0000FF00)>>8,
		(unsigned int)(psCDmxInfo->m_sVideoInfo.m_iFourCC&0x00FF0000)>>16,
		(unsigned int)(psCDmxInfo->m_sVideoInfo.m_iFourCC&0xFF000000)>>24,
		video_codec_type);

	omx_wfdvdec_component_Private->vdec_func = sym_tcc_cdk_get_decode_function(video_codec_type);
	vdec_func = omx_wfdvdec_component_Private->vdec_func;
	psVDecInit = &omx_wfdvdec_component_Private->sVDecInit;
	//psVDecInit->m_bEnableUserData = 1;
	psVDecInit->m_bEnableUserData = 0;

	psVDecInit->m_iBitstreamFormat = video_codec_type;

#if 0
	//useless
	psVDecInit->m_RegBaseAddr = (unsigned int)NULL;
	psVDecInit->m_BitstreamBufAddr[PA] = sym_tcc_cdk_get_vpu_bitstreambuf_addr(PA);
	psVDecInit->m_BitstreamBufAddr[VA] = sym_tcc_cdk_get_vpu_bitstreambuf_addr(VA);
	psVDecInit->m_iBitstreamBufSize = (unsigned int)NULL;
#endif

	psVDecInit->m_iPicWidth = psCDmxInfo->m_sVideoInfo.m_iWidth;
	psVDecInit->m_iPicHeight = psCDmxInfo->m_sVideoInfo.m_iHeight;
	psVDecInit->m_bEnableVideoCache = 0;
	psVDecInit->m_bCbCrInterleaveMode = VIDEO_DECODER_CHROMA_INTERLEAVE;
	psVDecInit->m_uiMaxResolution = 0;//pCdk->m_uiVideoMaxResolution;
    
	psVDecInit->m_bFilePlayEnable = 1;



	//omx_wfdvdec_component_Private->psVDecInfo = TCC_CDK_GetVDecInfo();
	memset(&omx_wfdvdec_component_Private->sVDecInfo,0,sizeof(vdec_info_t));
	psVDecInfo = &omx_wfdvdec_component_Private->sVDecInfo;

	vdec_user_info.bitrate_mbps = 0;
	vdec_user_info.frame_rate = 0;
	vdec_user_info.m_bStillJpeg = 0;
	ret = vdec_func( VDEC_INIT, NULL, psVDecInit, &vdec_user_info, omx_wfdvdec_component_Private->pVDecInstance);
    omx_wfdvdec_component_Private->vpu_handle = (void*)vdec_address;
    if( ret < 0 )
	{
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[CDK_CORE] [Err:%4d] video decoder init\n", ret );
		vdec_func( VDEC_CLOSE, NULL, NULL, NULL, omx_wfdvdec_component_Private->pVDecInstance );
		return ret;
	}

	#ifndef ATTACH_SEQ_HEADER_FOR_VPU_DEC
	psVDecInit->m_bHasSeqHeader = psCDmxInfo->m_sVideoInfo.m_iExtraDataLen ? 1 : 0;
    if( psCDmxInfo->m_sVideoInfo.m_iExtraDataLen )
	{
        cdk_memcpy( (void*)psVDecInit->m_BitstreamBufAddr[VA], psCDmxInfo->m_sVideoInfo.m_pExtraData, psCDmxInfo->m_sVideoInfo.m_iExtraDataLen );
        ret = vdec_func( VDEC_DEC_SEQ_HEADER, NULL, psCDmxInfo->m_sVideoInfo.m_iExtraDataLen, psVDecInfo, omx_wfdvdec_component_Private->pVDecInstance);
        if( ret < 0 )
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[CDK_CORE] [Err:%4d] video decoder init\n", ret );
			vdec_func( VDEC_CLOSE, NULL, NULL, NULL, omx_wfdvdec_component_Private->pVDecInstance);
			return ret;
		}
	}
	#endif


	if( omx_wfdvdec_component_Private->container_type == CONTAINER_TYPE_TS )
	{
		sym_tcc_cdk_init_ts_pts_control(psCDmxInfo->m_sVideoInfo.m_iFrameRate);
	}

	memset(&omx_wfdvdec_component_Private->dec_disp_info_ctrl,0,sizeof(dec_disp_info_ctrl_t));
	memset(omx_wfdvdec_component_Private->dec_disp_info,0,sizeof(dec_disp_info_t)*32);
	memset(&omx_wfdvdec_component_Private->dec_disp_info_input,0,sizeof(dec_disp_info_input_t));

    omx_wfdvdec_component_Private->dec_disp_info_input.m_iStdType = video_codec_type;
	omx_wfdvdec_component_Private->dec_disp_info_input.m_iFmtType =	omx_wfdvdec_component_Private->container_type;
	omx_wfdvdec_component_Private->dec_disp_info_input.m_iFrameRate = psCDmxInfo->m_sVideoInfo.m_iFrameRate;
	omx_wfdvdec_component_Private->dec_disp_info_input.m_iTimeStampType = CDMX_PTS_MODE;
 	sym_tcc_cdk_display_pic_info(CVDEC_DISP_INFO_INIT, (void*)&omx_wfdvdec_component_Private->dec_disp_info_ctrl, (void*)omx_wfdvdec_component_Private->dec_disp_info,(void*)&omx_wfdvdec_component_Private->dec_disp_info_input);
    tsem_up(omx_wfdvdec_component_Private->avCodecSyncSem);
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "done\n");
	return OMX_ErrorNone;
}

/** It Deinitializates the ffmpeg framework, and close the ffmpeg video decoder of selected coding type
  */
void omx_wfdvdec_component_LibDeinit(omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private) {

	int ret;
	cdk_func_t* vdec_func = omx_wfdvdec_component_Private->vdec_func;

	if(!vpu_error_state)
		ret = vdec_func( VDEC_CLOSE, NULL, NULL, &omx_wfdvdec_component_Private->sVDecOutput, omx_wfdvdec_component_Private->pVDecInstance );
}

/** The Initialization function of the video decoder
  */
OMX_ERRORTYPE omx_wfdvdec_component_Initialize(OMX_COMPONENTTYPE *openmaxStandComp) {

	omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private = openmaxStandComp->pComponentPrivate;
	OMX_ERRORTYPE eError = OMX_ErrorNone;

	int i;

	for(i=0;i<DISPLAY_INDEX_QUEUE_SIZE;i++)
	{
		g_disidx_queue[i] = 0;
	}
	g_disidx_queue_read = 0;
	g_disidx_queue_write = 0;
	g_disidx_queue_entry = 0;
	pthread_mutex_init(&vpu_index_mutex, NULL);
#ifdef	USE_DISP_INDEX_CHK_QUEUE
	for(i=0;i<DISPLAY_INDEX_CHK_QUEUE_SIZE;i++)
	{
		g_disidx_chk_queue[i] = 0;
	}
	g_disidx_chk_queue_read = 0;
	g_disidx_chk_queue_write = 0;
	g_disidx_chk_queue_entry = 0;
#endif

	g_curr_vc1_skip_index= -1;
	g_prev_vc1_skip_index= -1;

	vpu_error_state = 0;

	g_buffer_full_count = 0;

	/** Temporary First Output buffer size */
	omx_wfdvdec_component_Private->inputCurrBuffer = NULL;
	omx_wfdvdec_component_Private->inputCurrLength = 0;
	omx_wfdvdec_component_Private->isFirstBuffer = 1;
	omx_wfdvdec_component_Private->isNewBuffer = 1;
	omx_wfdvdec_component_Private->dispIFrameNum = 0;
 	omx_wfdvdec_component_Private->skipIFrameCnt = 0;
 	omx_wfdvdec_component_Private->currKeyFramePTS = 0;

	omx_wfdvdec_component_Private->play_speed = 1;

	omx_wfdvdec_component_Private->sendIFrameSearchEvt = 0;
	omx_wfdvdec_component_Private->is_sync_arrived = FALSE;

	omx_wfdvdec_component_Private->vdecAvcWeightedPredFlag = 0;
	omx_wfdvdec_component_Private->vdecAvcWeightedPredBskipFlag = 0;
	omx_wfdvdec_component_Private->isVideoSkip = OMX_FALSE;
	omx_wfdvdec_component_Private->Iframe_Search_Enable = OMX_FALSE;
    omx_wfdvdec_component_Private->current_decout_width = 0;
    omx_wfdvdec_component_Private->current_decout_height = 0;
	sequence_init_done=0;
	gvdecerrcnt = 0;
	gvdecerr_sendevent = FALSE;

    return eError;
}

/** The Deinitialization function of the video decoder
  */
OMX_ERRORTYPE omx_wfdvdec_component_Deinit(OMX_COMPONENTTYPE *openmaxStandComp) {

  omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private = openmaxStandComp->pComponentPrivate;
  OMX_ERRORTYPE eError = OMX_ErrorNone;

	pthread_mutex_destroy(&vpu_index_mutex);

    if (omx_wfdvdec_component_Private->avcodecReady) 
    {
        omx_wfdvdec_component_LibDeinit(omx_wfdvdec_component_Private);
        omx_wfdvdec_component_Private->avcodecReady = OMX_FALSE;
    }

  return eError;
}

/** Executes all the required steps after an output buffer frame-size has changed.
*/
static inline void UpdateFrameSize(OMX_COMPONENTTYPE *openmaxStandComp) {

  omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private = openmaxStandComp->pComponentPrivate;
  omx_base_video_PortType *outPort = (omx_base_video_PortType *)omx_wfdvdec_component_Private->ports[OMX_BASE_FILTER_OUTPUTPORT_INDEX];
  omx_base_video_PortType *inPort = (omx_base_video_PortType *)omx_wfdvdec_component_Private->ports[OMX_BASE_FILTER_INPUTPORT_INDEX];
  outPort->sPortParam.format.video.nFrameWidth = inPort->sPortParam.format.video.nFrameWidth;
  outPort->sPortParam.format.video.nFrameHeight = inPort->sPortParam.format.video.nFrameHeight;

  switch(outPort->sVideoParam.eColorFormat) {
    case OMX_COLOR_FormatYUV420Planar:
      if(outPort->sPortParam.format.video.nFrameWidth && outPort->sPortParam.format.video.nFrameHeight) {
        outPort->sPortParam.nBufferSize = VIDEO_DEC_IN_BUFFER_SIZE; //outPort->sPortParam.format.video.nFrameWidth * outPort->sPortParam.format.video.nFrameHeight * 3/2;
      }
      break;
    default:
      if(outPort->sPortParam.format.video.nFrameWidth && outPort->sPortParam.format.video.nFrameHeight) {
        outPort->sPortParam.nBufferSize = VIDEO_DEC_IN_BUFFER_SIZE; //outPort->sPortParam.format.video.nFrameWidth * outPort->sPortParam.format.video.nFrameHeight * 3;
      }
      break;
  }

}

/** This function is used to process the input buffer and provide one output buffer
  */

unsigned long long GetTime_MSec0(void) // milli second
{
	struct timeval now;
	gettimeofday (&now, NULL);
	return ((unsigned long long) now.tv_sec) * 1000 + ((unsigned long long) now.tv_usec) / 1000;
}


void omx_wfdvdec_component_BufferMgmtCallback(OMX_COMPONENTTYPE *openmaxStandComp, OMX_BUFFERHEADERTYPE* pInputBuffer	, OMX_BUFFERHEADERTYPE* pOutputBuffer)
{
	omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private = openmaxStandComp->pComponentPrivate;

	OMX_S32 ret;
	OMX_S32 nOutputFilled = 0;
	OMX_U8* outputCurrBuffer;
	int nLen = 0;
	int internalOutputFilled=0;
	int nSize;

	int i;
	cdk_func_t* vdec_func;
	vdec_init_t* psVDecInit;
	vdec_input_t* psVDecInput;
	vdec_output_t* psVDecOutput;
	dec_disp_info_t dec_disp_info_tmp;

	int displayindex;
	int	decode_result;
	mblk_t	*pVideostream;

	omx_base_video_PortType *outPort = (omx_base_video_PortType *)omx_wfdvdec_component_Private->ports[OMX_BASE_FILTER_OUTPUTPORT_INDEX];

	psVDecInit = &omx_wfdvdec_component_Private->sVDecInit;

	outputCurrBuffer = pOutputBuffer->pBuffer;
	pOutputBuffer->nFilledLen = 0;
	pOutputBuffer->nOffset = 0;
	pthread_mutex_lock(&omx_wfdvdec_component_Private->video_stop_mutex);
	/* don't move this routine!! this routine should be done first!! */
	if(!omx_wfdvdec_component_Private->avcodecReady){
		if (omx_wfdvdec_component_Private->isFirstBuffer) {
			tsem_down(omx_wfdvdec_component_Private->avCodecSyncSem);
			omx_wfdvdec_component_Private->isFirstBuffer = 0;
		}
		else
		{
			pthread_mutex_unlock(&omx_wfdvdec_component_Private->video_stop_mutex);
			return;
		}
	}
	if(omx_wfdvdec_component_Private->fast_stop)
	{
		pInputBuffer->nFilledLen = 0;
		pOutputBuffer->faststoprequest = OMX_TRUE;
		pOutputBuffer->nFilledLen = 0;
		usleep(10000);
		pthread_mutex_unlock(&omx_wfdvdec_component_Private->video_stop_mutex);
		return;
	}

	if(vpu_error_state)
	{
		pInputBuffer->nFilledLen = 0;
		usleep(10000);
		pthread_mutex_unlock(&omx_wfdvdec_component_Private->video_stop_mutex);
		return;
	}
 	vdec_func = omx_wfdvdec_component_Private->vdec_func;
	psVDecInput = &omx_wfdvdec_component_Private->sVDecInput;
	psVDecOutput = &omx_wfdvdec_component_Private->sVDecOutput;

    if(omx_wfdvdec_component_Private->isVPUClosed == OMX_TRUE)
    {
        vdec_init_t* psVDecInit;
        vdec_user_info_t vdec_user_info;

        psVDecInit = &omx_wfdvdec_component_Private->sVDecInit;

        vdec_user_info.bitrate_mbps = 0;
        vdec_user_info.frame_rate = 0;
        vdec_user_info.m_bStillJpeg = 0;

        TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[OMX_WFDVDEC]: VPU Re-Init..\n");
        if( (ret = vdec_vpu( VDEC_INIT, NULL, psVDecInit, &vdec_user_info, omx_wfdvdec_component_Private->pVDecInstance)) < 0 )    
        {
            TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[VDEC_INIT] [Err:-0x%x] video decoder Re-init", -ret );
        }
        omx_wfdvdec_component_Private->isVPUClosed = OMX_FALSE;
    }

	/*When scan Function(over 2x) is started on playing, VPU should discard frames in input port buffer until sync frame arrived */

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME, "In %s\n", __func__);
	/** Fill up the current input buffer when a new buffer has arrived */
	if(omx_wfdvdec_component_Private->isNewBuffer) {
		omx_wfdvdec_component_Private->inputCurrBuffer = pInputBuffer->pBuffer;
		omx_wfdvdec_component_Private->inputCurrLength = pInputBuffer->nFilledLen;
		omx_wfdvdec_component_Private->isNewBuffer = 0;
		DEBUG(DEB_LEV_FULL_SEQ, "New Buffer FilledLen = %d\n", (int)pInputBuffer->nFilledLen);
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "----------------------> new buffer!! \n");
	}

    while (!nOutputFilled) {

			OMX_U8 *input_src_ptr = (OMX_U8 *) (pInputBuffer->pBuffer);	
			/*Input Message Block Address*/
			pVideostream = (mblk_t	*)((*(input_src_ptr+3)<<24) | (*(input_src_ptr+2)<<16) | (*(input_src_ptr+1)<<8) | *(input_src_ptr));

            if(omx_wfdvdec_component_Private->Display_debug_Msg == OMX_TRUE){
                TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[OMX_WFDVDEC] Secured type %d Use Secured Memory %d\n",pVideostream->secure_type, omx_wfdvdec_component_Private->use_secured_memory);
                TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[OMX_WFDVDEC] PA %p VA %p Len %d \n", psVDecInput->m_pInp[PA], psVDecInput->m_pInp[VA], psVDecInput->m_iInpLen);
            }            
#ifdef USE_TEE_SECURE_MEMORY
		if((omx_wfdvdec_component_Private->use_secured_memory == OMX_TRUE) && (pVideostream->secure_type == OMX_TRUE))
		{
			/*Input Frame Buffer Address*/
			OMX_U8 * Frame_Addr = pVideostream->frame_addr;
			/*Input Frame Length*/
			int 	 Frame_Len = pVideostream->frame_size;

			/*Using Secured Memory for TEE*/
			unsigned int uiTargetAddr = (unsigned int)sym_tcc_cdk_get_vpu_bitstreambuf_addr(PA, omx_wfdvdec_component_Private->pVDecInstance);
			unsigned int uiSourceAddr = (unsigned int)Frame_Addr;
			pthread_mutex_lock(omx_wfdvdec_component_Private->secure_video_path_mutex);
			ret = TC_SecureMemoryCopy(uiTargetAddr, uiSourceAddr, Frame_Len);
			pthread_mutex_unlock(omx_wfdvdec_component_Private->secure_video_path_mutex);
			psVDecInput->m_pInp[PA] = (unsigned char*)sym_tcc_cdk_get_vpu_bitstreambuf_addr(PA, omx_wfdvdec_component_Private->pVDecInstance);
			psVDecInput->m_pInp[VA] = (unsigned char*)sym_tcc_cdk_get_vpu_bitstreambuf_addr(VA, omx_wfdvdec_component_Private->pVDecInstance);
			psVDecInput->m_iInpLen	= Frame_Len;
		}
		else
#endif
		{
			/*Input Frame Buffer Address*/
			OMX_U8 * Frame_Addr = pVideostream->b_rptr;
			/*Input Frame Length*/
			int 	 Frame_Len = pVideostream->frame_size;

 			int offset = pVideostream->b_rptr - sym_tcc_cdk_get_vpu_bitstreambuf_addr(VA, omx_wfdvdec_component_Private->pVDecInstance);
			psVDecInput->m_pInp[PA] = offset + (unsigned char*)sym_tcc_cdk_get_vpu_bitstreambuf_addr(PA, omx_wfdvdec_component_Private->pVDecInstance);
			psVDecInput->m_pInp[VA] = (unsigned int)Frame_Addr;
			psVDecInput->m_iInpLen	= Frame_Len;
		}

		if(pVideostream->packet_discon == OMX_TRUE)
		{
			omx_wfdvdec_component_Private->Iframe_Search_Enable = OMX_TRUE;
		}

		{
            if (sequence_init_done == OMX_FALSE)
            {   
                ret = vdec_func( VDEC_DEC_SEQ_HEADER, NULL, psVDecInput, psVDecOutput, omx_wfdvdec_component_Private->pVDecInstance);
                if(sequence_init_done == OMX_FALSE)
                {
                    if( ret != 0 )
                    {
                        g_number_of_vpu_error++;
                        omx_wfdvdec_component_Private->isNewBuffer = 1;
                        pInputBuffer->nFilledLen = 0;
                        printf("[OMX_WFDVDEC]:Err%d gspfVDec VDEC_DEC_SEQ_HEADER failed  frame len : %d\n", ret ,psVDecInput->m_iInpLen);
                        pthread_mutex_unlock(&omx_wfdvdec_component_Private->video_stop_mutex);
						freemsg((mblk_t *)pVideostream);
						printf("[OMX_WFDVDEC]: WFD Request IDR-Picture....\n");
                        
						(*(omx_wfdvdec_component_Private->callbacks->EventHandler))(openmaxStandComp, 
                    																omx_wfdvdec_component_Private->callbackData,
                    																OMX_EventVendorRequestIDRPicture,	/* The command was completed */
                    																NULL,	/* port Format Detected */
                    																NULL,	/* This is the output port index */
                    																NULL);

                        if(ret == -RETCODE_CODEC_EXIT || ret == -RETCODE_MULTI_CODEC_EXIT_TIMEOUT)
                            goto VDEC_ERROR_PROCESS;
                        
                         return ret;
                    }
                    else
                    {
                        TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_INFO, "[OMX_WFDVDEC]: Succeed in Sequence Header Decode\n");
                        sequence_init_done = OMX_TRUE;
                        (*(omx_wfdvdec_component_Private->callbacks->EventHandler))(openmaxStandComp, 
                        omx_wfdvdec_component_Private->callbackData,
                        OMX_EventVendorSeqHeaderDone,	/* The command was completed */
                        NULL,	/* port Format Detected */
                        NULL,	/* This is the output port index */
                        (void*)psVDecOutput);
                    }
                }
            }

            if(omx_wfdvdec_component_Private->Iframe_Search_Enable == OMX_TRUE)
            {
            	psVDecInput->m_iFrameSearchEnable = 1;	//I-frame (IDR-picture for H.264)
            	psVDecInput->m_iSkipFrameMode = 0;
            	omx_wfdvdec_component_Private->Iframe_Search_Enable = OMX_FALSE;
            	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "[OMX_WFDVDEC] VDEC Input : VDEC_SKIP_FRAME_EXCEPT_I \n");
            }
#if 0
            else
            {		
            	psVDecInput->m_iFrameSearchEnable = 0;	//I-frame (IDR-picture for H.264)
            	psVDecInput->m_iSkipFrameMode = 0;
            }
#endif
			while (1) {
 				if(display_queue_num(&omx_wfdvdec_component_Private->index_queue) > MIN_DISPLAY_QUEUE_SIZE)
				{
					displayindex = display_queue_pop(&omx_wfdvdec_component_Private->index_queue);
					if(displayindex >= 0 )
					{
						if( ( ret = vdec_func( VDEC_BUF_FLAG_CLEAR, NULL, &displayindex, NULL, omx_wfdvdec_component_Private->pVDecInstance) ) < 0 )
						{
							printf("[[OMX_WFDVDEC]] VDEC_BUF_FLAG_CLEAR error %d\n", displayindex );
							pthread_mutex_unlock(&omx_wfdvdec_component_Private->video_stop_mutex);
							return ret;
						}
					}
				}

				{
#if 0
					unsigned long long last_time;
					unsigned long long current_time;
					last_time = GetTime_MSec0();
#endif
					ret = vdec_func( VDEC_DECODE, NULL, psVDecInput, psVDecOutput, omx_wfdvdec_component_Private->pVDecInstance);
#if 0
					current_time = GetTime_MSec0();			
					printf("time diff %lld \n",current_time - last_time);
#endif
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"\n[WFDVDEC] ret:%d DEC OUT STATUS[%d][%lld][%f]: decstat:%d[%d] dispstat:%d[%d]  FrameLen:%d, speed:%f\n ",
                    ret,
                    psVDecOutput->m_DecOutInfo.m_iPicType,
                    pInputBuffer->nTimeStamp,
                    pInputBuffer->fPlaybackSpeed,
                    psVDecOutput->m_DecOutInfo.m_iDecodingStatus,
                    psVDecOutput->m_DecOutInfo.m_iDecodedIdx,
                    psVDecOutput->m_DecOutInfo.m_iOutputStatus,
                    psVDecOutput->m_DecOutInfo.m_iDispOutIdx,
                    pInputBuffer->nFilledLen,
                    pInputBuffer->fPlaybackSpeed);
                }
#if 0
				if(omx_wfdvdec_component_Private->isVideoSkip == OMX_TRUE)
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[[OMX_WFDVDEC]] Video Skip..... \n");
					pInputBuffer->nFilledLen = 0;
					pthread_mutex_unlock(&omx_wfdvdec_component_Private->video_stop_mutex);
					return;
				}
#endif
                if((omx_wfdvdec_component_Private->current_decout_width == 0) && (omx_wfdvdec_component_Private->current_decout_height == 0))
                {
                    omx_wfdvdec_component_Private->current_decout_width = psVDecOutput->m_DecOutInfo.m_iWidth;
                    omx_wfdvdec_component_Private->current_decout_height = psVDecOutput->m_DecOutInfo.m_iHeight;
                }

				if(ret == -RETCODE_CODEC_EXIT)
                    goto VDEC_ERROR_PROCESS;

                if((omx_wfdvdec_component_Private->current_decout_width != psVDecOutput->m_DecOutInfo.m_iWidth) ||
                    (omx_wfdvdec_component_Private->current_decout_height != psVDecOutput->m_DecOutInfo.m_iHeight)){
                
                    TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_INFO, "[OMX_WFDVDEC]: resolution change\n");

                    omx_wfdvdec_component_Private->current_decout_width  = psVDecOutput->m_DecOutInfo.m_iWidth;
                    omx_wfdvdec_component_Private->current_decout_height = psVDecOutput->m_DecOutInfo.m_iHeight;

                    goto VDEC_ERROR_PROCESS;
                }

				pOutputBuffer->nKeyFrameTimeStamp = omx_wfdvdec_component_Private->currKeyFramePTS;

				if(psVDecOutput->m_DecOutInfo.m_iDecodingStatus == VPU_DEC_BUF_FULL)
				{
				    TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR,"[OMX_WFDVDEC] VPU_DEC_BUF_FULL \n");                                    
					g_buffer_full_count++;
					if(psVDecOutput->m_DecOutInfo.m_iOutputStatus == VPU_DEC_OUTPUT_SUCCESS)
					{
						// Display Index  ó, Stream  Stream 
						decode_result = 0;
						break;
					} else {
						// Display Index ó, Stream  Stream 
						decode_result = 1;
						break;
					}
					// Stream  Stream ٽ  .
				}
				else
				{
					g_buffer_full_count = 0;
					if(psVDecOutput->m_DecOutInfo.m_iOutputStatus == VPU_DEC_OUTPUT_SUCCESS)
					{
						// Display Index ó, Stream  Stream
						decode_result = 2;
						break;
					}
					else
					{
						// Display Index ó, Stream  Stream
						decode_result = 3;
						break;
					}
				}

			}
			if(ret<0)
			{
				printf("[OMX_WFDVDEC]VPU Error ret:%d \n",ret);
				g_number_of_vpu_error++;
			}

			if(psVDecOutput->m_DecOutInfo.m_iDecodingStatus == VPU_DEC_SUCCESS && psVDecOutput->m_DecOutInfo.m_iDecodedIdx >= 0)
			{
                omx_wfdvdec_component_Private->decout_frame_info.nFrameWidth = psVDecOutput->m_DecOutInfo.m_iWidth;
                omx_wfdvdec_component_Private->decout_frame_info.nFrameHeight = psVDecOutput->m_DecOutInfo.m_iHeight;
                omx_wfdvdec_component_Private->decout_frame_info.VideoCropInfo.nTop = psVDecOutput->m_DecOutInfo.m_CropInfo.m_iCropTop;
                omx_wfdvdec_component_Private->decout_frame_info.VideoCropInfo.nBottom = psVDecOutput->m_DecOutInfo.m_CropInfo.m_iCropBottom;
                omx_wfdvdec_component_Private->decout_frame_info.VideoCropInfo.nLeft= psVDecOutput->m_DecOutInfo.m_CropInfo.m_iCropLeft;
                omx_wfdvdec_component_Private->decout_frame_info.VideoCropInfo.nRight= psVDecOutput->m_DecOutInfo.m_CropInfo.m_iCropRight;              
                pOutputBuffer->pInputPortPrivate = &omx_wfdvdec_component_Private->decout_frame_info;
                
				dec_disp_info_tmp.m_iTimeStamp = (int)(pInputBuffer->nTimeStamp/1000);
				dec_disp_info_tmp.m_iFrameType = psVDecOutput->m_DecOutInfo.m_iPicType;
				dec_disp_info_tmp.m_iPicStructure = psVDecOutput->m_DecOutInfo.m_iPictureStructure;
				dec_disp_info_tmp.m_iM2vFieldSequence = 0;
				dec_disp_info_tmp.m_iFrameSize = psVDecOutput->m_DecOutInfo.m_iConsumedBytes;// gsCDmxOutput.m_iPacketSize;
				dec_disp_info_tmp.m_iFrameDuration = 2;	

				omx_wfdvdec_component_Private->dec_disp_info_input.m_iFrameIdx = psVDecOutput->m_DecOutInfo.m_iDecodedIdx;
				omx_wfdvdec_component_Private->dec_disp_info_input.m_iFrameRate = psVDecOutput->m_DecOutInfo.m_iM2vFrameRate;
                sym_tcc_cdk_display_pic_info( CVDEC_DISP_INFO_UPDATE, (void*)&omx_wfdvdec_component_Private->dec_disp_info_ctrl, (void*)&dec_disp_info_tmp, (void*)&omx_wfdvdec_component_Private->dec_disp_info_input);
            }

            if( omx_wfdvdec_component_Private->frameSearchOrSkip_flag && 
                psVDecOutput->m_DecOutInfo.m_iDecodedIdx >= 0 && 
                (psVDecOutput->m_DecOutInfo.m_iDecodingStatus == VPU_DEC_SUCCESS || 
                psVDecOutput->m_DecOutInfo.m_iDecodingStatus == VPU_DEC_SUCCESS_FIELD_PICTURE )
            )
			{
				if( psVDecInput->m_iFrameSearchEnable )
				{
					 omx_wfdvdec_component_Private->frameSearchOrSkip_flag = 2;//I-frame Search Mode disable and B-frame Skip Mode enable
				}
				else if( psVDecInput->m_iSkipFrameMode == VDEC_SKIP_FRAME_ONLY_B )
				{
					 omx_wfdvdec_component_Private->frameSearchOrSkip_flag = 0;	//B-frame Skip Mode disable

				}
			}

#ifdef TCC_VDEC_DEBUG_INCLUDE
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[V:" );
			print_pic_type( omx_wfdvdec_component_Private->video_codec_type, psVDecOutput->m_DecOutInfo.m_iPicType, psVDecOutput->m_DecOutInfo.m_iPictureStructure );
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "]" );
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[LEN:%6d]", pInputBuffer->nFilledLen );
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[RT:%8d]", (int)(pInputBuffer->nTimeStamp/1000));
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[DecIdx:%2d][DecStat:%d]",psVDecOutput->m_DecOutInfo.m_iDecodedIdx,psVDecOutput->m_DecOutInfo.m_iDecodingStatus);


			if( psVDecOutput->m_DecOutInfo.m_iOutputStatus != VPU_DEC_OUTPUT_SUCCESS )
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[V:U ]");
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[LEN:%6d][RT:%8d]", psVDecOutput->m_DecOutInfo.m_iConsumedBytes, 99999999 );
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[DispIdx:%2d][OutStat:%d]\n",
					psVDecOutput->m_DecOutInfo.m_iDispOutIdx,
					psVDecOutput->m_DecOutInfo.m_iOutputStatus );
			}
#endif	
			/*VPU Processing*/
			if( psVDecOutput->m_DecOutInfo.m_iOutputStatus == VPU_DEC_OUTPUT_SUCCESS )
			{
				int outputstatus = psVDecOutput->m_DecOutInfo.m_iOutputStatus;
				int dispOutIdx = psVDecOutput->m_DecOutInfo.m_iDispOutIdx;
				int decodingstatus = psVDecOutput->m_DecOutInfo.m_iDecodingStatus;

				dec_disp_info_t * pdec_disp_info;

				if(psVDecInput->m_iFrameSearchEnable == 1)	//I-frame (IDR-picture for H.264))
					psVDecInput->m_iFrameSearchEnable = 0;


				/* Decoding Fail Ͼ  Reset*/
				gvdecerrcnt = 0 ;

				if(psVDecInit->m_bEnableUserData)
				{
					sym_tcc_cdk_print_user_data(psVDecOutput->m_DecOutInfo.m_UserDataAddress[VA]);
				}

				omx_wfdvdec_component_Private->dec_disp_info_input.m_iFrameIdx = dispOutIdx;
				sym_tcc_cdk_display_pic_info( CVDEC_DISP_INFO_GET, (void*)&omx_wfdvdec_component_Private->dec_disp_info_ctrl,&pdec_disp_info, (void*)&omx_wfdvdec_component_Private->dec_disp_info_input);
#ifdef TCC_VDEC_DEBUG_INCLUDE
				if( pdec_disp_info != (dec_disp_info_t*)0 )
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[V:" );
					print_pic_type(	omx_wfdvdec_component_Private->video_codec_type, pdec_disp_info->m_iFrameType, pdec_disp_info->m_iPicStructure);
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "]" );
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[LEN:%6d][RT:%8d]",pdec_disp_info->m_iFrameSize, pdec_disp_info->m_iTimeStamp );
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[DispIdx:%2d][OutStat:%d]\n", dispOutIdx, outputstatus );
				}
				else
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[V:U ]");
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[LEN:%6d][RT:%8d]", psVDecOutput->m_DecOutInfo.m_iConsumedBytes, 99999999 );
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[DispIdx:%2d][OutStat:%d]\n",
						psVDecOutput->m_DecOutInfo.m_iDispOutIdx,
						psVDecOutput->m_DecOutInfo.m_iOutputStatus );
				}
#endif
				if(pdec_disp_info == 0)
				{
					pdec_disp_info = (dec_disp_info_t*)omx_wfdvdec_component_Private->dec_disp_info_ctrl.m_pRegInfoPTS[omx_wfdvdec_component_Private->dec_disp_info_ctrl.m_iUsedIdxPTS];
    			}

				pOutputBuffer->nTimeStamp = pInputBuffer->nTimeStamp;

				//physical address
				for(i=0;i<3;i++)
				{
					memcpy(pOutputBuffer->pBuffer+i*4, &psVDecOutput->m_pDispOut[PA][i], 4);
				}
				display_queue_push(&omx_wfdvdec_component_Private->index_queue, psVDecOutput->m_DecOutInfo.m_iDispOutIdx);
				g_number_of_vpu_error = 0;
			}
			else if((psVDecOutput->m_DecOutInfo.m_iOutputStatus != VPU_DEC_OUTPUT_SUCCESS && omx_wfdvdec_component_Private->frameSearchOrSkip_flag == 0) ||
				(psVDecOutput->m_DecOutInfo.m_iDecodingStatus != VPU_DEC_SUCCESS &&  omx_wfdvdec_component_Private->frameSearchOrSkip_flag == 1 ))
			{
				/* 1 .  VIDEO Decoding Output   
				  2 .Seek  Iframe  ã Ͽ Decoding Fail   */

				//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR,"[OMX_WFDVDEC]No VDEC Output [%d]\n", gvdecerrcnt);
				if((gvdecerrcnt > 30) && (psVDecInput->m_iFrameSearchEnable == 1))
				{
					psVDecInput->m_iFrameSearchEnable = 0;
                    (*(omx_wfdvdec_component_Private->callbacks->EventHandler))(openmaxStandComp, 
                                                            omx_wfdvdec_component_Private->callbackData,
                                                            OMX_EventVendorRequestIDRPicture,	/* The command was completed */
                                                            NULL,	/* port Format Detected */
                                                            NULL,	/* This is the output port index */
                                                            NULL);
					//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR,"[OMX_WFDVDEC]No VDEC Output Over 60 [%d]\n", gvdecerrcnt);
				}
				gvdecerrcnt++;
				/*Count 30(1ʺз)  Ѵ. 30frame demuxing ð ٰ Ǹ 10̻  .  10frame     ֱ ...  */
				// Continuous VDEC Error   stopǴ    30 -> 256

			}
    		else
			{
				//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR,"[OMX_WFDVDEC]No VDEC Output  %d %d \n",omx_wfdvdec_component_Private->frameSearchOrSkip_flag,omx_wfdvdec_component_Private->vdecAvcWeightedPredFlag);
			}

		}

		internalOutputFilled = 1;

		// to monitoring
		switch (decode_result)
		{
			case 0:	// Display Output Success, Decode Failed Due to Buffer Full
			case 1:	// Display Output Failed, Decode Failed Due to Buffer Full
				break;

			case 2:	// Display Output Success, Decode Success
				// g_number_of_vpu_macro_error check κ 
				// ⼭ check Ұ ǵ ʰ stop ߻.
				if(psVDecOutput->m_DecOutInfo.m_iNumOfErrMBs > 0)
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR,"%s %d CDK VDEC Failed : %d\n",__func__,__LINE__, psVDecOutput->m_DecOutInfo.m_iNumOfErrMBs);
				}
				break;

			case 3:	// Display Output Failed, Decode Success
				if(psVDecOutput->m_DecOutInfo.m_iNumOfErrMBs > 0)
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR,"%s %d CDK VDEC Failed : %d\n",__func__,__LINE__, psVDecOutput->m_DecOutInfo.m_iNumOfErrMBs);
					g_number_of_vpu_macro_error++;
				}
				else
				{
					g_number_of_vpu_macro_error = 0;
				}
				break;

			default :
				break;
		}
		
		// To process input stream retry or next frame
		switch (decode_result)
		{
			case 0 :	// Display Output Success, Decode Failed Due to Buffer Full
			case 1 :	// Display Output Failed, Decode Failed Due to Buffer Full
				break;

			case 2 :	// Display Output Success, Decode Success
			case 3 :	// Display Output Failed, Decode Success
				nLen += omx_wfdvdec_component_Private->inputCurrLength;
				//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "----------------------> inputCurrLength : %d, nLen : %d\n", omx_wfdvdec_component_Private->inputCurrLength, nLen);
				if(nLen < 0)
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "----> A general error or simply frame not decoded?\n");
				}

				if(nLen >= 0 && internalOutputFilled)
				{
					omx_wfdvdec_component_Private->inputCurrBuffer += nLen;
					omx_wfdvdec_component_Private->inputCurrLength -= nLen;
					pInputBuffer->nFilledLen -= nLen;

					//Buffer is fully consumed. Request for new Input Buffer
					if(pInputBuffer->nFilledLen == 0)
					{
						omx_wfdvdec_component_Private->isNewBuffer = 1;
					}
				}
				else
				{

					/**  This condition becomes true when the input buffer has completely be consumed.
					* In this case is immediately switched because there is no real buffer consumption
					*/
					pInputBuffer->nFilledLen = 0;

					/** Few bytes may be left in the input buffer but can't generate one output frame.
					*  Request for new Input Buffer
					*/
					omx_wfdvdec_component_Private->isNewBuffer = 1;
				}
				break;

			default :
				break;
		}


		// To process output stream retry or next frame
		switch (decode_result)
		{
			case 0 :	// Display Output Success, Decode Failed Due to Buffer Full
			case 2 :	// Display Output Success, Decode Success
			{
				if ( nLen >= 0 && internalOutputFilled)
				{
					nSize = 4*8;
					if(pOutputBuffer->nAllocLen < nSize)
					{
						//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "[OMX_WFDVDEC]Ouch!!!! Output buffer Alloc Len %d less than Frame Size %d\n",(int)pOutputBuffer->nAllocLen,nSize);
						pthread_mutex_unlock(&omx_wfdvdec_component_Private->video_stop_mutex);
						return;
					}
					#ifndef TCC_ANDROID_INCLUDE
					pOutputBuffer->nFilledLen += nSize;
					#endif
				}
				else
				{
					/** Few bytes may be left in the input buffer but can't generate one output frame.
					*  Request for new Input Buffer
					*/
					omx_wfdvdec_component_Private->isNewBuffer = 1;
					#ifndef TCC_ANDROID_INCLUDE
					pOutputBuffer->nFilledLen = 0;
					#endif
				}

				#ifdef USE_DISP_INDEX_CHK_QUEUE
				{
					check_vpu_index_queue(psVDecOutput->m_DecOutInfo.m_iDecodedIdx);
				}

				if ( psVDecOutput->m_DecOutInfo.m_iDispOutIdx== g_curr_vc1_skip_index) {
					chk_queue_vpu_displayindex (psVDecOutput->m_DecOutInfo.m_iDispOutIdx|0x40000000);
				} else {
					chk_queue_vpu_displayindex (psVDecOutput->m_DecOutInfo.m_iDispOutIdx);
				}
				#endif
			}
			break;

			case 3 :	// Display Output Failed, Decode Success
			{
				
			}
			break;

			case 1 :	// Display Output Failed, Decode Failed Due to Buffer Full
				break;
			default :
				break;
		}

#if 1
		switch (decode_result)
		{
			case 0 :	// Display Output Success, Decode Failed Due to Buffer Full
			case 1 :	// Display Output Failed, Decode Failed Due to Buffer Full
			{
				if(g_buffer_full_count > 10)
				{
					nLen += omx_wfdvdec_component_Private->inputCurrLength;
					//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "----------------------> inputCurrLength : %d, nLen : %d\n", omx_wfdvdec_component_Private->inputCurrLength, nLen);
					if(nLen < 0)
					{
						//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "----> A general error or simply frame not decoded?\n");
					}

					if(nLen >= 0 && internalOutputFilled)
					{
						omx_wfdvdec_component_Private->inputCurrBuffer += nLen;
						omx_wfdvdec_component_Private->inputCurrLength -= nLen;
						pInputBuffer->nFilledLen -= nLen;

						//Buffer is fully consumed. Request for new Input Buffer
						if(pInputBuffer->nFilledLen == 0)
						{
							omx_wfdvdec_component_Private->isNewBuffer = 1;
						}
					}
					else
					{

						/**  This condition becomes true when the input buffer has completely be consumed.
						* In this case is immediately switched because there is no real buffer consumption
						*/
						pInputBuffer->nFilledLen = 0;

						/** Few bytes may be left in the input buffer but can't generate one output frame.
						*  Request for new Input Buffer
						*/
						omx_wfdvdec_component_Private->isNewBuffer = 1;
					}
					g_buffer_full_count = 0;
				}
			}
			break;
		}
#endif
		nOutputFilled = 1;
	}
	if(pInputBuffer->nFlags & OMX_BUFFERFLAG_DECODEONLY)
	{
		if(omx_wfdvdec_component_Private->video_codec_type != STD_SH263) // S/W Codec
		{
			queue_vpu_displayindex(psVDecOutput->m_DecOutInfo.m_iDispOutIdx);
		}
		pOutputBuffer->nFilledLen = 0;
		pInputBuffer->nFlags &= ~OMX_BUFFERFLAG_DECODEONLY;
		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"[OMX_WFDVDEC] Clear OMX_BUFFERFLAG_DECODEONLY \n");
	}
	freemsg((mblk_t *)pVideostream);
	// if video codec is H.264, we should drop few video frames when seeking.
	DEBUG(TCC_DEBUG_LEV_FULL_SEQ, "[OMX_WFDVDEC]One output buffer %x nLen=%d is full returning in video decoder\n",(int)pOutputBuffer->pBuffer, (int)pOutputBuffer->nFilledLen);
	pthread_mutex_unlock(&omx_wfdvdec_component_Private->video_stop_mutex);

    return;
VDEC_ERROR_PROCESS :
    TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR,"[OMX_WFDVDEC]: VDEC Error Process (err type %d) & VPU Close & Re-Init\n", ret);
    pInputBuffer->nFilledLen = 0;
    pOutputBuffer->nFilledLen = 0;
    omx_wfdvdec_component_Private->isNewBuffer = 1;

    if(omx_wfdvdec_component_Private->isVPUClosed == OMX_FALSE){
        if((ret = vdec_vpu(VDEC_CLOSE, NULL, NULL, &omx_wfdvdec_component_Private->sVDecOutput, omx_wfdvdec_component_Private->pVDecInstance)) < 0 )
            printf("[VDEC_CLOSE] [Err:%4d] video decoder Deinit", ret );
        
        omx_wfdvdec_component_Private->isVPUClosed = OMX_TRUE;
    }
    sequence_init_done = OMX_FALSE;
    freemsg((mblk_t *)pVideostream);
    (*(omx_wfdvdec_component_Private->callbacks->EventHandler))(openmaxStandComp, 
                                                                omx_wfdvdec_component_Private->callbackData,
                                                                OMX_EventVendorRequestIDRPicture,   /* The command was completed */
                                                                NULL,   /* port Format Detected */
                                                                NULL,   /* This is the output port index */
                                                                NULL);
    pthread_mutex_unlock(&omx_wfdvdec_component_Private->video_stop_mutex);
    return;
}

OMX_ERRORTYPE omx_wfdvdec_component_SetParameter(
OMX_IN  OMX_HANDLETYPE hComponent,
OMX_IN  OMX_INDEXTYPE nParamIndex,
OMX_IN  OMX_PTR ComponentParameterStructure) {

	OMX_ERRORTYPE eError = OMX_ErrorNone;
	OMX_U32 portIndex;

	/* Check which structure we are being fed and make control its header */
	OMX_COMPONENTTYPE *openmaxStandComp = hComponent;
	omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private = openmaxStandComp->pComponentPrivate;
	omx_base_video_PortType *port;

	if (ComponentParameterStructure == NULL) {
		return OMX_ErrorBadParameter;
	}

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "[OMX_WFDVDEC] Setting parameter %i\n", nParamIndex);
	switch(nParamIndex)
	{
		case OMX_IndexParamPortDefinition:
        {
            eError = omx_base_component_SetParameter(hComponent, nParamIndex, ComponentParameterStructure);
            if(eError == OMX_ErrorNone) 
            {
                OMX_PARAM_PORTDEFINITIONTYPE *pPortDef = (OMX_PARAM_PORTDEFINITIONTYPE*)ComponentParameterStructure;
                UpdateFrameSize (openmaxStandComp);
                portIndex = pPortDef->nPortIndex;
                port = (omx_base_video_PortType *)omx_wfdvdec_component_Private->ports[portIndex];
                port->sVideoParam.eColorFormat = port->sPortParam.format.video.eColorFormat;
            }
            break;
        }
		case OMX_IndexParamVideoPortFormat:
		{
			OMX_VIDEO_PARAM_PORTFORMATTYPE *pVideoPortFormat;
			pVideoPortFormat = ComponentParameterStructure;
			portIndex = pVideoPortFormat->nPortIndex;
			/*Check Structure Header and verify component state*/
			eError = omx_base_component_ParameterSanityCheck(hComponent, portIndex, pVideoPortFormat, sizeof(OMX_VIDEO_PARAM_PORTFORMATTYPE));
			if(eError!=OMX_ErrorNone)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Parameter Check Error=%x\n",__func__,eError);
				break;
			}
			if (portIndex <= 1)
			{
				port = (omx_base_video_PortType *)omx_wfdvdec_component_Private->ports[portIndex];
				memcpy(&port->sVideoParam, pVideoPortFormat, sizeof(OMX_VIDEO_PARAM_PORTFORMATTYPE));
				omx_wfdvdec_component_Private->ports[portIndex]->sPortParam.format.video.eColorFormat = port->sVideoParam.eColorFormat;

				if (portIndex == 1)
				{
					switch(port->sVideoParam.eColorFormat)
					{
						case OMX_COLOR_Format24bitRGB888 :
							omx_wfdvdec_component_Private->eOutFramePixFmt = PIX_FMT_RGB24;
							break;
						case OMX_COLOR_Format24bitBGR888 :
							omx_wfdvdec_component_Private->eOutFramePixFmt = PIX_FMT_BGR24;
							break;
						case OMX_COLOR_Format32bitBGRA8888 :
							omx_wfdvdec_component_Private->eOutFramePixFmt = PIX_FMT_BGR32;
							break;
						case OMX_COLOR_Format32bitARGB8888 :
							omx_wfdvdec_component_Private->eOutFramePixFmt = PIX_FMT_RGB32;
							break;
						case OMX_COLOR_Format16bitARGB1555 :
							omx_wfdvdec_component_Private->eOutFramePixFmt = PIX_FMT_RGB555;
							break;
						case OMX_COLOR_Format16bitRGB565 :
							omx_wfdvdec_component_Private->eOutFramePixFmt = PIX_FMT_RGB565;
							break;
						case OMX_COLOR_Format16bitBGR565 :
							omx_wfdvdec_component_Private->eOutFramePixFmt = PIX_FMT_BGR565;
						break;
						default:
							omx_wfdvdec_component_Private->eOutFramePixFmt = PIX_FMT_YUV420P;
						break;
					}
					UpdateFrameSize (openmaxStandComp);
				}
			}
			else
			{

				return OMX_ErrorBadPortIndex;
			}
			break;
		}
		case OMX_IndexParamStandardComponentRole:
		{
			OMX_PARAM_COMPONENTROLETYPE *pComponentRole;
			pComponentRole = ComponentParameterStructure;
			if (!strcmp((char *)pComponentRole->cRole, VIDEO_DEC_MPEG4_ROLE)) {
				omx_wfdvdec_component_Private->video_coding_type = OMX_VIDEO_CodingMPEG4;
			} else {
				return OMX_ErrorBadParameter;
			}
			SetInternalVideoParameters(openmaxStandComp);
			break;
		}
		case OMX_IndexVendorParamCDKDMXInfo:
		{
			memcpy(&omx_wfdvdec_component_Private->sCDmxInfo,ComponentParameterStructure,sizeof(cdmx_info_t));
			break;
		}

		case OMX_IndexVendorParamCDKContainerType:
		{
			unsigned int* pContainer_type;
	  		pContainer_type = (OMX_AUDIO_CONFIG_INFOTYPE*) ComponentParameterStructure;
			omx_wfdvdec_component_Private->container_type =  *pContainer_type;//info->codec_type;
			break;
		}

		default: /*Call the base component function*/
			return omx_base_component_SetParameter(hComponent, nParamIndex, ComponentParameterStructure);
	}
	return eError;
}

OMX_ERRORTYPE omx_wfdvdec_component_GetParameter(
	OMX_IN  OMX_HANDLETYPE hComponent,
	OMX_IN  OMX_INDEXTYPE nParamIndex,
	OMX_INOUT OMX_PTR ComponentParameterStructure) {

	omx_base_video_PortType *port;
	OMX_ERRORTYPE eError = OMX_ErrorNone;

	OMX_COMPONENTTYPE *openmaxStandComp = hComponent;
	omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private = openmaxStandComp->pComponentPrivate;
	if (ComponentParameterStructure == NULL)
	{
		return OMX_ErrorBadParameter;
	}
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "   Getting parameter %i\n", nParamIndex);
	/* Check which structure we are being fed and fill its header */
	switch(nParamIndex)
	{
		case OMX_IndexParamVideoInit:
			if ((eError = checkHeader(ComponentParameterStructure, sizeof(OMX_PORT_PARAM_TYPE))) != OMX_ErrorNone) {
			break;
			}
			memcpy(ComponentParameterStructure, &omx_wfdvdec_component_Private->sPortTypesParam[OMX_PortDomainVideo], sizeof(OMX_PORT_PARAM_TYPE));
			break;
		case OMX_IndexParamVideoPortFormat:
		{
			OMX_VIDEO_PARAM_PORTFORMATTYPE *pVideoPortFormat;
			pVideoPortFormat = ComponentParameterStructure;
			if ((eError = checkHeader(ComponentParameterStructure, sizeof(OMX_VIDEO_PARAM_PORTFORMATTYPE))) != OMX_ErrorNone) {
				break;
			}
			if (pVideoPortFormat->nPortIndex <= 1) {
				port = (omx_base_video_PortType *)omx_wfdvdec_component_Private->ports[pVideoPortFormat->nPortIndex];
				memcpy(pVideoPortFormat, &port->sVideoParam, sizeof(OMX_VIDEO_PARAM_PORTFORMATTYPE));
			} else {
				return OMX_ErrorBadPortIndex;
			}
			break;
		}
		case OMX_IndexParamStandardComponentRole:
		{
			OMX_PARAM_COMPONENTROLETYPE * pComponentRole;
			pComponentRole = ComponentParameterStructure;
			if ((eError = checkHeader(ComponentParameterStructure, sizeof(OMX_PARAM_COMPONENTROLETYPE))) != OMX_ErrorNone) {
				break;
			}
			if (omx_wfdvdec_component_Private->video_coding_type == OMX_VIDEO_CodingMPEG4) {
				strcpy((char *)pComponentRole->cRole, VIDEO_DEC_MPEG4_ROLE);
			} else {
				strcpy((char *)pComponentRole->cRole,"\0");
			}
			break;
		}
#ifdef TCC_ANDROID_INCLUDE
		case PV_OMX_COMPONENT_CAPABILITY_TYPE_INDEX:
		{
			PV_OMXComponentCapabilityFlagsType *pCap_flags =
			(PV_OMXComponentCapabilityFlagsType *) ComponentParameterStructure;
			if (NULL == pCap_flags)
			{
				return OMX_ErrorBadParameter;
			}

			memset(pCap_flags, 0, sizeof(PV_OMXComponentCapabilityFlagsType));
			pCap_flags->iIsOMXComponentMultiThreaded = OMX_TRUE;
			pCap_flags->iOMXComponentSupportsExternalInputBufferAlloc = OMX_TRUE;
#if USE_EXTERNAL_BUFFER
			pCap_flags->iOMXComponentSupportsExternalOutputBufferAlloc = OMX_TRUE;
#else
			pCap_flags->iOMXComponentSupportsExternalOutputBufferAlloc = OMX_FALSE;
#endif
			pCap_flags->iOMXComponentSupportsPartialFrames = OMX_FALSE;
			pCap_flags->iOMXComponentNeedsNALStartCode = OMX_FALSE;
			pCap_flags->iOMXComponentCanHandleIncompleteFrames = OMX_FALSE;
			break;
		}
#endif

		default: /*Call the base component function*/
			return omx_base_component_GetParameter(hComponent, nParamIndex, ComponentParameterStructure);
	}
	return OMX_ErrorNone;
}

OMX_ERRORTYPE omx_wfdvdec_component_MessageHandler(OMX_COMPONENTTYPE* openmaxStandComp,internalRequestMessageType *message) {
	omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private = (omx_wfdvdec_component_PrivateType*)openmaxStandComp->pComponentPrivate;
	OMX_ERRORTYPE err;
	OMX_STATETYPE eCurrentState = omx_wfdvdec_component_Private->state;


	if (message->messageType == OMX_CommandStateSet)
	{
		if ((message->messageParam == OMX_StateExecuting ) && (omx_wfdvdec_component_Private->state == OMX_StateIdle))
		{
            if (!omx_wfdvdec_component_Private->avcodecReady)
            {
                err = omx_wfdvdec_component_LibInit(omx_wfdvdec_component_Private);
                if (err != OMX_ErrorNone)
                {
                    return OMX_ErrorNotReady;
                }
                omx_wfdvdec_component_Private->avcodecReady = OMX_TRUE;
                omx_wfdvdec_component_Private->is_vdecinit = OMX_TRUE;
            }       
		}
		else if ((message->messageParam == OMX_StateIdle ) && (omx_wfdvdec_component_Private->state == OMX_StateLoaded))
		{
			err = omx_wfdvdec_component_Initialize(openmaxStandComp);
			if(err!=OMX_ErrorNone) 
            {
    			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Video Decoder Init Failed Error=%x\n",__func__,err);
    			return err;
			}
		}
		else if ((message->messageParam == OMX_StateLoaded) && (omx_wfdvdec_component_Private->state == OMX_StateIdle))
		{
			if (omx_wfdvdec_component_Private->avcodecReady)
			{

    		}

			err = omx_wfdvdec_component_Deinit(openmaxStandComp);
			if(err!=OMX_ErrorNone)
			{
				TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s Video Decoder Deinit Failed Error=%x\n",__func__,err);
				return err;
			}
		}
		else if ((message->messageParam == OMX_StateIdle ) && (omx_wfdvdec_component_Private->state == OMX_StateExecuting))
		{			

		}
	}
	// Execute the base message handling
	err =  omx_base_component_MessageHandler(openmaxStandComp,message);
	return err;
}
OMX_ERRORTYPE omx_wfdvdec_component_ComponentRoleEnum(
  OMX_IN OMX_HANDLETYPE hComponent,
  OMX_OUT OMX_U8 *cRole,
  OMX_IN OMX_U32 nIndex) {

  if (nIndex == 0) {
    strcpy((char *)cRole, VIDEO_DEC_MPEG4_ROLE);
  } else{
    return OMX_ErrorUnsupportedIndex;
  }
  return OMX_ErrorNone;
}

OMX_ERRORTYPE omx_wfdvdec_component_SetConfig(
  OMX_HANDLETYPE hComponent,
  OMX_INDEXTYPE nIndex,
  OMX_PTR pComponentConfigStructure) {

	OMX_ERRORTYPE err = OMX_ErrorNone;
	OMX_VENDOR_EXTRADATATYPE* pExtradata;

	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *)hComponent;
	omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private = (omx_wfdvdec_component_PrivateType*)openmaxStandComp->pComponentPrivate;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "   Getting configuration %i\n", nIndex);
	/* Check which structure we are being fed and fill its header */
	switch (nIndex) {
		case OMX_IndexVendorVideoExtraData :
			pExtradata = (OMX_VENDOR_EXTRADATATYPE*)pComponentConfigStructure;
			if (pExtradata->nPortIndex <= 1)
			{
				/** copy the extradata in the codec context private structure */
				omx_wfdvdec_component_Private->extradata_size = (OMX_U32)pExtradata->nDataSize;
				if(omx_wfdvdec_component_Private->extradata_size > 0)
				{
					if(omx_wfdvdec_component_Private->extradata) {
						TCC_free(omx_wfdvdec_component_Private->extradata);
					}
					omx_wfdvdec_component_Private->extradata = (unsigned char *)TCC_malloc((int)pExtradata->nDataSize*sizeof(char));
					// memcpy(omx_wfdvdec_component_Private->extradata,(unsigned char*)(pExtradata->pData),pExtradata->nDataSize);
				}
				else
				{
						TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ,"extradata size is 0 !!!\n");
				}
			}
			else
			{
				return OMX_ErrorBadPortIndex;
			}
		break;
#ifdef USE_TEE_SECURE_MEMORY
		case OMX_IndexVendorSetUSESecuredMemoryForTEE:
		{
			omx_wfdvdec_component_Private->use_secured_memory  = *(int *)pComponentConfigStructure;
		}
		break;
#endif
		case OMX_IndexVendorSetSecureVideoPathMutex:
		{
			omx_wfdvdec_component_Private->secure_video_path_mutex = (pthread_mutex_t *)pComponentConfigStructure;
		}
        break;

        case OMX_IndexVendorSetPlayerDebugMode:
            TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Enable Player Debug Msg. \n");
            omx_wfdvdec_component_Private->Display_debug_Msg = OMX_TRUE;
        break;

		case OMX_IncdxVendorConfigSetVideoSkip:
		{
			int video_skip = *(int *)pComponentConfigStructure;
			omx_wfdvdec_component_Private->isVideoSkip = video_skip;
		}
		break;

		case OMX_IndexVendorConfigDisplayIFrameNum:
		{
			omx_wfdvdec_component_Private->dispIFrameNum = *(OMX_U32 *)pComponentConfigStructure;
			omx_wfdvdec_component_Private->skipIFrameCnt = 0;
			//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"[OMX_WFDVDEC]SetConfig Frame Disp Iframe Num :%d \n ",omx_wfdvdec_component_Private->dispIFrameNum);
		}
		break;

		case OMX_IncdxVendorConfigSetIframeSearch:
		{
			omx_wfdvdec_component_Private->Iframe_Search_Enable = OMX_TRUE;
		}
		break;


		case OMX_IndexVendorConfigPlaySpeed:
		{
			float play_speed;
			play_speed = *(float *)pComponentConfigStructure;

			if(play_speed != 0)
			{
				omx_wfdvdec_component_Private->play_speed = play_speed;

				if( !(0<play_speed && play_speed < 2))
				{
					//omx_wfdvdec_component_Private->dispIFrameNum  = (OMX_U32)ABS(play_speed);
				}
				else if(0<play_speed && play_speed < 2)
				{

					omx_wfdvdec_component_Private->dispIFrameNum  = 0;
				}
			}
			//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"[OMX_WFDVDEC]SetConfig playspeed :%f  dispIFrameNum:%d \n ",omx_wfdvdec_component_Private->play_speed,omx_wfdvdec_component_Private->dispIFrameNum);
		}
		break;

		case OMX_IndexVendorConfigFastStop:
		{
			OMX_AUDIO_CONFIG_INFOTYPE *info;
			info = (OMX_AUDIO_CONFIG_INFOTYPE*)pComponentConfigStructure;
			omx_wfdvdec_component_Private->fast_stop = info->faststop;
		}
		break;

		case OMX_IndexVendorConfigSendIFrameSearchEvent:
		{
			omx_wfdvdec_component_Private->sendIFrameSearchEvt =  *(OMX_U32*)pComponentConfigStructure;;

		}
		break;
		case OMX_IndexVendorConfigDiscardFrame:
		{
			omx_wfdvdec_component_Private->doDiscardFrame = *(OMX_U32*)pComponentConfigStructure;
			//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"[OMX_WFDVDEC]OMX_IndexVendorConfigDiscardFrame :%d  \n",omx_wfdvdec_component_Private->doDiscardFrame);
		}
		break;

		case OMX_IndexVendorConfigClockBufferClear:
		{
			omx_base_video_PortType *pVideoPort;
			pVideoPort = (omx_base_video_PortType *) omx_wfdvdec_component_Private->ports[0];
			OMX_BUFFERHEADERTYPE *videoBuffer;

			//if(pVideoPort->pBufferSem->semval > 0)
			{
				pthread_mutex_lock(&omx_wfdvdec_component_Private->video_stop_mutex);
				pthread_mutex_unlock(&omx_wfdvdec_component_Private->video_stop_mutex);
			}
		}
		break;

		case OMX_IndexVenderConfigWeightedPrediction:
			omx_wfdvdec_component_Private->vdecAvcWeightedPredBskipFlag = *(OMX_U32 *)pComponentConfigStructure;
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"[CDKVDEC] SET CONFIG WP Flag: %d \n", omx_wfdvdec_component_Private->vdecAvcWeightedPredBskipFlag);
			break;

		case OMX_IndexVendorConfigGetDecodingInfo:
			omx_wfdvdec_component_Private->isGetDecodingInfo = 1 - omx_wfdvdec_component_Private->isGetDecodingInfo;
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"[CDKVDEC] SET CONFIG GetDecodingInfo: %d \n", omx_wfdvdec_component_Private->isGetDecodingInfo);
			break;

		default: // delegate to superclass
		return omx_base_component_SetConfig(hComponent, nIndex, pComponentConfigStructure);
	}
	return err;
}

/** getting configurations */
OMX_ERRORTYPE omx_wfdvdec_component_GetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_IN  OMX_PTR pComponentConfigStructure) {

	OMX_VENDOR_EXTRADATATYPE sExtraData;
	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *)hComponent;
	omx_wfdvdec_component_PrivateType* omx_wfdvdec_component_Private = openmaxStandComp->pComponentPrivate;

	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME,"In  %s OMX_IndexVendorVideoExtraData\n",__func__);
	switch (nIndex)
	{
		// daniel add
		case OMX_IndexVendorChangeResolution:
		{
			OMX_VIDEO_PORTDEFINITIONTYPE *p_video_port_param = pComponentConfigStructure;
			p_video_port_param->VideoCropInfo.nTop = omx_wfdvdec_component_Private->sVDecInfo.m_iAvcPicCrop.m_iCropTop;
			p_video_port_param->VideoCropInfo.nBottom = omx_wfdvdec_component_Private->sVDecInfo.m_iAvcPicCrop.m_iCropBottom;
			p_video_port_param->VideoCropInfo.nLeft = omx_wfdvdec_component_Private->sVDecInfo.m_iAvcPicCrop.m_iCropLeft;
			p_video_port_param->VideoCropInfo.nRight = omx_wfdvdec_component_Private->sVDecInfo.m_iAvcPicCrop.m_iCropRight;
			p_video_port_param->nFrameWidth = omx_wfdvdec_component_Private->sVDecInfo.m_iPicWidth;
			p_video_port_param->nFrameHeight = omx_wfdvdec_component_Private->sVDecInfo.m_iPicHeight;
		}
		break;

		case OMX_IndexVendorConfigDisplayIFrameNum:
		{
			*(OMX_U32 *)pComponentConfigStructure = omx_wfdvdec_component_Private->dispIFrameNum;
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"[OMX_WFDVDEC]GetConfig Frame Skip Num :%d \n ",*(OMX_U32 *)pComponentConfigStructure);
		}
		break;

		case OMX_IndexVendorConfigCurrentKeyFramePTS:
		{
			*(OMX_TICKS *)pComponentConfigStructure = omx_wfdvdec_component_Private->currKeyFramePTS;
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"[OMX_WFDVDEC]GetConfig Current Key Frame PTS :%d \n ",*(OMX_U32 *)pComponentConfigStructure);
		}
		break;
		default: // delegate to superclass
		return omx_base_component_GetConfig(hComponent, nIndex, pComponentConfigStructure);

	}
	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME,"In  %s OMX_ErrorNone\n",__func__);
	return OMX_ErrorNone;
}


OMX_ERRORTYPE omx_wfdvdec_component_GetExtensionIndex(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_STRING cParameterName,
  OMX_OUT OMX_INDEXTYPE* pIndexType) {

  TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FUNC_NAME,"In  %s \n",__func__);

  if(strcmp(cParameterName,"OMX.tcc.index.config.videoextradata") == 0) {
    *pIndexType = OMX_IndexVendorVideoExtraData;
  } else {
    return OMX_ErrorBadParameter;
  }
  return OMX_ErrorNone;
}

/** This is the central function for buffer processing.
  * It is executed in a separate thread.
  * @param param input parameter, a pointer to the OMX standard structure
  */
static void* omx_wfdvdec_component_BufferMgmtFunction (void* param)
{
	int inBufExchanged = 0;
	int fbOutBufExchanged = 0;

	OMX_BOOL isInputBufferNeeded = OMX_TRUE;
	OMX_BOOL isFBOutputBufferNeeded = OMX_TRUE;

	OMX_COMPONENTTYPE *openmaxStandComp = (OMX_COMPONENTTYPE *)param;

	omx_base_component_PrivateType *omx_base_component_Private = (omx_base_component_PrivateType *)openmaxStandComp->pComponentPrivate;
	omx_base_filter_PrivateType *omx_base_filter_Private = (omx_base_filter_PrivateType *)omx_base_component_Private;

	omx_base_PortType *pInPort = (omx_base_PortType *)omx_base_filter_Private->ports[VIDEO_DEC_INPUT_PORT_INDEX];
	omx_base_PortType *pFBOutPort = (omx_base_PortType *)omx_base_filter_Private->ports[VIDEO_DEC_FB_OUTPUT_PORT_INDEX];

	tsem_t *pInputSem = pInPort->pBufferSem;
	tsem_t *pFBOutputSem = pFBOutPort->pBufferSem;

	queue_t *pInputQueue = pInPort->pBufferQueue;
	queue_t *pFBOutputQueue = pFBOutPort->pBufferQueue;

	OMX_BUFFERHEADERTYPE *pInputBuffer = NULL;
	OMX_BUFFERHEADERTYPE *pFBOutputBuffer = NULL;

#if (TCC_DEBUG_GET_TID == 1)
    TCC_DEBUG_M(TCC_DEBUG_COMPONENT, TCC_DEBUG_LEV_FUNC_NAME, "In %s\n", __func__);
    TCC_DEBUG_M(TCC_DEBUG_COMPONENT, TCC_DEBUG_LEV_INFO, "[%s]->[%s] TID(%d) \n", COMPONENT_NAME(openmaxStandComp), __func__, syscall(__NR_gettid));
#endif

	while(omx_base_filter_Private->state == OMX_StateIdle
		|| omx_base_filter_Private->state == OMX_StateExecuting
		|| omx_base_filter_Private->state == OMX_StatePause
		|| omx_base_filter_Private->transientState == OMX_TransStateLoadedToIdle
		|| omx_base_filter_Private->transientState == OMX_TransStateExecutingToIdle)
	{
		/*Wait till the ports are being flushed*/
		pthread_mutex_lock(&omx_base_filter_Private->flush_mutex);
		while	(PORT_IS_BEING_FLUSHED(pInPort) || PORT_IS_BEING_FLUSHED(pFBOutPort))
		{
			pthread_mutex_unlock(&omx_base_filter_Private->flush_mutex);

			DEBUG(DEB_LEV_FULL_SEQ, "In %s 1 signalling flush all cond iE=%d,iF=%d,oE=%d,oF=%d iSemVal=%d,oSemval=%d\n",
				__func__, inBufExchanged, isInputBufferNeeded, fbOutBufExchanged, isFBOutputBufferNeeded, pInputSem->semval, pFBOutputSem->semval);

			if(isFBOutputBufferNeeded == OMX_FALSE && PORT_IS_BEING_FLUSHED(pFBOutPort))
			{
				pFBOutPort->ReturnBufferFunction(pFBOutPort, pFBOutputBuffer);
				fbOutBufExchanged--;
				pFBOutputBuffer = NULL;
				isFBOutputBufferNeeded = OMX_TRUE;
				DEBUG(DEB_LEV_FULL_SEQ, "Ports are flushing,so returning fb output buffer\n");
			}

			if(isInputBufferNeeded == OMX_FALSE && PORT_IS_BEING_FLUSHED(pInPort))
			{
				pInPort->ReturnBufferFunction(pInPort, pInputBuffer);
				inBufExchanged--;
				pInputBuffer = NULL;
				isInputBufferNeeded = OMX_TRUE;
				DEBUG(DEB_LEV_FULL_SEQ, "Ports are flushing,so returning input buffer\n");
			}

			DEBUG(DEB_LEV_FULL_SEQ, "In %s 2 signalling flush all cond iE=%d,iF=%d,oE=%d,oF=%d iSemVal=%d,oSemval=%d\n",
				__func__, inBufExchanged, isInputBufferNeeded, fbOutBufExchanged, isFBOutputBufferNeeded, pInputSem->semval, pFBOutputSem->semval);

			tsem_up(omx_base_filter_Private->flush_all_condition);
			tsem_down(omx_base_filter_Private->flush_condition);
			pthread_mutex_lock(&omx_base_filter_Private->flush_mutex);
		}
		pthread_mutex_unlock(&omx_base_filter_Private->flush_mutex);

		/*No buffer to process. So wait here*/
		if((isInputBufferNeeded == OMX_TRUE && pInputSem->semval == 0)
			&& (omx_base_filter_Private->state != OMX_StateLoaded
			&& omx_base_filter_Private->state != OMX_StateInvalid))
		{
			//Signalled from EmptyThisBuffer or FillThisBuffer or some thing else
			DEBUG(DEB_LEV_FULL_SEQ, "Waiting for next input/output buffer\n");
			tsem_down(omx_base_filter_Private->bMgmtSem);

		}

		if(omx_base_filter_Private->state == OMX_StateLoaded || omx_base_filter_Private->state == OMX_StateInvalid)
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "In %s Buffer Management Thread is exiting\n",__func__);
			break;
		}

		if (( (isFBOutputBufferNeeded == OMX_TRUE && pFBOutputSem->semval == 0))
			&& (omx_base_filter_Private->state != OMX_StateLoaded
			&& omx_base_filter_Private->state != OMX_StateInvalid)
			&& !(PORT_IS_BEING_FLUSHED(pInPort) || PORT_IS_BEING_FLUSHED(pFBOutPort)))
		{
			//Signalled from EmptyThisBuffer or FillThisBuffer or some thing else
			DEBUG(DEB_LEV_FULL_SEQ, "Waiting for next input/output buffer\n");
			tsem_down(omx_base_filter_Private->bMgmtSem);
		}

		if(omx_base_filter_Private->state == OMX_StateLoaded || omx_base_filter_Private->state == OMX_StateInvalid)
		{
			TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "In %s Buffer Management Thread is exiting\n",__func__);
			break;
		}

		TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ, "Waiting for input buffer semval=%d \n", pInputSem->semval);

		if(pInputSem->semval > 0 && isInputBufferNeeded == OMX_TRUE)
		{
			tsem_down(pInputSem);
			if(pInputQueue->nelem > 0)
			{
				inBufExchanged++;
				isInputBufferNeeded = OMX_FALSE;
				pInputBuffer = dequeue(pInputQueue);
				if(pInputBuffer == NULL)
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Had NULL input buffer!!\n");
					break;
				}

				if(checkHeader(pInputBuffer, sizeof(OMX_BUFFERHEADERTYPE)))
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "In %s, crashed input buffer!!\n" ,__func__);
					isInputBufferNeeded = OMX_TRUE;
					inBufExchanged--;
					pInputBuffer = NULL;
				}
			}
		}

		/*When we have input buffer to process then get one output buffer*/
		if(pFBOutputSem->semval > 0 && isFBOutputBufferNeeded == OMX_TRUE)
		{
			tsem_down(pFBOutputSem);
			if(pFBOutputQueue->nelem > 0)
			{
				fbOutBufExchanged++;
				isFBOutputBufferNeeded = OMX_FALSE;
				pFBOutputBuffer = dequeue(pFBOutputQueue);
				if(pFBOutputBuffer == NULL)
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "Had NULL fb output buffer!! op is=%d,iq=%d\n",pFBOutputSem->semval,pFBOutputQueue->nelem);
					break;
				}

				if(checkHeader(pFBOutputBuffer, sizeof(OMX_BUFFERHEADERTYPE)))
				{
					TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_ERR, "crashed fb output buffer!!\n");
					isFBOutputBufferNeeded = OMX_TRUE;
					fbOutBufExchanged--;
					pFBOutputBuffer = NULL;
				}
			}
		}

		if (isInputBufferNeeded == OMX_FALSE)
		{
			if(pInputBuffer->hMarkTargetComponent != NULL)
			{
				if((OMX_COMPONENTTYPE*)pInputBuffer->hMarkTargetComponent ==(OMX_COMPONENTTYPE *)openmaxStandComp)
				{
					/*Clear the mark and generate an event*/
					(*(omx_base_filter_Private->callbacks->EventHandler))(
						openmaxStandComp,
						omx_base_filter_Private->callbackData,
						OMX_EventMark, /* The command was completed */
						1, /* The commands was a OMX_CommandStateSet */
						0, /* The state has been changed in message->messageParam2 */
						pInputBuffer->pMarkData);
				}
				else
				{
					/*If this is not the target component then pass the mark*/
					omx_base_filter_Private->pMark.hMarkTargetComponent = pInputBuffer->hMarkTargetComponent;
					omx_base_filter_Private->pMark.pMarkData            = pInputBuffer->pMarkData;
				}
				pInputBuffer->hMarkTargetComponent = NULL;
			}
		}

		if (isInputBufferNeeded == OMX_FALSE && isFBOutputBufferNeeded == OMX_FALSE)
		{
			if(omx_base_filter_Private->pMark.hMarkTargetComponent != NULL)
			{
				pFBOutputBuffer->hMarkTargetComponent = omx_base_filter_Private->pMark.hMarkTargetComponent;
				pFBOutputBuffer->pMarkData            = omx_base_filter_Private->pMark.pMarkData;
				omx_base_filter_Private->pMark.hMarkTargetComponent = NULL;
				omx_base_filter_Private->pMark.pMarkData            = NULL;
			}

			pFBOutputBuffer->nTimeStamp = pInputBuffer->nTimeStamp;
			if(omx_base_filter_Private->BufferMgmtCallback && pInputBuffer->nFilledLen > 0)
			{
				if(omx_base_filter_Private->state == OMX_StateExecuting)
				{
					(*(omx_base_filter_Private->BufferMgmtCallback))(openmaxStandComp, pInputBuffer, pFBOutputBuffer);
				}
				else
				{
					pInputBuffer->nFilledLen = 0;
				}
			}
			else
			{
				/*It no buffer management call back the explicitly consume input buffer*/
				pInputBuffer->nFilledLen = 0;
			}

			if(pInputBuffer->nFlags)
			{
				pFBOutputBuffer->nFlags |= pInputBuffer->nFlags;
				pInputBuffer->nFlags = 0;
			}

			if(((pFBOutputBuffer->nFlags & OMX_BUFFERFLAG_EOS) == 1)
				&& pInputBuffer->nFilledLen==0
				&& omx_base_filter_Private->bIsEOSReached == OMX_FALSE)
			{
				//TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_FULL_SEQ,"OMX_BUFFERFLAG_EOS\n");
				//pOutputBuffer->nFlags = pInputBuffer->nFlags;
				pFBOutputBuffer->nFilledLen = 0;
				pFBOutputBuffer->ErrorType = pInputBuffer->ErrorType;
				pInputBuffer->ErrorType = 0;
				//pInputBuffer->nFlags = 0;
				(*(omx_base_filter_Private->callbacks->EventHandler))(
					openmaxStandComp,
					omx_base_filter_Private->callbackData,
					OMX_EventBufferFlag, /* The command was completed */
					1, /* The commands was a OMX_CommandStateSet */
					pFBOutputBuffer->nFlags, /* The state has been changed in message->messageParam2 */
					NULL);
				omx_base_filter_Private->bIsEOSReached = OMX_TRUE;
			}

			if(omx_base_filter_Private->state == OMX_StatePause
				&& !(PORT_IS_BEING_FLUSHED(pInPort)
				|| PORT_IS_BEING_FLUSHED(pFBOutPort)))
			{
				/*Waiting at paused state*/
				tsem_wait(omx_base_component_Private->bStateSem);
			}

			/*If EOS and Input buffer Filled Len Zero then Return output buffer immediately*/
			if((pFBOutputBuffer->nFilledLen != 0)
				|| ((pFBOutputBuffer->nFlags & OMX_BUFFERFLAG_EOS) == 1)
				|| (omx_base_filter_Private->bIsEOSReached == OMX_TRUE))
			{
				pFBOutPort->ReturnBufferFunction(pFBOutPort, pFBOutputBuffer);
				fbOutBufExchanged--;
				pFBOutputBuffer = NULL;
				isFBOutputBufferNeeded = OMX_TRUE;
			}
		}

		if(omx_base_filter_Private->state == OMX_StatePause
			&& !(PORT_IS_BEING_FLUSHED(pInPort)
			|| PORT_IS_BEING_FLUSHED(pFBOutPort)))
		{
			/*Waiting at paused state*/
			tsem_wait(omx_base_component_Private->bStateSem);
		}

		/*Input Buffer has been completely consumed. So, return input buffer*/
		if((isInputBufferNeeded == OMX_FALSE) && (pInputBuffer->nFilledLen == 0))
		{
			pInPort->ReturnBufferFunction(pInPort, pInputBuffer);
			inBufExchanged--;
			pInputBuffer = NULL;
			isInputBufferNeeded = OMX_TRUE;
		}
	}

  	TCC_DEBUG_M(TCC_DEBUG_COMPONENT,TCC_DEBUG_LEV_SIMPLE_SEQ,"Exiting Buffer Management Thread\n");

  	return NULL;
}

