/*
  @file component/openmax_wfdvdec_component/omx_wfdvdec_component.h

  OpenMAX Video decoder component. This component implements H.264 video decoder.

  Copyright (C) 2008  STMicroelectronics
  Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
  Copyright (C) 2015 Telechips Inc.

  This library is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the Free
  Software Foundation; either version 2.1 of the License, or (at your option)
  any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St, Fifth Floor, Boston, MA
  02110-1301  USA

  $Date: 2009/03/10 13:33:28 $
  Revision $Rev: 554 $
  Author $Author: B060934 $
  revised by B060955.
*/

#ifndef _OMX_WFDVDEC_COMPONENT_H_
#define _OMX_WFDVDEC_COMPONENT_H_

#include <OMX_Types.h>
#include <OMX_Component.h>
#include <OMX_Core.h>
#include <OMX_Video.h>
#include <OMX_TCC_Index.h>
#include <pthread.h>
#include <stdlib.h>
#include <string.h>
#include <omx_base_filter.h>

#include <string.h>
#include "tcc_memory.h"

#include <tcc_video_common.h>

#include "tcc_virtual_mem.h"
#include <cdk_core.h>
#include "pmap.h"
#include <str_utils.h>

//#define TCC_VDEC_DEBUG_INCLUDE
#define DISPLAY_QUEUE_MAX_SIZE			(16) // 1<<4
typedef struct
{
	int gs_nDisplayQueue[DISPLAY_QUEUE_MAX_SIZE];
	int gs_nQueueFrontIndex; //pop index
	int gs_nQueueRearIndex; //push index
	int gs_nQueueNumOfIndex; //num of index in queue
}display_index_queue_t;

/** Video Decoder component private structure.
  */
/**
 * Pixel format. Notes:
 *
 * PIX_FMT_RGB32 is handled in an endian-specific manner. A RGBA
 * color is put together as:
 *  (A << 24) | (R << 16) | (G << 8) | B
 * This is stored as BGRA on little endian CPU architectures and ARGB on
 * big endian CPUs.
 *
 * When the pixel format is palettized RGB (PIX_FMT_PAL8), the palettized
 * image data is stored in AVFrame.data[0]. The palette is transported in
 * AVFrame.data[1] and, is 1024 bytes long (256 4-byte entries) and is
 * formatted the same as in PIX_FMT_RGB32 described above (i.e., it is
 * also endian-specific). Note also that the individual RGB palette
 * components stored in AVFrame.data[1] should be in the range 0..255.
 * This is important as many custom PAL8 video codecs that were designed
 * to run on the IBM VGA graphics adapter use 6-bit palette components.
 */
DERIVEDCLASS(omx_wfdvdec_component_PrivateType, omx_base_filter_PrivateType)
#define omx_wfdvdec_component_PrivateType_FIELDS omx_base_filter_PrivateType_FIELDS \
  /** @param semaphore for avcodec access syncrhonization */\
  tsem_t* avCodecSyncSem; \
  /** @param avcodecReady boolean flag that is true when the video coded has been initialized */ \
  OMX_BOOL avcodecReady;  \
  /** @param minBufferLength Field that stores the minimun allowed size for FFmpeg decoder */ \
  OMX_U16 minBufferLength; \
  /** @param inputCurrBuffer Field that stores pointer of the current input buffer position */ \
  OMX_U8* inputCurrBuffer;\
  /** @param inputCurrLength Field that stores current input buffer length in bytes */ \
  OMX_U32 inputCurrLength;\
  /** @param isFirstBuffer Field that the buffer is the first buffer */ \
  OMX_S32 isFirstBuffer;\
  /** @param isNewBuffer Field that indicate a new buffer has arrived*/ \
  OMX_S32 isNewBuffer;  \
  /** @param video_coding_type Field that indicate the supported video format of video decoder */ \
  OMX_U32 video_coding_type;   \
  /** @param eOutFramePixFmt Field that indicate output frame pixel format */ \
  PIXEL_FORMAT_E eOutFramePixFmt; \
  /** @param extradata pointer to extradata*/ \
  OMX_U8* extradata; \
  /** @param extradata_size extradata size*/ \
  OMX_U32 extradata_size; \
  cdmx_info_t sCDmxInfo; \
  vdec_init_t sVDecInit; \
  vdec_info_t sVDecInfo; \
  vdec_input_t sVDecInput; \
  vdec_output_t sVDecOutput; \
  dec_disp_info_ctrl_t dec_disp_info_ctrl; \
  dec_disp_info_t	dec_disp_info[32]; \
  dec_disp_info_input_t	dec_disp_info_input; \
  display_index_queue_t index_queue; \
  OMX_U32 container_type; \
  OMX_U32 video_codec_type; \
  OMX_PTR pVDecInstance; \
  cdk_func_t* vdec_func; \
  void* vpu_handle; \
  OMX_U32 frameSearchOrSkip_flag; \
  OMX_U32 vdecAvcWeightedPredFlag ; \
  OMX_U32 vdecAvcWeightedPredBskipFlag;\
  OMX_U32 dispIFrameNum; \
  OMX_U32 skipIFrameCnt; \
  OMX_U32 currKeyFramePTS; \
  float   play_speed; \
  OMX_U32 broadcastflag; \
  OMX_U32 getOneFrameMode; \
  OMX_U32 LastKeyFrameTime; \
  OMX_U32 disablevecinit; \
  OMX_U32 is_vdecinit; \
  OMX_U32 fast_stop; \
  OMX_U32 sendIFrameSearchEvt;	\
  OMX_U32 is_sync_arrived; \
  OMX_U32 doDiscardFrame; \
  OMX_U32 isVideoSkip; \
  OMX_U32 frame_drop_count; \
  OMX_U32 Iframe_Search_Enable; \
  OMX_U32 avcinterlacedscanflag; \
  OMX_U32 isGetDecodingInfo; \
  OMX_BOOL use_secured_memory; \
  OMX_BOOL isVPUClosed; \
  OMX_BOOL Display_debug_Msg; \
  OMX_U32  current_decout_width;\
  OMX_U32  current_decout_height;\
  OMX_VIDEO_PORTDEFINITIONTYPE decout_frame_info;\
  pthread_mutex_t *secure_video_path_mutex; \
  pthread_mutex_t video_stop_mutex;
ENDCLASS(omx_wfdvdec_component_PrivateType)
/* Component private entry points declaration */
OMX_ERRORTYPE omx_wfdvdec_component_Init(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdvdec_component_Constructor(OMX_COMPONENTTYPE *openmaxStandComp,OMX_STRING cComponentName);
OMX_ERRORTYPE omx_wfdvdec_component_Destructor(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdvdec_component_Initialize(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdvdec_component_Deinit(OMX_COMPONENTTYPE *openmaxStandComp);
OMX_ERRORTYPE omx_wfdvdec_component_MessageHandler(OMX_COMPONENTTYPE*,internalRequestMessageType*);

void omx_wfdvdec_component_BufferMgmtCallback(
  OMX_COMPONENTTYPE *openmaxStandComp,
  OMX_BUFFERHEADERTYPE* inputbuffer,
  OMX_BUFFERHEADERTYPE* outputbuffer);

OMX_ERRORTYPE omx_wfdvdec_component_GetParameter(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nParamIndex,
  OMX_INOUT OMX_PTR ComponentParameterStructure);

OMX_ERRORTYPE omx_wfdvdec_component_SetParameter(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nParamIndex,
  OMX_IN  OMX_PTR ComponentParameterStructure);

OMX_ERRORTYPE omx_wfdvdec_component_ComponentRoleEnum(
  OMX_IN OMX_HANDLETYPE hComponent,
  OMX_OUT OMX_U8 *cRole,
  OMX_IN OMX_U32 nIndex);

OMX_ERRORTYPE omx_wfdvdec_component_GetConfig(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_INDEXTYPE nIndex,
  OMX_IN  OMX_PTR pComponentConfigStructure);

OMX_ERRORTYPE omx_wfdvdec_component_SetConfig(
  OMX_HANDLETYPE hComponent,
  OMX_INDEXTYPE nIndex,
  OMX_PTR pComponentConfigStructure);

OMX_ERRORTYPE omx_wfdvdec_component_GetExtensionIndex(
  OMX_IN  OMX_HANDLETYPE hComponent,
  OMX_IN  OMX_STRING cParameterName,
  OMX_OUT OMX_INDEXTYPE* pIndexType);



#endif //_OMX_WFDVDEC_COMPONENT_H_